@@ -1,11 +1,14 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 from collections import defaultdict, ChainMap
 from collections.abc import Iterable
 from abc import ABCMeta, abstractmethod
-from functools import lru_cache
-import numpy as np
+from functools import wraps, lru_cache
 
 from ._internal import set_module, singledispatchmethod
 
+
 __all__ = ["Category", "CompositeCategory", "NullCategory"]
 __all__ += ["AndCategory", "OrCategory", "XOrCategory"]
 __all__ += ["InstanceCache"]
@@ -73,7 +76,7 @@ class CategoryMeta(ABCMeta):
         If a category class is called, we will attempt to instantiate it.
 
         However, it may be that this is a parent class (e.g. `AtomCategory`)
-        that does not make sense to instantiate. Since this classes are abstract,
+        that does not make sense to instantiate. Since these classes are abstract,
         they will raise an error that we will use to build the categories that the user
         requested.
 
@@ -232,7 +235,7 @@ class Category(metaclass=CategoryMeta):
             # if neither is a compositecategory, then they cannot
             # be the same category
             return False
-        elif self.__class__ != other.__class__:
+        if self.__class__ != other.__class__:
             return False
         return self == other
 
@@ -364,7 +367,7 @@ class CompositeCategory(GenericCategory):
         self.A = A
         self.B = B
 
-    def categorize(self, *args, **kwargs):
+    def categorizeAB(self, *args, **kwargs):
         r""" Base method for queriyng whether an object is a certain category """
         catA = self.A.categorize(*args, **kwargs)
         catB = self.B.categorize(*args, **kwargs)
@@ -409,7 +412,7 @@ class OrCategory(CompositeCategory):
 
     def categorize(self, *args, **kwargs):
         r""" Base method for queriyng whether an object is a certain category """
-        catA, catB = super().categorize(*args, **kwargs)
+        catA, catB = self.categorizeAB(*args, **kwargs)
 
         def cmp(a, b):
             if isinstance(a, NullCategory):
@@ -441,12 +444,12 @@ class AndCategory(CompositeCategory):
 
     def categorize(self, *args, **kwargs):
         r""" Base method for queriyng whether an object is a certain category """
-        catA, catB = super().categorize(*args, **kwargs)
+        catA, catB = self.categorizeAB(*args, **kwargs)
 
         def cmp(a, b):
             if isinstance(a, NullCategory):
                 return a
-            elif isinstance(b, NullCategory):
+            if isinstance(b, NullCategory):
                 return b
             return self
 
@@ -475,12 +478,12 @@ class XOrCategory(CompositeCategory):
 
     def categorize(self, *args, **kwargs):
         r""" Base method for queriyng whether an object is a certain category """
-        catA, catB = super().categorize(*args, **kwargs)
+        catA, catB = self.categorizeAB(*args, **kwargs)
 
         def cmp(a, b):
             if isinstance(a, NullCategory):
                 return b
-            elif isinstance(b, NullCategory):
+            if isinstance(b, NullCategory):
                 return a
             # both are not NullCategory, in which case nothing
             # is exclusive, so we return the NullCategory
