@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """ Dispatcher classes for handling methods dispatched for wrapped function calls.
 
 This method allows classes to dispatch methods through other classes.
@@ -7,8 +10,7 @@ Here is a small snippet showing how to utilize this module.
 """
 
 from abc import ABCMeta, abstractmethod
-from functools import wraps
-from collections import namedtuple
+from collections import namedtuple, ChainMap
 
 
 __all__ = ["AbstractDispatch", "ObjectDispatcher", "MethodDispatcher",
@@ -36,6 +38,10 @@ class AbstractDispatch(metaclass=ABCMeta):
         # This could in principle contain anything.
         self._attrs = attrs
 
+    def copy(self):
+        """ Create a copy of this object """
+        return self.__class__(self._obj, **self._attrs)
+
     def renew(self, **attrs):
         """ Create a new class with updated attributes """
         return self.__class__(self._obj, **{**self._attrs, **attrs})
@@ -65,7 +71,6 @@ class AbstractDispatch(metaclass=ABCMeta):
                 return method(*args, **kwargs)
 
         """
-        pass
 
     def __getattr__(self, key):
         attr = getattr(self._obj, key)
@@ -74,7 +79,7 @@ class AbstractDispatch(metaclass=ABCMeta):
         return attr
 
 
-class AbstractDispatcher:
+class AbstractDispatcher(metaclass=ABCMeta):
     """ A container for dispatchers
 
     This is an abstract class holding the dispatch classes (`AbstractDispatch`)
@@ -84,13 +89,23 @@ class AbstractDispatcher:
 
     def __init__(self, dispatchs=None, default=None, **attrs):
         if dispatchs is None:
-            dispatchs = dict()
+            dispatchs = {}
+        if not isinstance(dispatchs, ChainMap):
+            dispatchs = ChainMap(dispatchs)
+        # we will always use a chainmap to store the dispatches
+        # We must not *copy*
+        # It should be the same memory location in case we are
+        # passing around the dispatch sequences
         self._dispatchs = dispatchs
         self._default = default
         self.__name__ = self.__class__.__name__
         # Attributes associated with the dispatcher
         self._attrs = attrs
 
+    def copy(self):
+        """ Create a copy of this object (making a new child for the dispatch lookup) """
+        return self.__class__(self._dispatchs.new_child(), self._default, **self._attrs)
+
     def renew(self, **attrs):
         """ Create a new class with updated attributes """
         return self.__class__(self._dispatchs, self._default, **{**self._attrs, **attrs})
@@ -123,6 +138,10 @@ class AbstractDispatcher:
         """
         self.register(key, dispatch)
 
+    def __dir__(self):
+        """ Return instances belonging to this object """
+        return list(self._dispatchs.keys()) + ["renew", "register"]
+
     def register(self, key, dispatch, default=False, overwrite=False):
         """ Register a dispatch class to this container
 
@@ -138,7 +157,7 @@ class AbstractDispatcher:
             if true and `key` already exists in the list of dispatchs, then
             it will be overwritten, otherwise a `LookupError` is raised.
         """
-        if key in self._dispatchs and not overwrite:
+        if key in self._dispatchs.maps[0] and not overwrite:
             raise LookupError(f"{self.__class__.__name__} already has {key} registered (and overwrite is false)")
         self._dispatchs[key] = dispatch
         if default:
@@ -154,7 +173,7 @@ class ErrorDispatcher(AbstractDispatcher):
     """
     __slots__ = ()
 
-    def __init__(self, obj, *args, **kwargs):
+    def __init__(self, obj, *args, **kwargs): # pylint: disable=W0231
         raise ValueError(f"Dispatcher on {obj} must not be called in this way, see documentation.")
 
 
@@ -177,6 +196,11 @@ class MethodDispatcher(AbstractDispatcher):
         # Storing the name is required for help on functions
         self.__name__ = method.__name__
 
+    def copy(self):
+        """ Create a copy of this object (making a new child for the dispatch lookup) """
+        return self.__class__(self._method, self._dispatchs.new_child(), self._default,
+                              self._obj, **self._attrs)
+
     def renew(self, **attrs):
         """ Create a new class with updated attributes """
         return self.__class__(self._method, self._dispatchs, self._default,
@@ -224,12 +248,21 @@ class ObjectDispatcher(AbstractDispatcher):
         self._obj_getattr = obj_getattr
         self._cls_attr_name = cls_attr_name
 
+    def copy(self):
+        """ Create a copy of this object (making a new child for the dispatch lookup) """
+        return self.__class__(self._obj, self._dispatchs.new_child(), self._default,
+                              self._cls_attr_name, self._obj_getattr,
+                              **self._attrs)
+
     def renew(self, **attrs):
         """ Create a new class with updated attributes """
         return self.__class__(self._obj, self._dispatchs, self._default,
                               self._cls_attr_name, self._obj_getattr,
                               **{**self._attrs, **attrs})
 
+    def __call__(self, **attrs):
+        return self.renew(**attrs)
+
     def __str__(self):
         obj = str(self._obj).replace("\n", "\n ")
         return super().__str__().replace("{", f"{{\n {obj},\n ", 1)
@@ -278,8 +311,8 @@ class ObjectDispatcher(AbstractDispatcher):
         attr = self._obj_getattr(self._obj, key)
         if callable(attr):
             # This will also ensure that if the user calls immediately after it will use the default
-            return MethodDispatcher(attr, dispatchs=self._dispatchs,
-                                    default=self._default, obj=self._obj, **self._attrs)
+            return MethodDispatcher(attr, self._dispatchs, self._default,
+                                    self._obj, **self._attrs)
         return attr
 
 
@@ -349,7 +382,7 @@ class TypeDispatcher(ObjectDispatcher):
 class ClassDispatcher(AbstractDispatcher):
     """ A dispatcher for classes, using `__get__` it converts into `ObjectDispatcher` upon invocation from an object, or a `TypeDispatcher` when invoked from a class
 
-    This is a class-placeholder allowing a dispatecher to be a class attribute and converted into an
+    This is a class-placeholder allowing a dispatcher to be a class attribute and converted into an
     `ObjectDispatcher` when invoked from an object.
 
     If it is called on the class, it will return a `TypeDispatcher`.
@@ -385,7 +418,7 @@ class ClassDispatcher(AbstractDispatcher):
     def __init__(self, attr_name, dispatchs=None, default=None,
                  obj_getattr=None,
                  instance_dispatcher=ObjectDispatcher,
-                 type_dispatcher=None,
+                 type_dispatcher=TypeDispatcher,
                  **attrs):
         # obj_getattr is necessary for the ObjectDispatcher to create the correct
         # MethodDispatcher
@@ -401,6 +434,13 @@ class ClassDispatcher(AbstractDispatcher):
                 return getattr(obj, key)
         self._obj_getattr = obj_getattr
 
+    def copy(self):
+        """ Create a copy of this object (making a new child for the dispatch lookup) """
+        return self.__class__(self._attr_name, self._dispatchs.new_child(), self._default,
+                              self._obj_getattr,
+                              self._get.instance, self._get.type,
+                              **self._attrs)
+
     def renew(self, **attrs):
         """ Create a new class with updated attributes """
         return self.__class__(self._attr_name, self._dispatchs, self._default,
@@ -438,7 +478,7 @@ class ClassDispatcher(AbstractDispatcher):
 
 
 '''
-For use when doing cached dispatechers
+For use when doing cached dispatchers
 class CachedClassDispatcher(ClassDispatcher):
     __slots__ = ("_obj_getattr", "_attr_name")
 
