@@ -1,4 +1,6 @@
-import sys
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import functools
 import warnings
 
@@ -27,7 +29,7 @@ try:
         defusedxml_version = list(map(int, defusedxml_version.split(".")))
         if defusedxml_version[0] == 0 and defusedxml_version[1] <= 5:
             raise ImportError
-    except:
+    except Exception:
         raise ImportError
 except ImportError:
     from xml.etree.ElementTree import parse as xml_parse
@@ -41,7 +43,7 @@ def array_fill_repeat(array, size, cls=None):
     """
     try:
         reps = size // len(array)
-    except:
+    except Exception:
         array = [array]
         reps = size // len(array)
     if size % len(array) != 0:
@@ -109,7 +111,7 @@ def isndarray(arr):
     return isinstance(arr, _ndarray)
 
 
-def get_dtype(var, int=None, other=None):
+def get_dtype(var, int=None, other=None): # pylint: disable=W0622
     """ Returns the `numpy.dtype` equivalent of `var`.
 
     Parameters
@@ -150,20 +152,20 @@ def get_dtype(var, int=None, other=None):
     # of ndarray (or something numpy-like)
     try:
         dtype = var.dtype
-    except:
+    except Exception:
         dtype = np.result_type(var)
 
     if dtype == np.int64:
         dtype = int
     try:
         dtype(1)
-    except:
+    except Exception:
         dtype = dtype.type
 
     if other is not None:
         try:
             other(1)
-        except:
+        except Exception:
             other = other.type
         return np.result_type(dtype(1), other(1))
 
@@ -213,7 +215,7 @@ def array_replace(array, *replace, **kwargs):
     [3, 1, 3]
     """
     ar = array.copy()
-    others = list()
+    others = []
 
     for idx, val in replace:
         if not val is None:
