@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """ Named groups via indices
 
 This module implements the base-class which allows named indices
@@ -107,7 +110,7 @@ class NamedIndex:
         try:
             i = self._name.index(name)
             return self._index[i]
-        except:
+        except Exception:
             if isinstance(name, str):
                 return None
             return name
@@ -260,5 +263,5 @@ class NamedIndex:
     def reduce(self):
         """ Removes names from the object which have no index associated (in-place) """
         for i in range(len(self))[::-1]:
-            if len(self._index[n]) == 0:
+            if len(self._index[i]) == 0:
                 del self[self.names[i]]
