@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """ An interface routine for plotting different classes in sisl
 
 It merely calls the `<>.__plot__(**)` routine and returns immediately
@@ -39,7 +42,7 @@ if has_matplotlib:
     plot = _plot
 else:
     def plot(obj, *args, **kwargs):
-        raise _matplotlib_import_exception
+        raise _matplotlib_import_exception   # pylint: disable=E0601
 
 # Clean up
 del has_matplotlib
