@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 from numbers import Integral, Real
 from collections.abc import Iterable
 
@@ -918,7 +921,7 @@ class AtomMeta(type):
         elif isinstance(key, (list, tuple)):
             # The key is a list,
             # we need to create a list of atoms
-            return [cls[k] for k in key]
+            return [cls[k] for k in key] # pylint: disable=E1136
         # Index Z based
         return cls(key)
 
@@ -986,18 +989,28 @@ class Atom(metaclass=AtomMeta):
 
         self._orbitals = None
         if isinstance(orbitals, (tuple, list, np.ndarray)):
-            if isinstance(orbitals[0], Orbital):
+            if len(orbitals) == 0:
+                # This may be the same as only regarding `R` argument
+                pass
+            elif isinstance(orbitals[0], Orbital):
                 # all is good
                 self._orbitals = orbitals
             elif isinstance(orbitals[0], Real):
                 # radius has been given
                 self._orbitals = [Orbital(R) for R in orbitals]
+            elif isinstance(orbitals[0], str):
+                # radius has been given
+                self._orbitals = [Orbital(-1, tag=tag) for tag in orbitals]
+            elif all(orb is None for orb in orbitals):
+                orbitals = None
         elif isinstance(orbitals, Orbital):
             self._orbitals = [orbitals]
         elif isinstance(orbitals, Real):
             self._orbitals = [Orbital(orbitals)]
 
         if self._orbitals is None:
+            if orbitals is not None:
+                raise ValueError(f"{self.__class__.__name__}.__init__ got unparseable 'orbitals' argument: {orbitals}")
             if 'R' in kwargs:
                 # backwards compatibility (possibly remove this in the future)
                 R = _a.asarrayd(kwargs['R']).ravel()
@@ -1015,6 +1028,9 @@ class Atom(metaclass=AtomMeta):
         else:
             self._tag = tag
 
+    def __hash__(self):
+        return hash((self._tag, self._mass, self._Z, *self._orbitals))
+
     @property
     def Z(self):
         """ Atomic number """
@@ -1064,7 +1080,8 @@ class Atom(metaclass=AtomMeta):
 
         Raises
         ------
-        ValueError : if the number of orbitals removed is too large or some indices are outside the allowed range
+        ValueError
+           if the number of orbitals removed is too large or some indices are outside the allowed range
         """
         orbitals = _a.arrayi(orbitals).ravel()
         if len(orbitals) > self.no:
@@ -1097,8 +1114,10 @@ class Atom(metaclass=AtomMeta):
 
     def copy(self, Z=None, orbitals=None, mass=None, tag=None):
         """ Return copy of this object """
+        if orbitals is None:
+            orbitals = [orb.copy() for orb in self]
         return self.__class__(self.Z if Z is None else Z,
-                              self.orbitals if orbitals is None else orbitals,
+                              orbitals,
                               self.mass if mass is None else mass,
                               self.tag if tag is None else tag)
 
@@ -1118,7 +1137,7 @@ class Atom(metaclass=AtomMeta):
         """ The orbital corresponding to index `key` """
         if isinstance(key, slice):
             ol = key.indices(len(self))
-            return [self.orbitals[o] for o in range(ol[0], ol[1], ol[2])]
+            return [self.orbitals[o] for o in range(*ol)]
         elif isinstance(key, Integral):
             return self.orbitals[key]
         elif isinstance(key, str):
@@ -1127,7 +1146,7 @@ class Atom(metaclass=AtomMeta):
             if not orbs:
                 return None
             return orbs if len(orbs) != 1 else orbs[0]
-        return [self.orbitals[o] for o in np.asarray(key).ravel()]
+        return [self.orbitals[o] for o in key]
 
     def maxR(self):
         """ Return the maximum range of orbitals. """
@@ -1346,7 +1365,7 @@ class Atoms:
     # Using the slots should make this class slightly faster.
     __slots__ = ['_atom', '_specie', '_firsto']
 
-    def __init__(self, atoms=None, na=None):
+    def __init__(self, atoms='H', na=None):
 
         # Default value of the atom object
         if atoms is None:
@@ -1369,6 +1388,7 @@ class Atoms:
             specie = [0]
 
         elif isinstance(atoms, Iterable):
+            # TODO this is very inefficient for large MD files
             uatoms = []
             specie = []
             for a in atoms:
@@ -1376,7 +1396,7 @@ class Atoms:
                     a = Atom(a)
                 try:
                     s = uatoms.index(a)
-                except:
+                except Exception:
                     s = len(uatoms)
                     uatoms.append(a)
                 specie.append(s)
@@ -1505,6 +1525,10 @@ class Atoms:
         return atoms
 
     def index(self, atom):
+        """ Return the indices of the atom object """
+        return (self._specie == self.specie_index(atom)).nonzero()[0]
+
+    def specie_index(self, atom):
         """ Return the species index of the atom object """
         if not isinstance(atom, Atom):
             atom = self[atom]
@@ -1643,8 +1667,8 @@ class Atoms:
 
     def swap_atom(self, a, b):
         """ Swap specie index positions """
-        speciea = self.index(a)
-        specieb = self.index(b)
+        speciea = self.specie_index(a)
+        specieb = self.specie_index(b)
 
         idx_a = (self._specie == speciea).nonzero()[0]
         idx_b = (self._specie == specieb).nonzero()[0]
@@ -1676,7 +1700,7 @@ class Atoms:
         spec = np.copy(other._specie)
         for i, atom in enumerate(other.atom):
             try:
-                s = atoms.index(atom)
+                s = atoms.specie_index(atom)
             except KeyError:
                 s = len(atoms.atom)
                 atoms._atom.append(atom)
@@ -1721,7 +1745,7 @@ class Atoms:
                 s = len(atoms.atom)
                 atoms._atom.append(atom)
             else:
-                s = atoms.index(atom)
+                s = atoms.specie_index(atom)
             spec = np.where(spec == i, s, spec)
         atoms._specie = np.insert(atoms._specie, index, spec)
         atoms._update_orbitals()
@@ -1729,10 +1753,10 @@ class Atoms:
 
     def __str__(self):
         """ Return the `Atoms` in str """
-        s = self.__class__.__name__ + '{{species: {0},\n'.format(len(self._atom))
+        s = f"{self.__class__.__name__}{{species: {len(self._atom)},\n"
         for a, idx in self.iter(True):
             s += ' {1}: {0},\n'.format(len(idx), str(a).replace('\n', '\n '))
-        return s + '}'
+        return f"{s}}}"
 
     def __repr__(self):
         return f"<{self.__module__}.{self.__class__.__name__} nspecies={len(self._atom)}, na={len(self)}, no={self.no}>"
@@ -1816,7 +1840,7 @@ class Atoms:
         for atom, s_i in other.iter(True):
             if atom not in self:
                 self._atom.append(atom)
-            self._specie[key[s_i]] = self.index(atom)
+            self._specie[key[s_i]] = self.specie_index(atom)
         self._update_orbitals()
 
     def replace(self, index, atom):
@@ -1845,7 +1869,7 @@ class Atoms:
             self._atom.append(atom)
 
         # Get specie index of the atom
-        specie = self.index(atom)
+        specie = self.specie_index(atom)
 
         # Loop unique species and check that we have the correct number of orbitals
         for ius in np.unique(self._specie[index]):
@@ -1877,8 +1901,10 @@ class Atoms:
 
         Raises
         ------
-        KeyError : if `atom_from` does not exist in the list of atoms
-        UserWarning : if the atoms does not have the same number of orbitals.
+        KeyError
+           if `atom_from` does not exist in the list of atoms
+        UserWarning
+           if the atoms does not have the same number of orbitals.
         """
         if not isinstance(atom_from, Atom):
             raise ValueError(f'{self.__class__.__name__}.replace_atom requires input arguments to '
@@ -1888,9 +1914,9 @@ class Atoms:
                              'be of the class Atom')
 
         # Get index of `atom_from`
-        idx_from = self.index(atom_from)
+        idx_from = self.specie_index(atom_from)
         try:
-            idx_to = self.index(atom_to)
+            idx_to = self.specie_index(atom_to)
             if idx_from == idx_to:
                 raise KeyError("")
 
