@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """
 Physical constants
 ==================
@@ -56,9 +59,11 @@ class PhysicalConstant(float):
     __slots__ = ['_unit']
 
     def __new__(cls, value, unit):
-        c = float.__new__(cls, value)
-        c._unit = unit
-        return c
+        constant = float.__new__(cls, value)
+        return constant
+
+    def __init__(self, value, unit):
+        self._unit = unit
 
     @property
     def unit(self):
@@ -74,6 +79,11 @@ class PhysicalConstant(float):
             return self
         return PhysicalConstant(self * units(self.unit, unit), unit)
 
+    def __eq__(self, other):
+        if isinstance(other, PhysicalConstant):
+            super().__eq__(self, other * units(other.unit, self.unit))
+        return super().__eq__(self, other)
+
 
 __all__ += ['q', 'c', 'h', 'hbar', 'm_e', 'm_p', 'G', 'G0']
 
