@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """
 Common geometries
 =================
@@ -8,10 +11,19 @@ Bulk
    sc
    bcc
    fcc
+   rocksalt
    hcp
    diamond
 
 
+Surfaces
+========
+
+   fcc_slab
+   bcc_slab
+   rocksalt_slab
+
+
 1D materials
 ============
 
@@ -32,11 +44,9 @@ Bulk
 """
 from .basic import *
 from .flat import *
+from .surfaces import *
 from .nanoribbon import *
 from .nanotube import *
 from .special import *
 from .bilayer import *
 from .category import *
-
-
-__all__ = [s for s in dir() if not s.startswith('_')]
