@@ -1,9 +1,13 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 from numbers import Integral
 import numpy as np
 from math import acos, pi
 
 from sisl._internal import set_module
 from sisl import geom, Atom, Cuboid
+from ._common import geometry_define_nsc
 
 __all__ = ['bilayer']
 
@@ -45,7 +49,7 @@ def bilayer(bond=1.42, bottom_atoms=None, top_atoms=None, stacking='AB',
 
     References
     ----------
-    .. [1] G. Trambly de Laissardiere, D. Mayou, L. Magaud, "Localization of Dirac Electrons in Rotated Graphene Bilayers", Nano Letts. 10, 804-808 (2010)
+    .. [1] :doi:`G. Trambly de Laissardiere, D. Mayou, L. Magaud, "Localization of Dirac Electrons in Rotated Graphene Bilayers", Nano Letters **10**, 804-808 (2010) <10.1021/nl902948m>`
     """
     if bottom_atoms is None:
         bottom_atoms = top_atoms
@@ -92,7 +96,7 @@ def bilayer(bond=1.42, bottom_atoms=None, top_atoms=None, stacking='AB',
         natoms = 2 * (n ** 2 + n * m + m ** 2)
 
     if rep > 1:
-        # Set origo through an A atom near the middle of the geometry
+        # Set origin through an A atom near the middle of the geometry
         align_vec = - rep * (ref_cell[0] + ref_cell[1]) / 2
 
         bottom = (bottom
@@ -154,6 +158,8 @@ def bilayer(bond=1.42, bottom_atoms=None, top_atoms=None, stacking='AB',
     # Sanity check
     assert len(bilayer) == natoms
 
+    geometry_define_nsc(bilayer, [True, True, False])
+
     if ret_angle:
         return bilayer, theta
     else:
