@@ -1,16 +1,20 @@
-from functools import reduce, partial, wraps
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
+from functools import reduce, wraps
 import operator as op
 from numbers import Integral
+import re
 
 import numpy as np
-from numpy import dot
 
-from sisl._internal import set_module, singledispatchmethod
+from sisl._internal import set_module
 from sisl._help import isiterable
+from sisl.utils import strmap, lstranges
 from .base import AtomCategory, NullCategory, _sanitize_loop
 
 
-__all__ = ["AtomZ", "AtomIndex", "AtomOdd", "AtomEven"]
+__all__ = ["AtomZ", "AtomIndex", "AtomSeq", "AtomTag", "AtomOdd", "AtomEven"]
 
 
 @set_module("sisl.geom")
@@ -49,6 +53,39 @@ class AtomZ(AtomCategory):
         return False
 
 
+@set_module("sisl.geom")
+class AtomTag(AtomCategory):
+    r""" Classify atoms based on their tag.
+
+    Parameters
+    ----------
+    tag : str
+       The tag you want atoms to match. It can be a regex expression.
+    """
+    __slots__ = ("_compiled_re", "_re")
+
+    def __init__(self, tag):
+        self._re = tag
+        self._compiled_re = re.compile(self._re)
+        super().__init__(f"tag={self._re}")
+
+    @_sanitize_loop
+    def categorize(self, geometry, atoms=None):
+        # _sanitize_loop will ensure that atoms will always be an integer
+        if self._compiled_re.match(geometry.atoms[atoms].tag):
+            return self
+        return NullCategory()
+
+    def __eq__(self, other):
+        if isinstance(other, (list, tuple, np.ndarray)):
+            # this *should* use the dispatch method for different classes
+            return super().__eq__(other)
+
+        if self.__class__ is other.__class__:
+            return self._re == other._re
+        return False
+
+
 @set_module("sisl.geom")
 class AtomIndex(AtomCategory):
     r""" Classify atoms based on indices
@@ -106,13 +143,12 @@ class AtomIndex(AtomCategory):
                 return is_true
             return make_partial
 
-        if len(idx) == 0:
-            operator = []
-        else:
+        operator = []
+        if len(idx) > 0:
             @wraps(op.contains)
             def func_wrap(a, b):
                 return op.contains(b, a)
-            operator.append((func_wrap, b))
+            operator.append((func_wrap, idx))
 
         for func, value in kwargs.items():
             if callable(func):
@@ -147,6 +183,85 @@ class AtomIndex(AtomCategory):
         return False
 
 
+@set_module("sisl.geom")
+class AtomSeq(AtomIndex):
+    r""" Classify atoms based on their indices using a sequence string.
+
+    Parameters
+    ----------
+    seq: str
+       sequence indicating the indices that you want to match (see examples)
+    **kwargs : key, value
+       if key is a function it must accept two values ``value, atom``
+       where ``value`` is the value on this command. The function should
+       return anything that can be interpreted as a True/False.
+       Multiple ``key`` equates to an `and` statement.
+
+    Examples
+    --------
+    >>> seq = AtomSeq("1-3")
+    >>> geom.sub(seq) == geom.sub([1,2,3])
+    >>> seq = AtomSeq("1-3,7")
+    >>> geom.sub(seq) == geom.sub([1,2,3,7])
+    >>> seq = AtomSeq("1-3,7:")
+    >>> geom.sub(seq) == geom.sub([1,2,3,*range(7, len(geom))])
+    >>> seq = AtomSeq("1-3,6,9:2:")
+    >>> geom.sub(seq) == geom.sub([1,2,3,6,*range(9, len(geom), 2)])
+
+    See also
+    ---------
+    `strmap`, `lstranges`:
+        the functions used to parse the sequence string into indices.
+    """
+
+    def __init__(self, seq):
+        self._seq = seq
+        self.set_name(self._seq)
+
+    @staticmethod
+    def _sanitize_negs(indices_map, end):
+        """Converts negative indices to their corresponding positive values.
+
+        Parameters
+        ----------
+        indices_map: list
+            The indices map returned by strmap.
+        end: int
+            The largest valid index.
+        """
+        def _sanitize(item):
+            if isinstance(item, int):
+                if item < 0:
+                    return end + item + 1
+                else:
+                    return item
+            elif isinstance(item, tuple):
+                return (_sanitize(item[0]), *item[1:-1], _sanitize(item[-1]))
+            elif item is None:
+                return item
+            else:
+                raise ValueError(f"Item {item} could not be parsed")
+
+        return [_sanitize(item) for item in indices_map]
+
+    def categorize(self, geometry, *args, **kwargs):
+        # Now that we have the geometry, we know what is the end index
+        # and we can finally safely convert the sequence to indices.
+        indices_map = strmap(int, self._seq, start=0, end=geometry.na - 1)
+        indices = lstranges(self._sanitize_negs(indices_map, end=geometry.na - 1))
+
+        # Initialize the machinery of AtomIndex
+        super().__init__(indices)
+        self.set_name(self._seq)
+        # Finally categorize
+        return super().categorize(geometry, *args, **kwargs)
+
+    def __eq__(self, other):
+        if self.__class__ is other.__class__:
+            return self._seq == other._seq
+        return False
+
+
 class AtomEven(AtomCategory):
     r""" Classify atoms based on indices (even in this case)"""
     __slots__ = []
