@@ -1,5 +1,6 @@
-from collections import namedtuple
-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 from numpy import ndarray
 
 from sisl._internal import set_module
@@ -19,7 +20,7 @@ class AtomNeighbours(AtomCategory):
        minimum number of neighbours
     max : int
        maximum number of neighbours
-    neigh_cat : Category, optional
+    neighbour : Category, optional
        a category the neighbour must be in to be counted
     R : tuple, float, callable or None, optional
        Value passed to `Geometry.close`.
@@ -42,7 +43,7 @@ class AtomNeighbours(AtomCategory):
     def __init__(self, *args, **kwargs):
         if len(args) > 0:
             if isinstance(args[-1], AtomCategory):
-                *args, kwargs["neigh_cat"] = args
+                *args, kwargs["neighbour"] = args
 
         self._min = 0
         self._max = 2 ** 31
@@ -67,7 +68,9 @@ class AtomNeighbours(AtomCategory):
         else:
             name = f" ∈ [{self._min};{self._max}]"
 
-        self._in = kwargs.get("neigh_cat", None)
+        self._in = kwargs.get("neighbour", None)
+        if isinstance(self._in, dict):
+            self._in = AtomCategory(**self._in)
         self._R = kwargs.get("R", None)
 
         # Determine name. If there are requirements for the neighbours
@@ -89,8 +92,7 @@ class AtomNeighbours(AtomCategory):
     @_sanitize_loop
     def categorize(self, geometry, atoms=None):
         """ Check if geometry and atoms matches the neighbour criteria """
-        idx, rij = geometry.close(atoms, R=self.R(geometry.atoms[atoms]), ret_rij=True)
-        idx, rij = idx[1], rij[1]
+        idx = geometry.close(atoms, R=self.R(geometry.atoms[atoms]))[1]
         if len(idx) < self._min:
             return NullCategory()
 
@@ -98,12 +100,8 @@ class AtomNeighbours(AtomCategory):
         if not self._in is None:
             # Get category of neighbours
             cat = self._in.categorize(geometry, geometry.asc2uc(idx))
-            idx1, rij1 = [], []
-            for i in range(len(idx)):
-                if not isinstance(cat[i], NullCategory):
-                    idx1.append(idx[i])
-                    rij1.append(rij[i])
-            idx, rij = idx1, rij1
+            idx = [i for i, c in zip(idx, cat)
+                   if not isinstance(c, NullCategory)]
         n = len(idx)
         if self._min <= n and n <= self._max:
             return self
