@@ -1,5 +1,7 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 from functools import wraps
-from sisl._internal import set_module
 from sisl._category import Category, NullCategory
 from sisl.geometry import AtomCategory
 
