@@ -1,4 +1,6 @@
-from functools import partial
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 from numbers import Integral, Real
 from math import pi
 
@@ -17,7 +19,6 @@ from .messages import deprecate_method
 from .shape import Shape
 from .utils import default_ArgumentParser, default_namespace
 from .utils import cmd, strseq, direction, str_spec
-from .utils import array_arange
 from .utils import import_attr
 from .utils.mathematics import fnorm
 
@@ -351,7 +352,8 @@ class Grid(SuperCellChild):
 
         Raises
         ------
-        ValueError : if specifying periodic one one boundary, so must the opposite side.
+        ValueError
+            if specifying periodic one one boundary, so must the opposite side.
         """
         if not boundary is None:
             if isinstance(boundary, Integral):
@@ -999,7 +1001,8 @@ class Grid(SuperCellChild):
 
         Raises
         ------
-        ValueError: if the grids are not compatible (different shapes)
+        ValueError
+            if the grids are not compatible (different shapes)
         """
         if isinstance(other, Grid):
             grid = self._compatible_copy(other, 'they cannot be added')
@@ -1014,7 +1017,8 @@ class Grid(SuperCellChild):
 
         Raises
         ------
-        ValueError: if the grids are not compatible (different shapes)
+        ValueError
+            if the grids are not compatible (different shapes)
         """
         if isinstance(other, Grid):
             self._check_compatibility(other, 'they cannot be added')
@@ -1028,7 +1032,8 @@ class Grid(SuperCellChild):
 
         Raises
         ------
-        ValueError: if the grids are not compatible (different shapes)
+        ValueError
+            if the grids are not compatible (different shapes)
         """
         if isinstance(other, Grid):
             grid = self._compatible_copy(other, 'they cannot be subtracted')
@@ -1043,7 +1048,8 @@ class Grid(SuperCellChild):
 
         Raises
         ------
-        ValueError: if the grids are not compatible (different shapes)
+        ValueError
+            if the grids are not compatible (different shapes)
         """
         if isinstance(other, Grid):
             self._check_compatibility(other, 'they cannot be subtracted')
@@ -1143,7 +1149,8 @@ class Grid(SuperCellChild):
 
         Raises
         ------
-        ValueError : if any of the passed indices are below 0 or above the number of elements per axis
+        ValueError
+            if any of the passed indices are below 0 or above the number of elements per axis
         """
         index = _a.asarrayi(index).reshape(-1, 3)
         grid = _a.arrayi(self.shape[:])
@@ -1186,7 +1193,7 @@ class Grid(SuperCellChild):
             raise ValueError(self.__class__.__name__ + '.pyamg_fix only works for csr/csc sparse matrices')
 
         # Clean all couplings between the respective indices and all other data
-        s = array_arange(A.indptr[pyamg_indices], A.indptr[pyamg_indices+1])
+        s = _a.array_arange(A.indptr[pyamg_indices], A.indptr[pyamg_indices+1])
         A.data[s] = 0.
         # clean-up
         del s
@@ -1225,7 +1232,7 @@ class Grid(SuperCellChild):
         """
         def Neumann(idx_bc, idx_p1):
             # Set all boundary equations to 0
-            s = array_arange(A.indptr[idx_bc], A.indptr[idx_bc+1])
+            s = _a.array_arange(A.indptr[idx_bc], A.indptr[idx_bc+1])
             A.data[s] = 0
             # force the boundary cells to equal the neighbouring cell
             A[idx_bc, idx_bc] = 1
@@ -1363,13 +1370,11 @@ class Grid(SuperCellChild):
            to create a new.
         limit_arguments : bool, True
            If `False` additional options will be created which are similar to other options.
-           For instance `--repeat-x` which is equivalent to `--repeat x`.
         short : bool, False
            Create short options for a selected range of options
         positional_out : bool, False
            If `True`, adds a positional argument which acts as --out. This may be handy if only the geometry is in the argument list.
         """
-        limit_args = kwargs.get('limit_arguments', True)
         short = kwargs.get('short', False)
 
         def opts(*args):
@@ -1406,7 +1411,6 @@ class Grid(SuperCellChild):
             def __call__(self, parser, ns, value, option_string=None):
                 grid = Grid.read(value)
                 ns._grid -= grid
-                del grid
         p.add_argument(*opts('--diff', '-d'), action=DiffGrid,
                        help='Subtract another grid (they must be commensurate).')
 
@@ -1751,7 +1755,7 @@ This may be unexpected but enables one to do advanced manipulations.
     try:
         if not hasattr(ns, '_input_file'):
             setattr(ns, '_input_file', input_file)
-    except:
+    except Exception:
         pass
 
     # Now try and figure out the actual arguments
