@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """
 Input/Output
 ============
@@ -163,5 +166,3 @@ from .vasp import *
 from .wannier90 import *
 from .xsf import *
 from .xyz import *
-
-__all__ = [s for s in dir() if not s.startswith('_')]
