@@ -1,4 +1,6 @@
-import os.path as osp
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import numpy as np
 
 from sisl.unit import units
@@ -67,7 +69,7 @@ class omxSileOpenMX(SileOpenMX):
             self._parent_fh.append(self.fh)
             self.fh = self.dir_file(f).open(self._mode)
         else:
-            warn(str(self) + f' is trying to include file: {f} but the file seems not to exist? Will disregard file!')
+            warn(f'{self!s} is trying to include file: {f} but the file seems not to exist? Will disregard file!')
 
     def _popfile(self):
         if len(self._parent_fh) > 0:
@@ -82,7 +84,7 @@ class omxSileOpenMX(SileOpenMX):
             while self._popfile():
                 pass
             self.fh.seek(0)
-        except:
+        except Exception:
             pass
 
     @sile_fh_open()
@@ -183,7 +185,7 @@ class omxSileOpenMX(SileOpenMX):
                     # a real number (otherwise an integer)
                     return 'r'
                 return 'i'
-            except:
+            except Exception:
                 pass
             # fall-back to name with everything
 
@@ -299,7 +301,7 @@ class omxSileOpenMX(SileOpenMX):
                 Z = Zr[:idx]
                 try:
                     R = float(Zr[idx:])
-                except:
+                except Exception:
                     pass
 
             # Now figure out the orbitals
@@ -316,11 +318,11 @@ class omxSileOpenMX(SileOpenMX):
                     l = 'spdfg'.index(c)
                     try:
                         nZ = int(spec[i+1])
-                    except:
+                    except Exception:
                         nZ = 1
                     for z in range(nZ):
                         orbs.extend(SphericalOrbital(l, rf_func(R)).toAtomicOrbital(m=m_order[l], zeta=z+1))
-                except:
+                except Exception:
                     pass
 
             return Z, orbs
