@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """
 Siesta
 ======
@@ -30,6 +33,7 @@ Remark that the `gridSileSiesta` file encompass all ``RHO``, ``RHOINIT``, ``DRHO
    fcSileSiesta - Force constant matrix
    kpSileSiesta - k-points from simulation
    rkpSileSiesta - k-points to simulation
+   structSileSiesta - geometry in STRUCT_* files
 
 
 The TranSiesta specific output files are:
@@ -52,6 +56,7 @@ from .kp import *
 from .orb_indx import *
 from .out import *
 from .pdos import *
+from .struct import *
 from .siesta_nc import *
 from .siesta_grid import *
 from .transiesta_grid import *
