@@ -1,3 +1,6 @@
+! This Source Code Form is subject to the terms of the Mozilla Public
+! License, v. 2.0. If a copy of the MPL was not distributed with this
+! file, You can obtain one at https://mozilla.org/MPL/2.0/.
 subroutine read_gf_sizes(iu, nspin, no_u, nkpt, NE)
   use io_m, only: iostat_update
 
@@ -263,6 +266,4 @@ subroutine read_gf_se( iu, no_u, iE, SE )
   read(iu, iostat=ierr) SE
   call iostat_update(ierr)
 
-  SE(:,:) = SE(:,:)
-
 end subroutine read_gf_se
