@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 from ..sile import add_sile
 from .sile import SileSiesta, SileCDFSiesta
 
@@ -8,7 +11,7 @@ from sisl.orbital import SphericalOrbital
 from sisl._array import arrayd, aranged
 from sisl.unit.siesta import unit_convert
 from sisl.utils.cmd import default_ArgumentParser, default_namespace
-from sisl.utils import PropertyDict
+from sisl.utils import PropertyDict, strmap
 
 
 __all__ = ['ionxmlSileSiesta', 'ionncSileSiesta']
