@@ -1,9 +1,11 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import numpy as np
 
 from ..sile import add_sile, sile_fh_open, SileError
 from .sile import SileSiesta
 
-from sisl._array import arrayd
 from sisl._internal import set_module
 from sisl.physics import get_distribution
 from sisl.utils import strmap
@@ -194,7 +196,7 @@ class eigSileSiesta(SileSiesta):
 
         # Energy grabs
         class DOSPlot(argparse.Action):
-            def __call__(dos_self, parser, ns, value, option_string=None):
+            def __call__(dos_self, parser, ns, value, option_string=None): # pylint: disable=E0213
                 import matplotlib.pyplot as plt
                 if not hasattr(ns, '_weight'):
                     # Try and read in the k-point-weights
@@ -218,18 +220,18 @@ class eigSileSiesta(SileSiesta):
                     try:
                         dE = float(value[i])
                         i += 1
-                    except: pass
+                    except Exception: pass
                     try:
                         kT = float(value[i])
                         i += 1
-                    except: pass
+                    except Exception: pass
                     try:
                         distr = get_distribution(value[i], smearing=kT)
                         i += 1
-                    except: pass
+                    except Exception: pass
                     try:
                         out = value[i]
-                    except: pass
+                    except Exception: pass
 
                 # Now we are ready to process
                 E = np.arange(ns._Emap[0] - kT * 4, ns._Emap[1] + kT * 4, dE)
