@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import numpy as np
 
 from ..sile import add_sile, sile_fh_open, sile_raise_write
@@ -49,3 +52,5 @@ class faSileSiesta(SileSiesta):
 
 add_sile('FA', faSileSiesta, gzip=True)
 add_sile('FAC', faSileSiesta, gzip=True)
+add_sile('TSFA', faSileSiesta, gzip=True)
+add_sile('TSFAC', faSileSiesta, gzip=True)
