@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import numpy as np
 
 from ..sile import add_sile, sile_fh_open
@@ -52,7 +55,7 @@ class fcSileSiesta(SileSiesta):
             self.fh.seek(0)
             try:
                 displacement = float(line[-1])
-            except:
+            except Exception:
                 warn(f"{self.__class__.__name__}.read_force assumes displacement=0.04 Bohr!")
                 displacement = 0.04 * unit_convert('Bohr', 'Ang')
 
@@ -83,7 +86,7 @@ class fcSileSiesta(SileSiesta):
         if na is None:
             try:
                 na = int(line[-2])
-            except:
+            except Exception:
                 na = None
 
         fc = list()
