@@ -1,13 +1,15 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 from numbers import Integral
 import numpy as np
 from functools import lru_cache
-from os.path import isfile
 
 from .sile import SileCDFSiesta
-from ..sile import add_sile, sile_fh_open, sile_raise_write
+from ..sile import add_sile, sile_raise_write, SileError
 
 from sisl._internal import set_module
-from sisl._array import aranged
+from sisl._array import aranged, array_arange
 from sisl.unit.siesta import unit_convert
 from sisl import Geometry, Atom, AtomGhost, Atoms, SuperCell, Grid, SphericalOrbital
 from sisl.sparse import _ncol_to_indptr
@@ -19,7 +21,7 @@ from sisl.physics.overlap import Overlap
 from ._help import *
 try:
     from . import _siesta
-except:
+except Exception:
     pass
 
 
@@ -154,7 +156,7 @@ class ncSileSiesta(SileCDFSiesta):
     @lru_cache(maxsize=1)
     def read_force(self):
         """ Returns a vector with final forces contained. """
-        return _a.arrayd(self._value('fa')) * Ry2eV / Bohr2Ang
+        return np.array(self._value('fa')) * Ry2eV / Bohr2Ang
 
     @lru_cache(maxsize=1)
     def read_fermi_level(self):
@@ -240,7 +242,7 @@ class ncSileSiesta(SileCDFSiesta):
 
         sp = self.groups['SPARSE']
         if sp.variables['H'].unit != 'Ry':
-            raise SileError(self.__class__.__name__ + '.read_hamiltonian requires the stored matrix to be in Ry!')
+            raise SileError(f'{self}.read_hamiltonian requires the stored matrix to be in Ry!')
 
         for i in range(len(H.spin)):
             H._csr._D[:, i] = sp.variables['H'][i, :] * Ry2eV
@@ -264,7 +266,7 @@ class ncSileSiesta(SileCDFSiesta):
 
         sp = self.groups['SPARSE']
         if sp.variables['H'].unit != 'Ry**2':
-            raise SileError(self.__class__.__name__ + '.read_dynamical_matrix requires the stored matrix to be in Ry**2!')
+            raise SileError(f'{self}.read_dynamical_matrix requires the stored matrix to be in Ry**2!')
         D._csr._D[:, 0] = sp.variables['H'][0, :] * Ry2eV ** 2
 
         return D.transpose(sort=kwargs.get("sort", True))
@@ -312,7 +314,7 @@ class ncSileSiesta(SileCDFSiesta):
                  contains the directions, and 3rd dimensions contains -/+ displacements.
         """
         if not 'FC' in self.groups:
-            raise SislError(str(self) + '.read_force_constant cannot find the FC group.')
+            raise SislError(f'{self}.read_force_constant cannot find the FC group.')
         fc = self.groups['FC']
 
         disp = fc.variables['disp'][0] * Bohr2Ang
@@ -378,7 +380,7 @@ class ncSileSiesta(SileCDFSiesta):
             grid.grid = v[spin, :, :, :] * unit
         else:
             if len(spin) > v.shape[0]:
-                raise SileError(self.__class__.__name__ + '.read_grid requires spin to be an integer or '
+                raise SileError(f'{self}.read_grid requires spin to be an integer or '
                                 'an array of length equal to the number of spin components.')
             grid.grid[:, :, :] = v[0, :, :, :] * (spin[0] * unit)
             for i, scale in enumerate(spin[1:]):
@@ -388,7 +390,7 @@ class ncSileSiesta(SileCDFSiesta):
             if v.unit == 'Ry':
                 # Convert to ev
                 grid *= Ry2eV
-        except:
+        except Exception:
             # Allowed pass due to pythonic reading
             pass
 
@@ -526,16 +528,32 @@ class ncSileSiesta(SileCDFSiesta):
             # We need to create the orthogonal pattern
             tmp = csr.copy(dims=[0])
             tmp.empty(keep_nnz=True)
-            for i in range(tmp.shape[0]):
-                tmp[i, i] = 1.
+            ptr = tmp.ptr
+            ncol = tmp.ncol
+            col = tmp.col
+            D = tmp._D
 
-            if tmp.nnz != csr.nnz:
-                # We have added more stuff, something that we currently do not allow.
-                raise ValueError(self.__class__.__name__ + '._write_overlap '
+            # Now retrieve rows and cols
+            idx = (ncol > 0).nonzero()[0]
+            row = np.repeat(idx.astype(np.int32, copy=False), ncol[idx])
+            idx = array_arange(ptr[:-1], n=ncol, dtype=np.int32)
+            col = col[idx]
+
+            # Now figure out the diagonal components
+            diag_idx = np.equal(row, col)
+
+            # Ensure we have only len(tmp) equal number of
+            # elements
+            if diag_idx.sum() != len(tmp):
+                # Figure out which row is missing
+                row = row[diag_idx]
+                idx = (np.diff(row) != 1).nonzero()[0]
+                row = row[idx] + 1
+                raise ValueError(f'{self}._write_overlap '
                                  'is trying to write an Overlap in Siesta format with '
-                                 'not all diagonal terms defined. Please correct. '
-                                 'I.e. explicitly add *all* diagonal overlap terms.')
+                                 f'missing diagonal terms on rows {row}. Please explicitly add *all* diagonal overlap terms.')
 
+            D[idx[diag_idx]] = 1.
             v[:] = tmp._D[:, 0]
             del tmp
         else:
@@ -545,7 +563,7 @@ class ncSileSiesta(SileCDFSiesta):
         """ Write the overlap matrix to the NetCDF file """
         csr = S.transpose(sort=False)._csr
         if csr.nnz == 0:
-            raise SileError(str(self) + '.write_overlap cannot write a zero element sparse matrix!')
+            raise SileError(f'{self}.write_overlap cannot write a zero element sparse matrix!')
 
         # Convert to siesta CSR
         _csr_to_siesta(S.geometry, csr)
@@ -571,7 +589,7 @@ class ncSileSiesta(SileCDFSiesta):
         """
         csr = H.transpose(spin=False, sort=False)._csr
         if csr.nnz == 0:
-            raise SileError(str(self) + '.write_hamiltonian cannot write a zero element sparse matrix!')
+            raise SileError(f'{self}.write_hamiltonian cannot write a zero element sparse matrix!')
 
         # Convert to siesta CSR
         _csr_to_siesta(H.geometry, csr)
@@ -619,7 +637,7 @@ class ncSileSiesta(SileCDFSiesta):
         """
         csr = DM.transpose(spin=False, sort=False)._csr
         if csr.nnz == 0:
-            raise SileError(str(self) + '.write_density_matrix cannot write a zero element sparse matrix!')
+            raise SileError(f'{self}.write_density_matrix cannot write a zero element sparse matrix!')
 
         # Convert to siesta CSR (we don't need to sort this matrix)
         _csr_to_siesta(DM.geometry, csr)
@@ -666,7 +684,7 @@ class ncSileSiesta(SileCDFSiesta):
         """
         csr = EDM.transpose(spin=False, sort=False)._csr
         if csr.nnz == 0:
-            raise SileError(str(self) + '.write_energy_density_matrix cannot write a zero element sparse matrix!')
+            raise SileError(f'{self}.write_energy_density_matrix cannot write a zero element sparse matrix!')
 
         # no need to sort this matrix
         _csr_to_siesta(EDM.geometry, csr)
@@ -718,7 +736,7 @@ class ncSileSiesta(SileCDFSiesta):
         """
         csr = D.transpose(sort=False)._csr
         if csr.nnz == 0:
-            raise SileError(str(self) + '.write_dynamical_matrix cannot write a zero element sparse matrix!')
+            raise SileError(f'{self}.write_dynamical_matrix cannot write a zero element sparse matrix!')
 
         # Convert to siesta CSR
         _csr_to_siesta(D.geometry, csr)
