@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
 import os.path as osp
 from sisl.io.siesta.fc import *
@@ -28,7 +31,7 @@ def test_read_fc(sisl_tmp):
     assert np.allclose(fc, fc2)
 
 
-@pytest.mark.filterwarnings("ignore:assumes displacement=")
+@pytest.mark.filterwarnings("ignore", message="*assumes displacement=")
 def test_read_fc_old(sisl_tmp):
     f = sisl_tmp('test2.FC', _dir)
 
