@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """ pytest test configures """
 
 import pytest
@@ -39,7 +42,7 @@ def test_si_pdos_kgrid_tsde_edm(sisl_files):
     assert np.allclose(EDM1._csr._D[:, :-1], EDM2._csr._D[:, :-1])
 
 
-@pytest.mark.filterwarnings("ignore:wrong sparse pattern")
+@pytest.mark.filterwarnings("ignore", message="*wrong sparse pattern")
 def test_si_pdos_kgrid_tsde_dm_edm_rw(sisl_files, sisl_tmp):
     fdf = sisl.get_sile(sisl_files(_dir, 'si_pdos_kgrid.fdf'), base=sisl_files(_dir))
     geom = fdf.read_geometry()
