@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """ pytest test configures """
 
 import pytest
@@ -12,6 +15,7 @@ _dir = osp.join('sisl', 'io', 'siesta')
 
 def test_tshs_si_pdos_kgrid(sisl_files, sisl_tmp):
     si = sisl.get_sile(sisl_files(_dir, 'si_pdos_kgrid.TSHS'))
+    assert si.version == 1
     HS1 = si.read_hamiltonian()
     f = sisl_tmp('tmp.TSHS', _dir)
     HS1.write(f)
@@ -131,6 +135,7 @@ def test_tshs_si_pdos_kgrid_overlap(sisl_files):
     assert np.allclose(HS._csr._D[:, HS.S_idx], S._csr._D[:, 0])
 
 
+@pytest.mark.filterwarnings("ignore", message="*is NOT Hermitian for on-site")
 def test_tshs_spin_orbit(sisl_tmp):
     H1 = sisl.Hamiltonian(sisl.geom.graphene(), spin=sisl.Spin('SO'))
     H1.construct(([0.1, 1.44],
@@ -150,6 +155,7 @@ def test_tshs_spin_orbit(sisl_tmp):
     assert np.allclose(H1._csr._D, H3._csr._D)
 
 
+@pytest.mark.filterwarnings("ignore", message="*is NOT Hermitian for on-site")
 def test_tshs_spin_orbit_tshs2nc2tshs(sisl_tmp):
     H1 = sisl.Hamiltonian(sisl.geom.graphene(), spin=sisl.Spin('SO'))
     H1.construct(([0.1, 1.44],
@@ -174,3 +180,26 @@ def test_tshs_spin_orbit_tshs2nc2tshs(sisl_tmp):
     assert np.allclose(H1._csr._D, H2._csr._D)
     assert H1._csr.spsame(H3._csr)
     assert np.allclose(H1._csr._D, H3._csr._D)
+
+
+def test_tshs_missing_diagonal(sisl_tmp):
+    H1 = sisl.Hamiltonian(sisl.geom.graphene())
+    H1.construct(([0.1, 1.44], [0., -2.7]))
+    # remove diagonal component here
+    del H1[0, 0]
+
+    f1 = sisl_tmp('tmp1.TSHS', _dir)
+    H1.write(f1)
+
+    f2 = sisl_tmp('tmp2.TSHS', _dir)
+    H2 = sisl.get_sile(f1).read_hamiltonian()
+    H2.write(f2)
+    H3 = sisl.get_sile(f2).read_hamiltonian()
+
+    H1.finalize()
+    assert not H1._csr.spsame(H2._csr)
+    assert H2._csr.spsame(H3._csr)
+    assert np.allclose(H2._csr._D, H3._csr._D)
+    H1[0, 0] = 0.
+    H1.finalize()
+    assert H1._csr.spsame(H2._csr)
