@@ -1,6 +1,9 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import numpy as np
 
-from ..sile import add_sile, sile_fh_open, sile_raise_write
+from ..sile import add_sile, sile_fh_open, sile_raise_write, SileError
 from .sile import SileSiesta
 
 from sisl._internal import set_module
@@ -18,12 +21,12 @@ class xvSileSiesta(SileSiesta):
     """ Geometry file """
 
     @sile_fh_open()
-    def write_geometry(self, geom, fmt='.9f', velocity=None):
+    def write_geometry(self, geometry, fmt='.9f', velocity=None):
         """ Writes the geometry to the contained file
 
         Parameters
         ----------
-        geom : Geometry
+        geometry : Geometry
            geometry to write in the XV file
         fmt : str, optional
            the precision used for writing the XV file
@@ -35,9 +38,9 @@ class xvSileSiesta(SileSiesta):
         sile_raise_write(self)
 
         if velocity is None:
-            velocity = np.zeros([geom.na, 3], np.float32)
-        if geom.xyz.shape != velocity.shape:
-            raise SislError(str(self) + '.write_geometry requires the input'
+            velocity = np.zeros([geometry.na, 3], np.float32)
+        if geometry.xyz.shape != velocity.shape:
+            raise SileError(f'{self}.write_geometry requires the input'
                             'velocity to have equal length to the input geometry.')
 
         # Write unit-cell
@@ -46,16 +49,16 @@ class xvSileSiesta(SileSiesta):
         # Create format string for the cell-parameters
         fmt_str = ('   ' + ('{:' + fmt + '} ') * 3) * 2 + '\n'
         for i in range(3):
-            tmp[0:3] = geom.cell[i, :] / Bohr2Ang
+            tmp[0:3] = geometry.cell[i, :] / Bohr2Ang
             self._write(fmt_str.format(*tmp))
-        self._write(f'{geom.na:12d}\n')
+        self._write(f'{geometry.na:12d}\n')
 
         # Create format string for the atomic coordinates
         fmt_str = '{:3d}{:6d} '
         fmt_str += ('{:' + fmt + '} ') * 3 + '   '
         fmt_str += ('{:' + fmt + '} ') * 3 + '\n'
-        for ia, a, ips in geom.iter_species():
-            tmp[0:3] = geom.xyz[ia, :] / Bohr2Ang
+        for ia, a, ips in geometry.iter_species():
+            tmp[0:3] = geometry.xyz[ia, :] / Bohr2Ang
             tmp[3:] = velocity[ia, :] / Bohr2Ang
             if isinstance(a, AtomGhost):
                 self._write(fmt_str.format(ips + 1, -a.Z, *tmp))
