@@ -1,10 +1,11 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 from functools import wraps
-from os.path import splitext, isfile, dirname, join, abspath, basename
+from os.path import splitext, basename
 import gzip
 from pathlib import Path
 
-import numpy as np
-
 from sisl._internal import set_module
 from sisl.messages import SislWarning, SislInfo
 from sisl.utils.misc import str_spec
@@ -80,11 +81,11 @@ class _sile_rule:
         while len(children) != nl:
             nl = len(children)
             # Remove baseclasses everybody have
-            for obj in [object, BaseSile, Sile, SileBin, SileCDF]:
+            for obj in (object, BaseSile, Sile, SileBin, SileCDF):
                 try:
                     i = children.index(obj)
                     children.pop(i)
-                except:
+                except Exception:
                     pass
             for child in list(children): # ensure we have a copy for infinite loops
                 for c in child.__bases__:
@@ -370,11 +371,7 @@ class BaseSile:
         """ File of the current `Sile` """
         if filename is None:
             filename = Path(self._file).name
-        return Path(self._directory) / filename_base / filename
-
-    def exist(self):
-        """ Query whether the file exists """
-        return isfile(self.file)
+        return self._directory / filename_base / filename
 
     def read(self, *args, **kwargs):
         """ Generic read method which should be overloaded in child-classes
@@ -451,7 +448,7 @@ class BaseSile:
 
     def _base_file(self, f):
         """ Make `f` refer to the file with the appropriate base directory """
-        return join(self._directory, f)
+        return self._directory / f
 
     def __getattr__(self, name):
         """ Override to check the handle """
@@ -473,7 +470,7 @@ class BaseSile:
         p : ArgumentParser
            the argument parser to add the arguments to.
         """
-        raise NotImplementedError("The ArgumentParser of '"+self.__class__.__name__+"' has not been implemented yet.")
+        raise NotImplementedError(f"The ArgumentParser of '{self.__class__.__name__}' has not been implemented yet.")
 
     def ArgumentParser_out(self, p=None, *args, **kwargs):
         """ Appends additional arguments based on the output of the file
@@ -487,7 +484,14 @@ class BaseSile:
 
     def __str__(self):
         """ Return a representation of the `Sile` """
-        return "{0}({1!s}, base={2!s})".format(self.__class__.__name__, self.base_file, self._directory)
+        # Check if the directory is relative to the current path
+        # If so, only print the relative path, otherwise print the full path
+        d = self._directory
+        try:
+            # bypass d.is_relative_to, added in 3.9
+            d = d.relative_to(Path('.').resolve())
+        except Exception: pass
+        return f"{self.__class__.__name__}({self.base_file!s}, base={d!s})"
 
 
 def sile_fh_open(from_closed=False):
@@ -545,7 +549,12 @@ class Sile(BaseSile):
 
     def _open(self):
         if self.file.suffix == ".gz":
-            self.fh = gzip.open(str(self.file))
+            if self._mode == 'r':
+                # assume the file is a text file and open in text-mode
+                self.fh = gzip.open(str(self.file), mode='rt')
+            else:
+                # assume this is opening in binary or write mode
+                self.fh = gzip.open(str(self.file), mode=self._mode)
         else:
             self.fh = self.file.open(self._mode)
         self._line = 0
@@ -564,8 +573,8 @@ class Sile(BaseSile):
 
     @staticmethod
     def is_keys(keys):
-        """ Returns true if ``isinstance(keys,(list,np.ndarray))`` """
-        return isinstance(keys, (list, np.ndarray))
+        """ Returns true if ``not isinstance(keys, str)`` """
+        return not isinstance(keys, str)
 
     @staticmethod
     def key2case(key, case):
@@ -614,26 +623,22 @@ class Sile(BaseSile):
             self._line += 1
         return l
 
-    def step_to(self, keywords, case=True, reread=True):
-        r""" Steps the file-handle until the keyword is found in the input """
+    def step_to(self, keywords, case=True, reread=True, ret_index=False):
+        r""" Steps the file-handle until the keyword(s) is found in the input """
         # If keyword is a list, it just matches one of the inputs
         found = False
         # The previously read line...
         line = self._line
-
-        # Do checking outside line checks
-        if self.is_keys(keywords):
-            line_has = self.line_has_keys
-            keys = self.keys2case(keywords, case)
-        else:
-            line_has = self.line_has_key
-            keys = self.key2case(keywords, case)
+        if isinstance(keywords, str):
+            # convert to list
+            keywords = [keywords]
+        keys = self.keys2case(keywords, case)
 
         while not found:
             l = self.readline()
             if l == '':
                 break
-            found = line_has(l, keys, case=case)
+            found = self.line_has_keys(l, keys, case)
 
         if not found and (l == '' and line > 0) and reread:
             # We may be in the case where the user request
@@ -648,43 +653,23 @@ class Sile(BaseSile):
                 l = self.readline()
                 if l == '':
                     break
-                found = line_has(l, keys, case=case)
+                found = self.line_has_keys(l, keys, case)
+
+        if ret_index:
+            idx = -1
+            if found:
+                idx = 0
+
+            # force return an index
+            for i, key in enumerate(keys):
+                if self.line_has_key(l, key, case):
+                    return found, l, i
+            return found, l, idx
 
         # sometimes the line contains information, as a
         # default we return the line found
         return found, l
 
-    def step_either(self, keywords, case=True):
-        """ Steps the file-handle until the keyword is found in the input """
-        # If keyword is a list, it just matches one of the inputs
-
-        # Do checking outside line checks
-        if self.is_keys(keywords):
-            line_has = self.line_has_key
-            keys = self.keys2case(keywords, case)
-        else:
-            found, l = self.step_to(keywords, case)
-            return found, 0, l
-
-        # initialize
-        found = False
-        j = -1
-
-        while not found:
-            l = self.readline()
-            if l == '':
-                break
-
-            for i, key in enumerate(keys):
-                found = line_has(l, key, case=case)
-                if found:
-                    j = i
-                    break
-
-        # sometimes the line contains information, as a
-        # default we return the line found
-        return found, j, l
-
     def _write(self, *args, **kwargs):
         """ Wrapper to default the write statements """
         self.fh.write(*args, **kwargs)
@@ -725,7 +710,7 @@ class SileCDF(BaseSile):
         self._lvl = lvl
         # Initialize the _data dictionary for access == 1
         self._data = dict()
-        if self.exist():
+        if self.file.is_file():
             self._access = access
         else:
             # If it has not been created we should not try
@@ -822,7 +807,7 @@ class SileCDF(BaseSile):
             grp = n
             for group in groups:
                 if len(group) > 0:
-                    grp = _crt_grp(grp, group)
+                    grp = SileCDF._crt_grp(grp, group)
             return grp
 
         if name in n.groups:
@@ -832,8 +817,8 @@ class SileCDF(BaseSile):
     @staticmethod
     def _crt_dim(n, name, l):
         if name in n.dimensions:
-            return
-        n.createDimension(name, l)
+            return n.dimensions[name]
+        return n.createDimension(name, l)
 
     @staticmethod
     def _crt_var(n, name, *args, **kwargs):
@@ -1003,7 +988,7 @@ class SileError(IOError):
 
     def __str__(self):
         if self.obj:
-            return self.value + ' in ' + str(self.obj)
+            return f"{self.value!s} in {self.obj!s}"
         else:
             return self.value
 
