@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """ pytest test configures """
 
 import pytest
@@ -12,7 +15,7 @@ _dir = osp.join('sisl', 'io', 'tbtrans')
 
 
 @pytest.mark.slow
-@pytest.mark.filterwarnings("ignore:.*.o2p")
+@pytest.mark.filterwarnings("ignore", message="*.*.o2p")
 def test_1_graphene_all_content(sisl_files):
     """ This tests manifolds itself as:
 
@@ -371,6 +374,8 @@ def test_1_graphene_all_fail_kavg_E(sisl_files, sisl_tmp):
 
 def test_1_graphene_all_ArgumentParser(sisl_files, sisl_tmp):
     pytest.importorskip("matplotlib", reason="matplotlib not available")
+    import matplotlib as mpl
+    mpl.rcParams["text.usetex"] = False
 
     # Local routine to run the collected actions
     def run(ns):
@@ -508,3 +513,35 @@ def test_1_graphene_all_warn_atom(sisl_files):
     tbt = sisl.get_sile(sisl_files(_dir, '1_graphene_all.TBT.nc'))
     with pytest.warns(sisl.SislWarning):
         tbt.a2p(1)
+
+
+def test_1_graphene_all_sparse_data_isc_request(sisl_files):
+    tbt = sisl.get_sile(sisl_files(_dir, '1_graphene_all.TBT.nc'))
+
+    # get supercell with isc
+    sc = tbt.read_supercell()
+
+    # request the full matrix
+    for elec in [0, 1]:
+        J_all = tbt.orbital_current(elec, 204)
+        J_all.eliminate_zeros()
+
+        # Ensure we actually have something
+        assert J_all.nnz > 0
+
+        # partial summed isc
+        # Test that the full matrix and individual access is the same
+        J_sum = sum(tbt.orbital_current(elec, 204, isc=isc)
+                    for isc in sc.sc_off)
+        assert J_sum.nnz == J_all.nnz
+        assert (J_sum - J_all).nnz == 0
+
+
+def test_1_graphene_all_sparse_data_orbitals(sisl_files):
+    tbt = sisl.get_sile(sisl_files(_dir, '1_graphene_all.TBT.nc'))
+
+    # request the full matrix
+    J_all = tbt.orbital_current(0, 204)
+    J_12 = tbt.orbital_current(0, 204, orbitals=[2, 3])
+
+    assert J_12.nnz < J_all.nnz // 2
