@@ -1,4 +1,8 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
+import sys
 import os.path as osp
 import numpy as np
 import os
@@ -7,6 +11,7 @@ from pathlib import Path
 from sisl.io import *
 from sisl.io.siesta.binaries import _gfSileSiesta
 from sisl.io.tbtrans._cdf import *
+from sisl.io.vasp import chgSileVASP
 from sisl import Geometry, Grid, Hamiltonian
 from sisl import DensityMatrix, EnergyDensityMatrix
 
@@ -236,17 +241,22 @@ class TestObject:
         # These files does not store the atomic species
         if issubclass(sile, (_ncSileTBtrans, deltancSileTBtrans)):
             return
+        if sys.platform.startswith("win") and issubclass(sile, chgSileVASP):
+            pytest.xfail("Windows reading/writing supercell fails for some unknown reason")
+
         # Write
         sile(f, mode="w").write_geometry(G)
         # Read 1
         try:
-            g = sile(f, mode="r").read_geometry()
+            with sile(f, mode='r') as s:
+                g = s.read_geometry()
             assert g.equal(G, R=False, tol=1e-3) # pdb files have 8.3 for atomic coordinates
         except UnicodeDecodeError as e:
             pass
         # Read 2
         try:
-            g = Geometry.read(sile(f, mode="r"))
+            with sile(f, mode='r') as s:
+                g = Geometry.read(s)
             assert g.equal(G, R=False, tol=1e-3)
         except UnicodeDecodeError as e:
             pass
@@ -261,16 +271,19 @@ class TestObject:
         H.construct([[0.1, 1.45], [0.1, -2.7]])
         f = sisl_tmp("test_read_write_hamiltonian.win", _dir)
         # Write
-        sile(f, mode="w").write_hamiltonian(H)
+        with sile(f, mode='w') as s:
+            s.write_hamiltonian(H)
         # Read 1
         try:
-            h = sile(f, mode="r").read_hamiltonian()
+            with sile(f, mode='r') as s:
+                h = s.read_hamiltonian()
             assert H.spsame(h)
         except UnicodeDecodeError as e:
             pass
         # Read 2
         try:
-            h = Hamiltonian.read(sile(f, mode="r"))
+            with sile(f, mode='r') as s:
+                h = Hamiltonian.read(s)
             assert H.spsame(h)
         except UnicodeDecodeError as e:
             pass
@@ -282,16 +295,19 @@ class TestObject:
         DM.construct([[0.1, 1.45], [0.1, -2.7]])
         f = sisl_tmp("test_read_write_density_matrix.win", _dir)
         # Write
-        sile(f, mode="w").write_density_matrix(DM)
+        with sile(f, mode='w') as s:
+            s.write_density_matrix(DM)
         # Read 1
         try:
-            dm = sile(f, mode="r").read_density_matrix(geometry=DM.geometry)
+            with sile(f, mode='r') as s:
+                dm = s.read_density_matrix(geometry=DM.geometry)
             assert DM.spsame(dm)
         except UnicodeDecodeError as e:
             pass
         # Read 2
         try:
-            dm = DensityMatrix.read(sile(f, mode="r"), geometry=DM.geometry)
+            with sile(f, mode='r') as s:
+                dm = DensityMatrix.read(s, geometry=DM.geometry)
             assert DM.spsame(dm)
         except UnicodeDecodeError as e:
             pass
@@ -303,16 +319,19 @@ class TestObject:
         EDM.construct([[0.1, 1.45], [0.1, -2.7]])
         f = sisl_tmp("test_read_write_energy_density_matrix.win", _dir)
         # Write
-        sile(f, mode="w").write_energy_density_matrix(EDM)
+        with sile(f, mode='w') as s:
+            s.write_energy_density_matrix(EDM)
         # Read 1
         try:
-            edm = sile(f, mode="r").read_energy_density_matrix(geometry=EDM.geometry)
+            with sile(f, mode='r') as s:
+                edm = s.read_energy_density_matrix(geometry=EDM.geometry)
             assert EDM.spsame(edm)
         except UnicodeDecodeError as e:
             pass
         # Read 2
         try:
-            edm = EnergyDensityMatrix.read(sile(f, mode="r"), geometry=EDM.geometry)
+            with sile(f, mode='r') as s:
+                edm = EnergyDensityMatrix.read(s, geometry=EDM.geometry)
             assert EDM.spsame(edm)
         except UnicodeDecodeError as e:
             pass
@@ -327,21 +346,24 @@ class TestObject:
         H.construct([[0.1, 1.45], [(0.1, 1), (-2.7, 0.1)]])
         f = sisl_tmp("test_read_write_hamiltonian_overlap.win", _dir)
         # Write
-        sile(f, mode="w").write_hamiltonian(H)
+        with sile(f, mode='w') as s:
+            s.write_hamiltonian(H)
         # Read 1
         try:
-            h = sile(f, mode="r").read_hamiltonian()
+            with sile(f, mode='r') as s:
+                h = s.read_hamiltonian()
             assert H.spsame(h)
         except UnicodeDecodeError as e:
             pass
         # Read 2
         try:
-            h = Hamiltonian.read(sile(f, mode="r"))
+            with sile(f, mode='r') as s:
+                h = Hamiltonian.read(s)
             assert H.spsame(h)
         except UnicodeDecodeError as e:
             pass
 
-    @pytest.mark.filterwarnings("ignore:gridncSileSiesta.read_grid cannot determine")
+    @pytest.mark.filterwarnings("ignore", message="*gridncSileSiesta.read_grid cannot determine")
     @pytest.mark.parametrize("sile", _my_intersect(["read_grid"], ["write_grid"]))
     def test_read_write_grid(self, sisl_tmp, sisl_system, sile):
         g = sisl_system.g.rotate(-30, sisl_system.g.cell[2, :])
@@ -351,18 +373,22 @@ class TestObject:
         f = sisl_tmp("test_read_write_grid.win", _dir)
         # Write
         try:
-            sile(f, mode="w").write_grid(G)
+            with sile(f, mode="w") as s:
+                s.write_grid(G)
         except SileError:
-            sile(f, mode="wb").write_grid(G)
+            with sile(f, mode="wb") as s:
+                s.write_grid(G)
         # Read 1
         try:
-            g = sile(f, mode="r").read_grid()
+            with sile(f, mode="r") as s:
+                g = s.read_grid()
             assert np.allclose(g.grid, G.grid, atol=1e-5)
         except UnicodeDecodeError as e:
             pass
         # Read 2
         try:
-            g = Grid.read(sile(f, mode="r"))
+            with sile(f, mode='r') as s:
+                g = Grid.read(s)
             assert np.allclose(g.grid, G.grid, atol=1e-5)
         except UnicodeDecodeError as e:
             pass
@@ -372,7 +398,7 @@ class TestObject:
         for sile in get_siles(["ArgumentParser"]):
             try:
                 sile(f).ArgumentParser()
-            except:
+            except Exception:
                 pass
 
     def test_arg_parser2(self, sisl_tmp):
@@ -380,5 +406,5 @@ class TestObject:
         for sile in get_siles(["ArgumentParser_out"]):
             try:
                 sile(f).ArgumentParser()
-            except:
+            except Exception:
                 pass
