@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
 import os.path as osp
 import numpy as np
@@ -17,6 +20,8 @@ def test_xyz1(sisl_tmp, sisl_system):
     assert np.allclose(g.cell, sisl_system.g.cell)
     assert np.allclose(g.xyz, sisl_system.g.xyz)
     assert sisl_system.g.atoms.equal(g.atoms, R=False)
+    g = xyzSile(f).read_geometry(sc=g.sc)
+    assert np.allclose(g.cell, sisl_system.g.cell)
 
 
 def test_xyz_sisl(sisl_tmp):
@@ -36,6 +41,9 @@ C   2.00000  0.00000000  0.00000000
     assert np.allclose(g.xyz[:, 2], 0.)
     assert np.allclose(g.nsc, [1, 1, 3])
 
+    g = xyzSile(f).read_geometry(sc=[10, 11, 13])
+    assert np.allclose(g.cell, [[10, 0, 0], [0, 11, 0], [0, 0, 13]])
+
 
 def test_xyz_ase(sisl_tmp):
     f = sisl_tmp('ase.xyz', _dir)
