@@ -1,12 +1,15 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """
 Define a common VASP Sile
 """
-from ..sile import Sile, SileCDF, SileBin
+from ..sile import Sile, SileBin
 from sisl._internal import set_module
 import sisl._array as _a
 
 
-__all__ = ['SileVASP', 'SileCDFVASP', 'SileBinVASP']
+__all__ = ["SileVASP", "SileBinVASP"]
 
 
 def _geometry_group(geometry, ret_index=False):
@@ -56,11 +59,6 @@ class SileVASP(Sile):
     geometry_group = staticmethod(_geometry_group)
 
 
-@set_module("sisl.io.vasp")
-class SileCDFVASP(SileCDF):
-    geometry_group = staticmethod(_geometry_group)
-
-
 @set_module("sisl.io.vasp")
 class SileBinVASP(SileBin):
     geometry_group = staticmethod(_geometry_group)
