@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """
 Sile object for reading/writing Wannier90 in/output
 """
@@ -177,7 +180,7 @@ class winSileWannier90(SileWannier90):
         sc = self.read_supercell()
 
         self._set_file('_centres.xyz')
-        if self.exist():
+        if self.file.is_file():
             geom = self._read_geometry_centres()
         else:
             self._set_file()
