@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import os.path as osp
 from numbers import Integral
 import numpy as np
@@ -6,6 +9,7 @@ import numpy as np
 from .sile import *
 
 from sisl._internal import set_module
+from sisl import PeriodicTable
 from sisl import Geometry, AtomUnknown, SuperCell
 from sisl.utils import str_spec
 import sisl._array as _a
@@ -31,7 +35,7 @@ class xsfSile(Sile):
         self._comment = ['#']
 
     def _write_key(self, key):
-        self._write(key + "\n")
+        self._write(f"{key}\n")
 
     _write_once = Sile._write
 
@@ -98,7 +102,7 @@ class xsfSile(Sile):
 
         self._write_once('#\n# Atomic coordinates (in primitive coordinates)\n#\n')
         self._write_key("PRIMCOORD")
-        self._write('{} {}\n'.format(len(geometry), 1))
+        self._write(f'{len(geometry)} 1\n')
 
         non_valid_Z = (geometry.atoms.Z <= 0).nonzero()[0]
         if len(non_valid_Z) > 0:
@@ -133,6 +137,8 @@ class xsfSile(Sile):
         line = " "
         all_loaded = False
 
+        pt = PeriodicTable()
+
         while line != '' and not all_loaded:
             line = self.readline()
 
@@ -202,7 +208,7 @@ class xsfSile(Sile):
                 for _ in range(int(line[0])):
                     line = self.readline().split()
                     if not xyz_set[istep]:
-                        iatom.append(int(line[0]))
+                        iatom.append(pt.Z(line[0]))
                         ixyz.append([float(x) for x in line[1:4]])
                     if ret_data and len(line) > 4:
                         idata.append([float(x) for x in line[4:]])
@@ -231,7 +237,7 @@ class xsfSile(Sile):
         geoms = []
         for istep in range(len(steps)):
             if len(atom) == 0:
-                geoms.append(si.Geometry(xyz[istep], sc=SuperCell(cell[istep])))
+                geoms.append(Geometry(xyz[istep], sc=SuperCell(cell[istep])))
             elif len(atom[0]) == 1 and atom[0][0] == -999:
                 # should we perhaps do AtomUnknown?
                 geoms.append(None)
@@ -303,7 +309,7 @@ class xsfSile(Sile):
         def write_cell(grid):
             # Now write the grid
             self._write('  {} {} {}\n'.format(*grid.shape))
-            self._write('  ' + _v3.format(*grid.origo))
+            self._write('  ' + _v3.format(*grid.origin))
             self._write('  ' + _v3.format(*grid.cell[0, :]))
             self._write('  ' + _v3.format(*grid.cell[1, :]))
             self._write('  ' + _v3.format(*grid.cell[2, :]))
@@ -311,7 +317,7 @@ class xsfSile(Sile):
         for i, grid in enumerate(args):
             is_complex = np.iscomplexobj(grid.grid)
 
-            name = kwargs.get('grid' + str(i), str(i))
+            name = kwargs.get(f'grid{i}', str(i))
             if is_complex:
                 self._write(f' BEGIN_DATAGRID_3D_real_{name}\n')
             else:
@@ -376,7 +382,6 @@ class xsfSile(Sile):
 
         # We will add the vector data
         class VectorScale(argparse.Action):
-
             def __call__(self, parser, ns, value, option_string=None):
                 setattr(ns, '_vector_scale', float(value))
         p.add_argument('--vector-scale', '-sv', metavar='SCALE',
@@ -385,7 +390,6 @@ class xsfSile(Sile):
 
         # We will add the vector data
         class Vectors(argparse.Action):
-
             def __call__(self, parser, ns, values, option_string=None):
                 routine = values.pop(0)
 
