@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """
 Linear algebra
 ==============
@@ -21,5 +24,4 @@ like `numpy` and `scipy`.
 
 """
 from .base import *
-
-__all__ = [s for s in dir() if not s.startswith('_')]
+from .special import *
