@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
 
 import numpy as np
@@ -8,6 +11,7 @@ pytestmark = [pytest.mark.linalg, pytest.mark.eig]
 
 
 def test_solve1():
+    np.random.seed(1285947159)
     a = np.random.rand(10, 10)
     b = np.random.rand(10, 10)
     xs = sl.solve(a, b)
@@ -16,6 +20,7 @@ def test_solve1():
 
 
 def test_solve2():
+    np.random.seed(1285947159)
     a = np.random.rand(10, 10)
     ac = a.copy()
     b = np.random.rand(10)
@@ -29,6 +34,7 @@ def test_solve2():
 
 
 def test_solve3():
+    np.random.seed(1285947159)
     a = np.random.rand(10, 2)
     b = np.random.rand(10)
     with pytest.raises(ValueError):
@@ -36,6 +42,7 @@ def test_solve3():
 
 
 def test_solve4():
+    np.random.seed(1285947159)
     a = np.random.rand(10, 10)
     b = np.random.rand(10)
     xs = sl.solve(a, b)
