@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """ Module to expose messages to users
 
 These routines implement a layer of interaction with the user.
@@ -21,11 +24,12 @@ import warnings
 from functools import wraps
 
 from ._internal import set_module
+from ._environ import get_environ_variable
 
 
 __all__ = ['SislDeprecation', 'SislInfo', 'SislWarning', 'SislException', 'SislError']
 __all__ += ['warn', 'info', 'deprecate', "deprecate_method"]
-__all__ += ['tqdm_eta']
+__all__ += ['progressbar', 'tqdm_eta']
 
 # The local registry for warnings issued
 _sisl_warn_registry = {}
@@ -62,29 +66,36 @@ class SislInfo(SislWarning):
 
 
 @set_module("sisl")
-def deprecate(message):
+def deprecate(message, from_version=None):
     """ Issue sisl deprecation warnings
 
     Parameters
     ----------
     message : str
+       the displayed message
+    from_version : optional
+       which version to deprecate this method from
     """
+    if from_version is not None:
+        message = f"{message} [>={from_version}]"
     warnings.warn_explicit(message, SislDeprecation, 'dep', 0, registry=_sisl_warn_registry)
 
 
 @set_module("sisl")
-def deprecate_method(msg):
+def deprecate_method(message, from_version=None):
     """ Decorator for deprecating a method
 
     Parameters
     ----------
-    msg : str
+    message : str
        message displayed
+    from_version : optional
+       which version to deprecate this method from
     """
     def install_deprecate(func):
         @wraps(func)
         def wrapped(*args, **kwargs):
-            deprecate(msg)
+            deprecate(message, from_version)
             return func(*args, **kwargs)
         return wrapped
     return install_deprecate
@@ -195,28 +206,37 @@ except ImportError:
             _stdout.flush()
 
 
+_default_eta = get_environ_variable("SISL_SHOW_PROGRESS")
+
+
 @set_module("sisl")
-def tqdm_eta(count, desc, unit, eta):
-    """ Create a TQDM eta progress bar in when it is requested. Otherwise returns a fake object
+def progressbar(total, desc, unit, eta, **kwargs):
+    """ Create a progress bar in when it is requested. Otherwise returns a fake object
 
     Parameters
     ----------
-    count : int
+    total : int
        number of total iterations
     desc : str
        description on the stdout when running the progressbar
     unit : str
        unit shown in the progressbar
-    eta : bool
-       if True a ``tqdm`` progressbar is returned. Else a fake instance is returned.
+    eta : bool or str or None
+       if True a progressbar is returned (default from ``tqdm``). Else a fake instance is returned.
+       If a str, that will be used as the description.
+       If None, use SISL_SHOW_PROGRESS environment variable as the value
 
     Returns
     -------
     object
        progress bar if `eta` is true, otherwise an object which does nothing
     """
+    if eta is None:
+        eta = _default_eta
     if eta:
-        bar = _tqdm(total=count, desc=desc, unit=unit)
+        if isinstance(eta, str):
+            desc = eta
+        bar = _tqdm(total=total, desc=desc, unit=unit, **kwargs)
     else:
         # Since the eta bar is not needed we simply create a fake object which
         # has the required 2 methods, update and close.
@@ -228,3 +248,5 @@ def tqdm_eta(count, desc, unit, eta):
                 pass
         bar = Fake()
     return bar
+
+tqdm_eta = deprecate_method("Use sisl.messages.progress_bar instead", "0.13")(progressbar)
