@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 from functools import partial
 from numbers import Integral
 from math import pi
@@ -102,7 +105,7 @@ class Orbital:
     >>> orbq.q0
     1.
     """
-    __slots__ = ['_R', '_tag', '_q0']
+    __slots__ = ('_R', '_tag', '_q0')
 
     def __init__(self, R, q0=0., tag=''):
         """ Initialize orbital object """
@@ -110,6 +113,9 @@ class Orbital:
         self._q0 = float(q0)
         self._tag = tag
 
+    def __hash__(self):
+        return hash((self._R, self._q0, self._tag))
+
     @property
     def R(self):
         """ Maxmimum radius of orbital """
@@ -164,7 +170,10 @@ class Orbital:
         radial : bool, optional
            also compare that the radial parts are the same
         """
-        if not isinstance(other, Orbital):
+        if isinstance(other, str):
+            # just check for the same name
+            return self.name == other
+        elif not isinstance(other, Orbital):
             return False
         same = abs(self.R - other.R) <= 1e-4 and abs(self.q0 - other.q0) < 1e-4
         if not same:
@@ -239,7 +248,7 @@ class Orbital:
             # Plot data
             cax = axes.contourf(theta, phi, s, *args, **kwargs)
             cax.set_clim(s.min(), s.max())
-            cab = axes.get_figure().colorbar(cax)
+            axes.get_figure().colorbar(cax)
             axes.set_title(r'${}$'.format(self.name(True)))
             # I don't know how exactly to handle this...
             #axes.set_xlabel(r'Azimuthal angle $\theta$')
@@ -286,7 +295,7 @@ class Orbital:
         from .grid import Grid
         from .atom import Atom
         from .physics.electron import wavefunction
-        sc = SuperCell(R*2, origo=[-R] * 3)
+        sc = SuperCell(R*2, origin=[-R] * 3)
         if isinstance(atom, Atom):
             atom = atom.copy(orbitals=self)
         else:
@@ -353,19 +362,19 @@ class SphericalOrbital(Orbital):
     True
     """
     # Additional slots (inherited classes retain the same slots)
-    __slots__ = ['_l', 'f']
+    __slots__ = ('_l', 'f')
 
     def __init__(self, l, rf_or_func, q0=0., tag='', **kwargs):
         """ Initialize spherical orbital object """
         self._l = l
 
+        self._R = kwargs.pop('R', -1.)
         # Set the internal function
         if callable(rf_or_func):
             self.set_radial(rf_or_func, **kwargs)
-        elif rf_or_func is None:
+        elif rf_or_func is None or rf_or_func is NotImplemented:
             # We don't do anything
             self.f = NotImplemented
-            self._R = -1.
         else:
             # it must be two arguments
             self.set_radial(rf_or_func[0], rf_or_func[1], **kwargs)
@@ -375,6 +384,9 @@ class SphericalOrbital(Orbital):
         # maximum value in r.
         super().__init__(self.R, q0, tag)
 
+    def __hash__(self):
+        return hash((super(Orbital, self), self._l, self.f))
+
     @property
     def l(self):
         r""" :math:`l` quantum number """
@@ -553,7 +565,7 @@ class SphericalOrbital(Orbital):
         numpy.ndarray
             radial orbital value at point `r`
         """
-        r = _a.asarrayd(r).ravel()
+        r = _a.asarray(r).ravel()
         if is_radius:
             s = r.shape
         else:
@@ -588,7 +600,7 @@ class SphericalOrbital(Orbital):
         numpy.ndarray
              basis function value at point `r`
         """
-        r = _a.asarrayd(r)
+        r = _a.asarray(r)
         s = r.shape[:-1]
         # Convert to spherical coordinates
         n, idx, r, theta, phi = cart2spher(r, theta=m != 0, cos_phi=True, maxR=self.R)
@@ -678,8 +690,8 @@ class SphericalOrbital(Orbital):
         if m is None:
             m = range(-self.l, self.l + 1)
         elif isinstance(m, Integral):
-            return AtomicOrbital(n=n, l=self.l, m=m, zeta=zeta, P=P, spherical=self, q0=q0)
-        return [AtomicOrbital(n=n, l=self.l, m=mm, zeta=zeta, P=P, spherical=self, q0=q0) for mm in m]
+            return AtomicOrbital(n=n, l=self.l, m=m, zeta=zeta, P=P, spherical=self, q0=q0, R=self.R)
+        return [AtomicOrbital(n=n, l=self.l, m=mm, zeta=zeta, P=P, spherical=self, q0=q0, R=self.R) for mm in m]
 
     def __getstate__(self):
         """ Return the state of this object """
@@ -739,7 +751,7 @@ class AtomicOrbital(Orbital):
     #   P = polarization shell or not
     # orb is the SphericalOrbital class that retains the radial
     # grid and enables to calculate psi(r)
-    __slots__ = ['_n', '_l', '_m', '_zeta', '_P', '_orb']
+    __slots__ = ('_n', '_l', '_m', '_zeta', '_P', '_orb')
 
     def __init__(self, *args, **kwargs):
         """ Initialize atomic orbital object """
@@ -790,11 +802,11 @@ class AtomicOrbital(Orbital):
                     n = int(s[0])
                     # Remove n specification
                     s = s[1:]
-                except:
-                    n = _n.get(s[0])
+                except Exception:
+                    n = _n.get(s[0], n)
 
                 # Get l
-                l = _l.get(s[0])
+                l = _l.get(s[0], l)
 
                 # Get number of zeta shell
                 iZ = s.find('Z')
@@ -806,12 +818,12 @@ class AtomicOrbital(Orbital):
                         zeta = int(s[iZ+1])
                         # Remove Z + int
                         s = s[:iZ] + s[iZ+2:]
-                    except:
+                    except Exception:
                         zeta = 1
                         s = s[:iZ] + s[iZ+1:]
 
                 # We should be left with m specification
-                m = _m.get(s)
+                m = _m.get(s, m)
 
                 # Now we should figure out how the spherical orbital
                 # has been passed.
@@ -824,7 +836,7 @@ class AtomicOrbital(Orbital):
                     if isinstance(args[0], SphericalOrbital):
                         self._orb = args.pop(0)
                     else:
-                        self._orb = SphericalOrbital(l, args.pop(0))
+                        self._orb = SphericalOrbital(l, args.pop(0), q0=self.q0)
             else:
 
                 # Arguments *have* to be
@@ -833,11 +845,8 @@ class AtomicOrbital(Orbital):
                     n = args.pop(0)
                 if l is None and len(args) > 0:
                     l = args.pop(0)
-                if l > 0:
-                    if m is None and len(args) > 0:
-                        m = args.pop(0)
-                else:
-                    m = 0
+                if m is None and len(args) > 0:
+                    m = args.pop(0)
 
                 # Now we need to figure out if they are shell
                 # information or radial functions
@@ -853,11 +862,14 @@ class AtomicOrbital(Orbital):
                     if isinstance(args[0], SphericalOrbital):
                         self._orb = args.pop(0)
                     else:
-                        self._orb = SphericalOrbital(l, args.pop(0))
+                        self._orb = SphericalOrbital(l, args.pop(0), q0=self.q0)
 
         # Still if n is None, we assign the default (lowest) quantum number
         if n is None:
             n = l + 1
+        # Still if m is None, we assign the default value of 0
+        if m is None:
+            m = 0
 
         # Copy over information
         self._n = n
@@ -880,15 +892,19 @@ class AtomicOrbital(Orbital):
         elif isinstance(s, Orbital):
             self._orb = s
         else:
-            self._orb = SphericalOrbital(l, s)
+            self._orb = SphericalOrbital(l, s, q0=self.q0)
 
         if self._orb is None:
             # Default orbital to none, this will not create any radial functions
             # But any use of the orbital will still work
-            self._orb = Orbital(self.R)
+            self._orb = Orbital(self.R, q0=self.q0)
 
         self._R = self._orb.R
 
+    def __hash__(self):
+        return hash((super(Orbital, self), self._l, self._n, self._m,
+                     self._zeta, self._P, self._orb))
+
     @property
     def n(self):
         r""" :math:`n` shell """
@@ -992,8 +1008,8 @@ class AtomicOrbital(Orbital):
     def __str__(self):
         """ A string representation of the object """
         if len(self.tag) > 0:
-            return f'{self.__class__.__name__}{{{self.name()}, q0: {self.q0}, tag: {self.tag}, {str(self.orb)}}}'
-        return f'{self.__class__.__name__}{{{self.name()}, q0: {self.q0}, {str(self.orb)}}}'
+            return f'{self.__class__.__name__}{{{self.name()}, q0: {self.q0}, tag: {self.tag}, {self.orb!s}}}'
+        return f'{self.__class__.__name__}{{{self.name()}, q0: {self.q0}, {self.orb!s}}}'
 
     def __repr__(self):
         if self.tag:
