@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """
 Physical objects
 ================
@@ -83,6 +86,8 @@ Sparse matrices
    SparseOrbitalBZSpin - sparse orbital matrix with k-dependent properties and spin configuration
 
 """
+from ._feature import *
+
 from .distribution import *
 from .brillouinzone import *
 # Patch BrillouinZone objects and import apply classes
