@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 from numbers import Integral
 import math as m
 from scipy.sparse import csr_matrix, triu, tril
@@ -12,8 +15,7 @@ from sisl.supercell import SuperCell
 import sisl._array as _a
 from sisl._indices import indices_le, indices_fabs_le
 from sisl._math_small import xyz_to_spherical_cos_phi
-from sisl.messages import warn, tqdm_eta
-from sisl.utils.ranges import array_arange
+from sisl.messages import warn, progressbar
 from .spin import Spin
 from sisl.sparse import SparseCSR, _ncol_to_indptr
 from sisl.sparse_geometry import SparseOrbital
@@ -194,8 +196,7 @@ class _densitymatrix(SparseOrbitalBZSpin):
 
             # align with vector
             # add factor 1/2 here (instead when unwrapping)
-            A[:, :] = 0.5 * vec.reshape(1, 3) * ((A ** 2)
-                                                 .sum(1)
+            A[:, :] = 0.5 * vec.reshape(1, 3) * (np.sum(A ** 2, axis=1)
                                                  .reshape(-1, 1)) ** 0.5
 
             out = self.copy()
@@ -225,8 +226,7 @@ class _densitymatrix(SparseOrbitalBZSpin):
 
             # align with vector
             # add factor 1/2 here (instead when unwrapping)
-            A[:, :, :] = 0.5 * vec.reshape(1, 1, 3) * ((A ** 2)
-                                                       .sum(2)
+            A[:, :, :] = 0.5 * vec.reshape(1, 1, 3) * (np.sum(A ** 2, axis=2)
                                                        .reshape(-1, 2, 1)) ** 0.5
 
             out = self.copy()
@@ -341,7 +341,7 @@ class _densitymatrix(SparseOrbitalBZSpin):
             else:
                 D = self._csr.copy(range(self.shape[2] - 1))
                 D._D *= self._csr._D[:, -1].reshape(-1, 1)
-                D = D.sum(0)
+                D = np.sum(D, axis=1)
 
             return _convert(D)
 
@@ -352,7 +352,7 @@ class _densitymatrix(SparseOrbitalBZSpin):
             else:
                 D = self._csr.copy(range(self.shape[2] - 1))
                 D._D *= self._csr._D[:, -1].reshape(-1, 1)
-                D = D.sum(0)
+                D = np.sum(D, axis=1)
 
             # Now perform summation per atom
             geom = self.geometry
@@ -364,7 +364,7 @@ class _densitymatrix(SparseOrbitalBZSpin):
 
         raise NotImplementedError(f"{self.__class__.__name__}.mulliken only allows projection [orbital, atom]")
 
-    def density(self, grid, spinor=None, tol=1e-7, eta=False):
+    def density(self, grid, spinor=None, tol=1e-7, eta=None):
         r""" Expand the density matrix to the charge density on a grid
 
         This routine calculates the real-space density components on a specified grid.
@@ -409,7 +409,7 @@ class _densitymatrix(SparseOrbitalBZSpin):
             # use it in this routine
             # Otherwise we raise an ImportError
             unique([[0, 1], [2, 3]], axis=0)
-        except:
+        except Exception:
             raise NotImplementedError(f"{self.__class__.__name__}.density requires numpy >= 1.13, either update "
                                       "numpy or do not use this function!")
 
@@ -445,7 +445,7 @@ class _densitymatrix(SparseOrbitalBZSpin):
                 raise ValueError(f"{self.__class__.__name__}.density with NC/SO spin, requires a 2x2 matrix.")
 
             DM = _a.emptyz([self.nnz, 2, 2])
-            idx = array_arange(csr.ptr[:-1], n=csr.ncol)
+            idx = _a.array_arange(csr.ptr[:-1], n=csr.ncol)
             if self.spin.kind == Spin.NONCOLINEAR:
                 # non-collinear
                 DM[:, 0, 0] = csr._D[idx, 0]
@@ -478,11 +478,11 @@ class _densitymatrix(SparseOrbitalBZSpin):
                 raise ValueError(f"{self.__class__.__name__}.density with polarized spin, requires spinor "
                                  "argument as an integer, or a vector of length 2")
 
-            idx = array_arange(csr.ptr[:-1], n=csr.ncol)
+            idx = _a.array_arange(csr.ptr[:-1], n=csr.ncol)
             DM = csr._D[idx, 0] * spinor[0] + csr._D[idx, 1] * spinor[1]
 
         else:
-            idx = array_arange(csr.ptr[:-1], n=csr.ncol)
+            idx = _a.array_arange(csr.ptr[:-1], n=csr.ncol)
             DM = csr._D[idx, 0]
 
         # Create the DM csr matrix.
@@ -543,15 +543,15 @@ class _densitymatrix(SparseOrbitalBZSpin):
         # For extremely skewed lattices this will be way too much, hence we make
         # them square.
         o = sc.toCuboid(True)
-        sc = SuperCell(o._v + np.diag(2 * add_R), origo=o.origo - add_R)
+        sc = SuperCell(o._v + np.diag(2 * add_R), origin=o.origin - add_R)
 
         # Retrieve all atoms within the grid supercell
         # (and the neighbours that connect into the cell)
         IA, XYZ, ISC = geometry.within_inf(sc, periodic=pbc)
-        XYZ -= grid.sc.origo.reshape(1, 3)
+        XYZ -= grid.sc.origin.reshape(1, 3)
 
         # Retrieve progressbar
-        eta = tqdm_eta(len(IA), f"{self.__class__.__name__}.density", "atom", eta)
+        eta = progressbar(len(IA), f"{self.__class__.__name__}.density", "atom", eta)
 
         cell = geometry.cell
         atoms = geometry.atoms
@@ -613,13 +613,13 @@ class _densitymatrix(SparseOrbitalBZSpin):
         # The below complexity is because we are not finalizing spA
         csr = spA._csr
         a_ptr = _ncol_to_indptr(csr.ncol)
-        a_col = csr.col[array_arange(csr.ptr, n=csr.ncol)]
+        a_col = csr.col[_a.array_arange(csr.ptr, n=csr.ncol)]
         del spA, csr
 
         # Get offset in supercell in orbitals
         off = geometry.no * primary_i_s
-        origo = grid.origo
-        # TODO sum the non-origo atoms to the csrDM matrix
+        origin = grid.origin
+        # TODO sum the non-origin atoms to the csrDM matrix
         #      this would further decrease the loops required.
 
         # Loop over all atoms in the grid-cell
@@ -628,7 +628,7 @@ class _densitymatrix(SparseOrbitalBZSpin):
             ia_atom = atoms[ia]
             IO = a2o(ia)
             IO_range = range(ia_atom.no)
-            cell_offset = (cell * isc.reshape(3, 1)).sum(0) - origo
+            cell_offset = (cell * isc.reshape(3, 1)).sum(0) - origin
 
             # Extract maximum R
             R = ia_atom.maxR()
@@ -890,7 +890,7 @@ class DensityMatrix(_densitymatrix):
         aidx = geom.o2a(idx)
 
         # Sparse matrix indices for data
-        sidx = array_arange(csr.ptr[:-1], n=csr.ncol, dtype=np.int32)
+        sidx = _a.array_arange(csr.ptr[:-1], n=csr.ncol, dtype=np.int32)
         jdx = csr.col[sidx]
         ajdx = geom.o2a(jdx)
 
@@ -931,7 +931,7 @@ class DensityMatrix(_densitymatrix):
 
         def Lb(idx_l, DM, sub):
             if len(sub) == 0:
-                return
+                return []
             return (idx_l[sub] * (idx_l[sub] + 1) - 2) ** 0.5 * 0.5 * DM[sub]
 
         def Lc(idx, idx_l, DM, sub):
@@ -1044,7 +1044,7 @@ class DensityMatrix(_densitymatrix):
 
         if "orbital" == projection:
             return L
-        elif "atom" == projection:
+        if "atom" == projection:
             # Now perform summation per atom
             l = np.zeros([geom.na, 3], dtype=L.dtype)
             add.at(l, geom.o2a(np.arange(geom.no)), L)
