@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """Distribution functions
 =========================
 
@@ -51,19 +54,19 @@ def get_distribution(method, smearing=0.1, x0=0.):
         a function which accepts one argument
     """
     m = method.lower().replace('-', '_')
-    if m in ['gauss', 'gaussian']:
+    if m in ('gauss', 'gaussian'):
         return partial(gaussian, sigma=smearing, x0=x0)
-    elif m in ['lorentz', 'lorentzian']:
+    elif m in ('lorentz', 'lorentzian'):
         return partial(lorentzian, gamma=smearing, x0=x0)
-    elif m in ['fd', 'fermi', 'fermi_dirac']:
+    elif m in ('fd', 'fermi', 'fermi_dirac'):
         return partial(fermi_dirac, kT=smearing, mu=x0)
-    elif m in ['be', 'bose_einstein']:
+    elif m in ('be', 'bose_einstein'):
         return partial(bose_einstein, kT=smearing, mu=x0)
-    elif m in ['cold']:
+    elif m in ('cold'):
         return partial(cold, kT=smearing, mu=x0)
-    elif m in ['step', 'step_function']:
+    elif m in ('step', 'step_function'):
         return partial(step_function, x0=x0)
-    elif m in ['heavi', 'heavy', 'heaviside']:
+    elif m in ('heavi', 'heavy', 'heaviside'):
         return partial(heaviside, x0=x0)
     raise ValueError(f"get_distribution does not implement the {method} distribution function, have you mispelled?")
 
@@ -167,7 +170,7 @@ def bose_einstein(E, kT=0.1, mu=0.):
 
 @set_module("sisl.physics")
 def cold(E, kT=0.1, mu=0.):
-    r""" Cold smearing function, Marzari-Vanderbilt, PRL 82, 16, 1999
+    r""" Cold smearing function, see [1]_
 
     .. math::
         C(E,k_BT,\mu) = \frac12 &+ \mathrm{erf}\Big(-\frac{E-\mu}{k_BT}-\frac1{\sqrt2}\Big)
@@ -183,6 +186,10 @@ def cold(E, kT=0.1, mu=0.):
     mu : float, optional
         chemical potential
 
+    References
+    ----------
+    .. [1] :doi:`N. Marzari, D. Vanderbilt, A. de Vita, M.C. Payne "Thermal Contraction and Disordering of the Al(110) Surface", PRL **82**, 16 (1999) <10.1103/PhysRevLett.82.3296>`
+
     Returns
     -------
     numpy.ndarray
