@@ -1,7 +1,11 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import numpy as np
 from scipy.sparse import lil_matrix
 
 from sisl._internal import set_module
+from sisl.messages import deprecate_method
 from .sparse import SparseOrbitalBZ
 from .phonon import EigenvaluePhonon, EigenmodePhonon
 
@@ -341,15 +345,16 @@ class DynamicalMatrix(SparseOrbitalBZ):
             with get_sile(sile, 'w') as fh:
                 fh.write_dynamical_matrix(self, *args, **kwargs)
 
-    def velocity(self, k=(0, 0, 0), project=False, **kwargs):
+    @deprecate_method("use DynamicalMatrix.eigenstate(...).velocity() instead", "0.13.0")
+    def velocity(self, k=(0, 0, 0), matrix=False, **kwargs):
         r""" Calculate the velocity for the eigenmodes for a given `k` point
 
         Parameters
         ----------
         k : array_like, optional
             k-point at which the velocities are calculated
-        project : {}
-            see `sisl.physics.phonon.velocity` for details and possible arguments
+        matrix : bool, optional
+            see `EigenmodePhonon.velocity` for details
         **kwargs : optional
             additional parameters passed to the `eigenmode` routine
 
@@ -359,8 +364,9 @@ class DynamicalMatrix(SparseOrbitalBZ):
         displacement : Calculate mode displacements
         EigenmodePhonon.velocity : Underlying method used to calculate the velocity
         """
-        return self.eigenmode(k, **kwargs).velocity(project=project)
+        return self.eigenmode(k, **kwargs).velocity(matrix=matrix)
 
+    @deprecate_method("use DynamicalMatrix.eigenstate(...).displacement() instead", "0.13.0")
     def displacement(self, k=(0, 0, 0), **kwargs):
         r""" Calculate the displacement for the eigenmodes for a given `k` point
 
@@ -379,6 +385,7 @@ class DynamicalMatrix(SparseOrbitalBZ):
         """
         return self.eigenmode(k, **kwargs).displacement()
 
+    @deprecate_method("use DynamicalMatrix.eigenstate(...).DOS() instead", "0.13.0")
     def DOS(self, E, k=(0, 0, 0), distribution='gaussian', **kwargs):
         r""" Calculate the DOS at the given energies for a specific `k` point
 
@@ -403,6 +410,7 @@ class DynamicalMatrix(SparseOrbitalBZ):
         """
         return self.eigenvalue(k, **kwargs).DOS(E, distribution)
 
+    @deprecate_method("use DynamicalMatrix.eigenstate(...).PDOS() instead", "0.13.0")
     def PDOS(self, E, k=(0, 0, 0), distribution='gaussian', **kwargs):
         r""" Calculate the projected DOS at the given energies for a specific `k` point
 
