@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import numpy as np
 
 from sisl._internal import set_module
@@ -255,7 +258,7 @@ class EnergyDensityMatrix(_densitymatrix):
             # When the energy is zero, there is no shift
             return
 
-        for i in range(min(self.spin.spins, 2)):
+        for i in range(self.spin.spinor):
             self._csr._D[:, i] += DM._csr._D[:, i] * E[i]
 
     @staticmethod
