@@ -1,5 +1,7 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import numpy as np
-from scipy.interpolate import CubicSpline
 
 from sisl._internal import set_module
 import sisl._array as _a
@@ -255,12 +257,12 @@ class Hamiltonian(SparseOrbitalBZSpin):
 
         if self.orthogonal:
             for i in range(self.shape[0]):
-                for j in range(min(self.spin.spins, 2)):
+                for j in range(self.spin.spinor):
                     self[i, i, j] = self[i, i, j] + E[j]
         else:
             # For non-collinear and SO only the diagonal (real) components
             # should be shifted.
-            for i in range(min(self.spin.spins, 2)):
+            for i in range(self.spin.spinor):
                 self._csr._D[:, i] += self._csr._D[:, self.S_idx] * E[i]
 
     def eigenvalue(self, k=(0, 0, 0), gauge='R', **kwargs):
@@ -376,7 +378,7 @@ class Hamiltonian(SparseOrbitalBZSpin):
             with get_sile(sile, 'w') as fh:
                 fh.write_hamiltonian(self, *args, **kwargs)
 
-    @deprecate_method("Please use Hamiltonian.eigenstate(...).spin_squared() instead")
+    @deprecate_method("use Hamiltonian.eigenstate(...).spin_squared() instead", "0.13.0")
     def spin_squared(self, k=(0, 0, 0), n_up=None, n_down=None, **kwargs):
         r""" Calculate spin-squared expectation value, see `~sisl.physics.electron.spin_squared` for details
 
@@ -403,27 +405,27 @@ class Hamiltonian(SparseOrbitalBZSpin):
         # es_alpha.Sk should equal es_beta.Sk, so just pass one of them
         return spin_squared(es_alpha.state, es_beta.state, es_alpha.Sk())
 
-    @deprecate_method("Please use Hamiltonian.eigenstate(...).velocity() instead")
-    def velocity(self, k=(0, 0, 0), project=False, **kwargs):
+    @deprecate_method("use Hamiltonian.eigenstate(...).velocity() instead", "0.13.0")
+    def velocity(self, k=(0, 0, 0), matrix=False, **kwargs):
         r""" Calculate the velocity for the eigenstates for a given `k` point
 
         Parameters
         ----------
         k : array_like, optional
             k-point at which the velocities are calculated
-        project : {}
-            see `sisl.physics.electron.velocity` for details and possible arguments
+        matrix : bool, optional
+            see `EigenstateElectron.velocity` for details and possible arguments
         **kwargs : optional
             additional parameters passed to the `eigenstate` routine
 
         See Also
         --------
         eigenstate : method used to calculate the eigenstates
-        EigenvalueElectron.velocity : Underlying method used to calculate the velocity
+        EigenstateElectron.velocity : Underlying method used to calculate the velocity
         """
-        return self.eigenstate(k, **kwargs).velocity(project=project)
+        return self.eigenstate(k, **kwargs).velocity(matrix=matrix)
 
-    @deprecate_method("Please use Hamiltonian.eigenstate(...).spin_moment() instead")
+    @deprecate_method("use Hamiltonian.eigenstate(...).spin_moment() instead", "0.13.0")
     def spin_moment(self, k=(0, 0, 0), project=False, **kwargs):
         r""" Calculate the spin moment for the eigenstates for a given `k` point
 
@@ -443,7 +445,7 @@ class Hamiltonian(SparseOrbitalBZSpin):
         """
         return self.eigenstate(k, **kwargs).spin_moment()
 
-    @deprecate_method("Please use Hamiltonian.eigenstate(...).DOS() instead")
+    @deprecate_method("use Hamiltonian.eigenstate(...).DOS() instead", "0.13.0")
     def DOS(self, E, k=(0, 0, 0), distribution='gaussian', **kwargs):
         r""" Calculate the DOS at the given energies for a specific `k` point
 
@@ -468,7 +470,7 @@ class Hamiltonian(SparseOrbitalBZSpin):
         """
         return self.eigenvalue(k, **kwargs).DOS(E, distribution)
 
-    @deprecate_method("Please use Hamiltonian.eigenstate(...).PDOS() instead")
+    @deprecate_method("use Hamiltonian.eigenstate(...).PDOS() instead", "0.13.0")
     def PDOS(self, E, k=(0, 0, 0), distribution='gaussian', **kwargs):
         r""" Calculate the projected DOS at the given energies for a specific `k` point
 
