@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """Phonon related functions and classes
 =======================================
 
@@ -34,12 +37,13 @@ from numpy import conj, dot, fabs, exp, einsum
 from numpy import delete
 
 from sisl._internal import set_module
+from sisl.messages import deprecate_method
 import sisl._array as _a
 from sisl import units, constant
 from sisl._help import dtype_complex_to_real
-from .state import Coefficient, State, StateC
+from .state import degenerate_decouple, Coefficient, State, StateC
 
-from .electron import _decouple_eigh
+from .distribution import get_distribution
 from .electron import DOS as electron_DOS
 from .electron import PDOS as electron_PDOS
 
@@ -127,7 +131,8 @@ def PDOS(E, mode, hw, distribution='gaussian'):
 
 
 @set_module("sisl.physics.phonon")
-def velocity(mode, hw, dDk, degenerate=None, project=False):
+@deprecate_method("use DynamicalMatrix.eigenstate(...).velocity() instead", "0.13.0")
+def velocity(mode, hw, dDk, degenerate=None, degenerate_dir=(1, 1, 1), project=False):
     r""" Calculate the velocity of a set of modes
 
     These are calculated using the analytic expression (:math:`\alpha` corresponding to the Cartesian directions):
@@ -150,9 +155,15 @@ def velocity(mode, hw, dDk, degenerate=None, project=False):
     degenerate : list of array_like, optional
        a list containing the indices of degenerate modes. In that case a prior diagonalization
        is required to decouple them. This is done 3 times along each of the Cartesian directions.
+    degenerate_dir : (3,), optional
+       a direction used for degenerate decoupling. The decoupling based on the velocity along this direction
     project : bool, optional
        if true, velocities will be returned projected per mode component
 
+    See Also
+    --------
+    DynamicalMatrix.dDk : function for generating the dynamical matrix derivatives (`dDk` argument)
+
     Returns
     -------
     numpy.ndarray
@@ -163,9 +174,8 @@ def velocity(mode, hw, dDk, degenerate=None, project=False):
         Units *may* change in future releases.
     """
     if mode.ndim == 1:
-        return velocity(mode.reshape(1, -1), hw, dDk, degenerate, project)[0]
-
-    return _velocity(mode, hw, dDk, degenerate, project)
+        return velocity(mode.reshape(1, -1), hw, dDk, degenerate, degenerate_dir, project)[0]
+    return _velocity(mode, hw, dDk, degenerate, degenerate_dir, project)
 
 
 # dDk is in [Ang * eV ** 2]
@@ -173,10 +183,13 @@ def velocity(mode, hw, dDk, degenerate=None, project=False):
 _velocity_const = units('ps', 's') / constant.hbar('eV s')
 
 
-def _velocity(mode, hw, dDk, degenerate, project):
+def _velocity(mode, hw, dDk, degenerate, degenerate_dir, project):
     r""" For modes in an orthogonal basis """
     # Decouple the degenerate modes
     if not degenerate is None:
+        degenerate_dir = _a.asarrayd(degenerate_dir)
+        degenerate_dir /= (degenerate_dir ** 2).sum() ** 0.5
+        deg_dDk = sum(d*dd for d, dd in zip(degenerate_dir, dDk))
         for deg in degenerate:
             # Set the average frequency
             hw[deg] = np.average(hw[deg])
@@ -184,12 +197,8 @@ def _velocity(mode, hw, dDk, degenerate, project):
             # Now diagonalize to find the contributions from individual modes
             # then re-construct the seperated degenerate modes
             # Since we do this for all directions we should decouple them all
-            vv = conj(mode[deg, :]).dot(dDk[0].dot(mode[deg, :].T))
-            S = _decouple_eigh(vv).dot(mode[deg, :])
-            vv = conj(S).dot((dDk[1]).dot(S.T))
-            S = _decouple_eigh(vv).dot(S)
-            vv = conj(S).dot((dDk[2]).dot(S.T))
-            mode[deg, :] = _decouple_eigh(vv).dot(S)
+            mode[deg] = degenerate_decouple(mode[deg], deg_dDk)
+        del deg_dDk
 
     cm = conj(mode)
     if project:
@@ -270,55 +279,6 @@ def _displacement(mode, hw, mass):
     return U
 
 
-class _phonon_Mode:
-    __slots__ = []
-
-    @property
-    def mode(self):
-        """ Eigenmodes (states) """
-        return self.state
-
-    def change_gauge(self, gauge):
-        r""" In-place change of the gauge of the mode coefficients
-
-        The two gauges are related through:
-
-        .. math::
-
-            \tilde C_j = e^{i\mathbf k\mathbf r_j} C_j
-
-        where :math:`C_j` and :math:`\tilde C_j` belongs to the ``r`` and ``R`` gauge, respectively.
-
-        Parameters
-        ----------
-        gauge : {'R', 'r'}
-            specify the new gauge for the mode coefficients
-        """
-        # These calls will fail if the gauge is not specified.
-        # In that case it will not do anything
-        if self.info.get('gauge', gauge) == gauge:
-            # Quick return
-            return
-
-        # Update gauge value
-        self.info['gauge'] = gauge
-
-        # Check that we can do a gauge transformation
-        k = _a.asarrayd(self.info.get('k'))
-        if k.dot(k) <= 0.000000001:
-            return
-
-        g = self.parent.geometry
-        phase = dot(g.xyz[g.o2a(_a.arangei(g.no)), :], dot(k, g.rcell))
-
-        if gauge == 'r':
-            # R -> r gauge tranformation.
-            self.state *= exp(-1j * phase).reshape(1, -1)
-        elif gauge == 'R':
-            # r -> R gauge tranformation.
-            self.state *= exp(1j * phase).reshape(1, -1)
-
-
 @set_module("sisl.physics.phonon")
 class CoefficientPhonon(Coefficient):
     """ Coefficients describing some physical quantity related to phonons """
@@ -326,46 +286,52 @@ class CoefficientPhonon(Coefficient):
 
 
 @set_module("sisl.physics.phonon")
-class ModePhonon(_phonon_Mode, State):
+class ModePhonon(State):
     """ A mode describing a physical quantity related to phonons """
     __slots__ = []
 
+    @property
+    def mode(self):
+        """ Eigenmodes (states) """
+        return self.state
+
 
 @set_module("sisl.physics.phonon")
-class ModeCPhonon(_phonon_Mode, StateC):
+class ModeCPhonon(StateC):
     """ A mode describing a physical quantity related to phonons, with associated coefficients of the mode """
     __slots__ = []
 
-    def velocity(self, eps=1e-7, project=False):
-        r""" Calculate velocity for the modes
+    @property
+    def mode(self):
+        """ Eigenmodes (states) """
+        return self.state
 
-        This routine calls `~sisl.physics.phonon.velocity` with appropriate arguments
+    def velocity(self, *args, **kwargs):
+        r""" Calculate velocity of the modes
+
+        This routine calls `derivative` with appropriate arguments (1st order derivative)
         and returns the velocity for the modes.
 
         Note that the coefficients associated with the `ModeCPhonon` *must* correspond
         to the energies of the modes.
 
-        See `~sisl.physics.phonon.velocity` for details.
+        See `derivative` for details and possible arguments. One cannot pass the ``order`` argument
+        as that is fixed to ``1`` in this call.
 
         Notes
         -----
-        The eigenvectors for the modes *may* have changed after calling this routine.
+        The states and energies for the modes *may* have changed after calling this routine.
         This is because of the velocity un-folding for degenerate modes. I.e. calling
         `displacement` and/or `PDOS` after this method *may* change the result.
 
-        Parameters
-        ----------
-        eps : float, optional
-           precision used to find degenerate modes.
-        project: bool, optional
+        See Also
+        --------
+        derivative : for details of the implementation
         """
-        opt = {'k': self.info.get('k', (0, 0, 0))}
-        gauge = self.info.get('gauge', None)
-        if not gauge is None:
-            opt['gauge'] = gauge
-
-        deg = self.degenerate(eps)
-        return velocity(self.mode, self.hw, self.parent.dDk(**opt), degenerate=deg, project=project)
+        d = self.derivative(1, *args, **kwargs).real
+        axes = tuple(i for i in range(1, d.ndim))
+        c = np.expand_dims(self.c, axis=axes) * 2 / _velocity_const
+        return np.divide(d, c, where=(c != 0))
 
 
 @set_module("sisl.physics.phonon")
