@@ -1,10 +1,13 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 from itertools import product
 import pytest
 import numpy as np
 
 from sisl import Atom, geom, Hamiltonian, Bloch
 
-pytestmark = pytest.mark.bloch
+pytestmark = [pytest.mark.physics, pytest.mark.bloch]
 
 
 def get_H():
