@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
 
 import math as m
@@ -65,6 +68,7 @@ def setup():
     return t()
 
 
+@pytest.mark.physics
 @pytest.mark.density_matrix
 class TestDensityMatrix:
 
@@ -111,7 +115,7 @@ class TestDensityMatrix:
         D[1, 2] = (2., 0.5)
         mulliken = D.mulliken('orbital')
         assert np.allclose(mulliken[:2, 0], [1., 3.])
-        assert mulliken.sum() == pytest.approx(4.)
+        assert mulliken.sum() == pytest.approx(4)
         mulliken = D.mulliken('atom')
         assert mulliken[0, 0] == pytest.approx(4)
         assert mulliken.sum() == pytest.approx(4)
@@ -122,6 +126,7 @@ class TestDensityMatrix:
         grid = Grid(0.2, geometry=setup.D.geometry)
         D.density(grid)
 
+    @pytest.mark.filterwarnings("ignore", message="*is NOT Hermitian for on-site")
     def test_rho2(self, setup):
         bond = 1.42
         sq3h = 3.**.5 * 0.5
@@ -165,6 +170,7 @@ class TestDensityMatrix:
         D.density(grid, Spin.Y)
         D.density(grid, Spin.Z)
 
+    @pytest.mark.filterwarnings("ignore", message="*is NOT Hermitian for on-site")
     def test_orbital_momentum(self, setup):
         bond = 1.42
         sq3h = 3.**.5 * 0.5
@@ -226,6 +232,7 @@ class TestDensityMatrix:
         assert not np.allclose(D_mull, d_mull)
         assert np.allclose(D_mull[:, 0], d_mull[:, 0])
 
+    @pytest.mark.filterwarnings("ignore", message="*is NOT Hermitian for on-site")
     def test_spin_align_so(self, setup):
         bond = 1.42
         sq3h = 3.**.5 * 0.5
@@ -289,6 +296,7 @@ class TestDensityMatrix:
         assert not np.allclose(D_mull, d_mull)
         assert np.allclose(D_mull[:, 0], d_mull[:, 0])
 
+    @pytest.mark.filterwarnings("ignore", message="*is NOT Hermitian for on-site")
     def test_spin_rotate_so(self, setup):
         bond = 1.42
         sq3h = 3.**.5 * 0.5
@@ -374,3 +382,52 @@ class TestDensityMatrix:
         d = p.loads(s)
         assert d.spsame(D)
         assert np.allclose(d.eigh(), D.eigh())
+
+    def test_transform(self, setup):
+        D = DensityMatrix(setup.g, spin='so')
+        a = np.arange(8)
+        for ia in setup.g:
+            D[ia, ia] = a
+        Dcsr = [D.tocsr(i) for i in range(D.shape[2])]
+
+        Dt = D.transform(spin='unpolarized', dtype=np.float32)
+        assert np.abs(0.5 * Dcsr[0] + 0.5 * Dcsr[1] - Dt.tocsr(0)).sum() == 0
+
+        Dt = D.transform(spin='polarized', orthogonal=False)
+        assert np.abs(Dcsr[0] - Dt.tocsr(0)).sum() == 0
+        assert np.abs(Dcsr[1] - Dt.tocsr(1)).sum() == 0
+        assert np.abs(Dt.tocsr(2)).sum() != 0
+
+        Dt = D.transform(spin='non-colinear', orthogonal=False)
+        assert np.abs(Dcsr[0] - Dt.tocsr(0)).sum() == 0
+        assert np.abs(Dcsr[1] - Dt.tocsr(1)).sum() == 0
+        assert np.abs(Dcsr[2] - Dt.tocsr(2)).sum() == 0
+        assert np.abs(Dcsr[3] - Dt.tocsr(3)).sum() == 0
+        assert np.abs(Dt.tocsr(-1)).sum() != 0
+
+    def test_transform_nonortho(self, setup):
+        D = DensityMatrix(setup.g, spin='polarized', orthogonal=False)
+        a = np.arange(3)
+        a[-1] = 1.
+        for ia in setup.g:
+            D[ia, ia] = a
+
+        Dt = D.transform(spin='unpolarized', dtype=np.float32)
+        assert np.abs(0.5 * D.tocsr(0) + 0.5 * D.tocsr(1) - Dt.tocsr(0)).sum() == 0
+        assert np.abs(D.tocsr(-1) - Dt.tocsr(-1)).sum() == 0
+        Dt = D.transform(spin='polarized')
+        assert np.abs(D.tocsr(0) - Dt.tocsr(0)).sum() == 0
+        assert np.abs(D.tocsr(1) - Dt.tocsr(1)).sum() == 0
+        Dt = D.transform(spin='polarized', orthogonal=True)
+        assert np.abs(D.tocsr(0) - Dt.tocsr(0)).sum() == 0
+        assert np.abs(D.tocsr(1) - Dt.tocsr(1)).sum() == 0
+        Dt = D.transform(spin='non-colinear', orthogonal=False)
+        assert np.abs(D.tocsr(0) - Dt.tocsr(0)).sum() == 0
+        assert np.abs(D.tocsr(1) - Dt.tocsr(1)).sum() == 0
+        assert np.abs(Dt.tocsr(2)).sum() == 0
+        assert np.abs(Dt.tocsr(3)).sum() == 0
+        assert np.abs(D.tocsr(-1) - Dt.tocsr(-1)).sum() == 0
+        Dt = D.transform(spin='so', orthogonal=True)
+        assert np.abs(D.tocsr(0) - Dt.tocsr(0)).sum() == 0
+        assert np.abs(D.tocsr(1) - Dt.tocsr(1)).sum() == 0
+        assert np.abs(Dt.tocsr(-1)).sum() == 0
