@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
 
 import math as m
@@ -5,7 +8,7 @@ import numpy as np
 
 from sisl.physics.distribution import *
 
-pytestmark = pytest.mark.distribution
+pytestmark = [pytest.mark.physics, pytest.mark.distribution]
 
 
 def test_distribution1():
