@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
 
 import math as m
@@ -6,7 +9,7 @@ import numpy as np
 from sisl import Geometry, Atom, SuperCell, DynamicalMatrix
 
 
-pytestmark = pytest.mark.dynamicalmatrix
+pytestmark = [pytest.mark.physics, pytest.mark.dynamicalmatrix]
 
 
 @pytest.fixture
@@ -105,6 +108,7 @@ class TestDynamicalMatrix:
         em2.change_gauge('R')
         assert np.allclose(em.mode, em2.mode)
 
+    @pytest.mark.filterwarnings('ignore', category=np.ComplexWarning)
     def test_dos_pdos_velocity(self, setup):
         D = setup.D.copy()
         D.construct(setup.func)
@@ -115,7 +119,7 @@ class TestDynamicalMatrix:
         assert np.allclose(D.PDOS(E), em.PDOS(E))
         assert np.allclose(D.displacement(), em.displacement())
         assert np.allclose(D.velocity(), em.velocity())
-        assert np.allclose(D.velocity(project=True).sum(1), em.velocity())
+        assert np.allclose(np.diagonal(D.velocity(matrix=True)).T, em.velocity())
 
     def test_pickle(self, setup):
         import pickle as p
