@@ -1,9 +1,13 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
 
 from functools import partial
 import warnings
 import numpy as np
 from scipy.linalg import block_diag
+from scipy.sparse import SparseEfficiencyWarning
 
 from sisl import Geometry, Atom, SuperCell, Hamiltonian, Spin, BandStructure, MonkhorstPack, BrillouinZone
 from sisl import get_distribution
@@ -12,7 +16,8 @@ from sisl import Grid, SphericalOrbital, SislError
 from sisl.physics.electron import berry_phase, spin_squared, conductivity
 
 
-pytestmark = pytest.mark.hamiltonian
+pytestmark = [pytest.mark.physics, pytest.mark.hamiltonian,
+              pytest.mark.filterwarnings("ignore", category=SparseEfficiencyWarning)]
 
 
 @pytest.fixture
@@ -41,13 +46,6 @@ def setup():
     return t()
 
 
-def _to_voight(m):
-    idx1 = [0, 1, 2, 1, 0, 0]
-    idx2 = [0, 1, 2, 2, 2, 1]
-    return m[:, idx1, idx2]
-
-
-@pytest.mark.hamiltonian
 class TestHamiltonian:
 
     def test_objects(self, setup):
@@ -489,7 +487,7 @@ class TestHamiltonian:
         h = 1.j ** H
         assert h.dtype == np.complex128
 
-    def test_cut1(self, setup):
+    def test_untile1(self, setup):
         # Test of eigenvalues using a cut
         # Hamiltonian
         R, param = [0.1, 1.5], [1., 0.1]
@@ -501,14 +499,14 @@ class TestHamiltonian:
         H = Hamiltonian(g)
         H.construct([R, param])
         # Create cut Hamiltonian
-        Hc = H.cut(2, 1).cut(2, 0)
+        Hc = H.untile(2, 1).untile(2, 0)
         eigc = Hc.eigh()
         eigg = Hg.eigh()
         assert np.allclose(eigc, eigg)
         assert np.allclose(Hg.eigh(), Hc.eigh())
         del Hc, H
 
-    def test_cut2(self, setup):
+    def test_untile2(self, setup):
         # Test of eigenvalues using a cut
         # Hamiltonian
         R, param = [0.1, 1.5], [(1., 1.), (0.1, 0.1)]
@@ -521,7 +519,7 @@ class TestHamiltonian:
         H = Hamiltonian(g, orthogonal=False)
         H.construct([R, param])
         # Create cut Hamiltonian
-        Hc = H.cut(2, 1).cut(2, 0)
+        Hc = H.untile(2, 1).untile(2, 0)
         eigc = Hc.eigh()
         eigg = Hg.eigh()
         assert np.allclose(Hg.eigh(), Hc.eigh())
@@ -575,6 +573,7 @@ class TestHamiltonian:
         assert np.allclose(eigs, eig2)
         setup.HS.empty()
 
+    @pytest.mark.filterwarnings("ignore", message="*uses an overlap matrix that")
     def test_eig4(self, setup):
         # Test of eigenvalues vs eigenstate class
         HS = setup.HS.copy()
@@ -586,16 +585,48 @@ class TestHamiltonian:
             assert np.allclose(e, es.eig)
             assert np.allclose(v, es.state.T)
             assert np.allclose(es.norm2(), 1)
-            assert np.allclose(es.inner(diagonal=False) - np.eye(len(es)), 0)
+            # inner is not norm2
+            assert not np.allclose(es.inner(diag=False), np.eye(len(es)))
 
-            assert es.inner(es.sub(0)).shape == (1, )
-            assert es.inner(es.sub(0), diagonal=False).shape == (len(es), 1)
+            with pytest.raises(ValueError):
+                es.inner(es.sub(0))
+
+            assert es.inner(es.sub(0), diag=False).shape == (len(es), 1)
 
             eig1 = HS.eigh(k)
             eig2 = np.sort(HS.eig(k).real)
             eig3 = np.sort(HS.eig(k, eigvals_only=False)[0].real)
+            eig4 = es.inner(matrix=HS.Hk(k))
+            eig5 = es.inner(ket=es, matrix=HS.Hk(k))
             assert np.allclose(eig1, eig2, atol=1e-5)
             assert np.allclose(eig1, eig3, atol=1e-5)
+            assert np.allclose(eig1, eig4, atol=1e-5)
+            assert np.allclose(eig1, eig5, atol=1e-5)
+
+            assert es.inner(matrix=HS.Hk([0.1] * 3), ket=HS.eigenstate([0.3] * 3),
+                            diag=False).shape == (len(es), len(es))
+
+    @pytest.mark.filterwarnings("ignore", message="*uses an overlap matrix that")
+    def test_inner(self, setup):
+        HS = setup.HS.copy()
+        HS.construct([(0.1, 1.5), ((2., 1.), (3., 0.))])
+        HS = HS.tile(2, 0).tile(2, 1)
+
+        es1 = HS.eigenstate([0.1] * 3)
+        es2 = HS.eigenstate([0.2] * 3)
+        m1 = es1.inner(es2, diag=False)
+        assert m1.shape == (len(es1), len(es2))
+        m2 = es2.inner(es1, diag=False)
+        assert np.allclose(m1, m2.conj().T)
+        m3 = es2.inner(es1.state, diag=False)
+        assert np.allclose(m1, m3.conj().T)
+
+        r = range(3)
+        m1 = es1.sub(r).inner(es2, diag=False)
+        m2 = es2.inner(es1.sub(r), diag=False)
+        assert np.allclose(m1, m2.conj().T)
+        assert es1.sub(r).inner(es2, diag=False).shape == (len(r), len(es2))
+        assert es1.inner(es2.sub(r), diag=False).shape == (len(es1), len(r))
 
     def test_gauge_eig(self, setup):
         # Test of eigenvalues
@@ -615,6 +646,79 @@ class TestHamiltonian:
         assert np.allclose(es1.eig, es2.eig)
         assert not np.allclose(es1.state, es2.state)
 
+    def test_eigenstate_ipr(self, setup):
+        # Test of eigenvalues
+        R, param = [0.1, 1.5], [1., 0.1]
+        g = setup.g.tile(2, 0).tile(2, 1).tile(2, 2)
+        H = Hamiltonian(g)
+        H.construct((R, param))
+
+        k = [0.1] * 3
+        es = H.eigenstate(k)
+        ipr = es.ipr()
+        assert ipr.shape == (len(es),)
+
+    def test_eigenstate_tile(self, setup):
+        # Test of eigenvalues
+        R, param = [0.1, 1.5], [0., 2.7]
+        H1 = setup.H.copy()
+        H1.construct((R, param))
+        H2 = H1.tile(2, 1)
+
+        k = [0] * 3
+        # we must select a k that does not fold on
+        # itself which then creates degenerate states
+        for k1 in [0.5, 1/3]:
+            k[1] = k1
+            es1 = H1.eigenstate(k)
+            es1_2 = es1.tile(2, 1, normalize=True)
+            es2 = H2.eigenstate(es1_2.info['k'])
+
+            # we need to check that these are somewhat the same
+            out = es1_2.inner(es2, diag=False)
+            abs_out = np.absolute(out)
+            assert np.isclose(abs_out, 1).sum() == len(es1)
+
+    def test_eigenstate_tile_offset(self, setup):
+        # Test of eigenvalues
+        R, param = [0.1, 1.5], [0., 2.7]
+        H1 = setup.H.copy()
+        H1.construct((R, param))
+        H2 = H1.tile(2, 1)
+
+        k = [0] * 3
+        # we must select a k that does not fold on
+        # itself which then creates degenerate states
+        for k1 in [0.5, 1/3]:
+            k[1] = k1
+            es1 = H1.eigenstate(k)
+            es1_2 = es1.tile(2, 1, normalize=True, offset=1)
+            es2 = H2.eigenstate(es1_2.info['k']).translate([0, 1, 0])
+
+            # we need to check that these are somewhat the same
+            out = es1_2.inner(es2, diag=False)
+            abs_out = np.absolute(out)
+            assert np.isclose(abs_out, 1).sum() == len(es1)
+
+    def test_eigenstate_translate(self, setup):
+        # Test of eigenvalues
+        R, param = [0.1, 1.5], [0., 2.7]
+        H = setup.H.copy()
+        H.construct((R, param))
+
+        k = [0] * 3
+        # we must select a k that does not fold on
+        # itself which then creates degenerate states
+        for k1 in [0.5, 1/3]:
+            k[1] = k1
+            es1 = H.eigenstate(k)
+            es1_2 = es1.tile(2, 1)
+            es2 = es1.translate([0, 1, 0])
+            assert np.allclose(es1_2.state[:, :len(es1)],
+                               es1.state)
+            assert np.allclose(es1_2.state[:, len(es1):],
+                               es2.state)
+
     def test_gauge_velocity(self, setup):
         R, param = [0.1, 1.5], [1., 0.1]
         g = setup.g.tile(2, 0).tile(2, 1).tile(2, 2)
@@ -622,9 +726,16 @@ class TestHamiltonian:
         H.construct((R, param))
 
         k = [0.1] * 3
+        # This is a degenerate eigenstate:
+        #  2, 2, 4, 4, 2, 2
+        # and hence a decoupling is necessary
+        # This test is the reason why a default degenerate=1e-5 is used
+        # since the gauge='R' yields the correct *decoupled* states
+        # where as gauge='r' mixes them in a bad way.
         es1 = H.eigenstate(k, gauge='R')
         es2 = H.eigenstate(k, gauge='r')
         assert np.allclose(es1.velocity(), es2.velocity())
+        assert np.allclose(es1.velocity(), es2.velocity(degenerate_dir=(1, 1, 0)))
 
         es2.change_gauge('R')
         assert np.allclose(es1.velocity(), es2.velocity())
@@ -636,15 +747,34 @@ class TestHamiltonian:
         assert np.allclose(v1, v2)
 
         # Projected velocity
-        pv1 = es1.velocity(project=True)
-        pv2 = es2.velocity(project=True)
-        assert np.allclose(pv1.sum(1), v2)
-        assert np.allclose(pv2.sum(1), v1)
-        # since degenerate states *could* swap states
-        # we can't for sure compare states
-        # This test is one of those cases
-        # hence the below is disabled
-        #assert np.allclose(pv1, pv2)
+        vv1 = es1.velocity(matrix=True)
+        vv2 = es2.velocity(matrix=True)
+        assert np.allclose(np.diagonal(vv1).T, v2)
+        assert np.allclose(np.diagonal(vv2).T, v1)
+
+    def test_derivative_orthogonal(self, setup):
+        R, param = [0.1, 1.5], [1., 0.1]
+        g = setup.g.tile(2, 0).tile(2, 1).tile(2, 2)
+        H = Hamiltonian(g)
+        H.construct((R, param))
+
+        k = [0.1] * 3
+        es = H.eigenstate()
+        v1, vv1 = es.derivative(2)
+        v = es.derivative(1)
+        assert np.allclose(v1, v)
+
+    def test_derivative_non_orthogonal(self, setup):
+        R, param = [0.1, 1.5], [(1., 1.), (0.1, 0.1)]
+        g = setup.g.tile(2, 0).tile(2, 1).tile(2, 2)
+        H = Hamiltonian(g, orthogonal=False)
+        H.construct((R, param))
+
+        k = [0.1] * 3
+        es = H.eigenstate()
+        v1, vv1 = es.derivative(2)
+        v = es.derivative(1)
+        assert np.allclose(v1, v)
 
     def test_berry_phase(self, setup):
         R, param = [0.1, 1.5], [1., 0.1]
@@ -654,7 +784,7 @@ class TestHamiltonian:
         bz = BandStructure.param_circle(H, 20, 0.01, [0, 0, 1], [1/3] * 3)
         berry_phase(bz)
         berry_phase(bz, sub=0)
-        berry_phase(bz, eigvals=True, sub=0)
+        berry_phase(bz, eigvals=True, sub=0, method='berry:svd')
 
     def test_berry_phase_fail_sc(self, setup):
         g = setup.g.tile(2, 0).tile(2, 1).tile(2, 2)
@@ -670,32 +800,95 @@ class TestHamiltonian:
         bz2 = BandStructure.param_circle(H, 20, 0.01, [0, 0, 1], [1/3] * 3, loop=True)
         assert np.allclose(berry_phase(bz1), berry_phase(bz2))
 
-    def test_berry_phase_zak(self):
+    def test_berry_phase_non_orthogonal(self, setup):
+        R, param = [0.1, 1.5], [(1., 1.), (0.1, 0.1)]
+        g = setup.g.tile(2, 0).tile(2, 1).tile(2, 2)
+        H = Hamiltonian(g, orthogonal=False)
+        H.construct((R, param))
+
+        bz = BandStructure.param_circle(H, 20, 0.01, [0, 0, 1], [1/3] * 3)
+        berry_phase(bz)
+
+    def test_berry_phase_zak_x_topological(self):
         # SSH model, topological cell
-        g = Geometry([[-.6, 0, 0], [0.6, 0, 0]], Atom(1, 1.001), sc=[2, 10, 10])
+        # |t2| < |t1|
+        g = Geometry([[0, 0, 0], [1.2, 0, 0]], Atom(1, 1.001), sc=[2, 10, 10])
         g.set_nsc([3, 1, 1])
         H = Hamiltonian(g)
         H.construct([(0.1, 1.0, 1.5), (0, 1., 0.5)])
         # Contour
-        k = np.linspace(0.0, 1.0, 101)
-        K = np.zeros([k.size, 3])
-        K[:, 0] = k
-        bz = BrillouinZone(H, K)
+        def func(parent, N, i):
+            return [i/N, 0, 0]
+        bz = BrillouinZone.parametrize(H, func, 101)
         assert np.allclose(np.abs(berry_phase(bz, sub=0, method='zak')), np.pi)
         # Just to do the other branch
         berry_phase(bz, method='zak')
 
+    def test_berry_phase_zak_x_topological_non_orthogonal(self):
+        # SSH model, topological cell
+        # |t2| < |t1|
+        g = Geometry([[0, 0, 0], [1.2, 0, 0]], Atom(1, 1.001), sc=[2, 10, 10])
+        g.set_nsc([3, 1, 1])
+        H = Hamiltonian(g, orthogonal=False)
+        H.construct([(0.1, 1.0, 1.5), ((0, 1), (1., 0.25), (0.5, 0.1))])
+        # Contour
+        def func(parent, N, i):
+            return [i/N, 0, 0]
+        bz = BrillouinZone.parametrize(H, func, 101)
+        assert np.allclose(np.abs(berry_phase(bz, sub=0, method='zak')), np.pi)
+        # Just to do the other branch
+        berry_phase(bz, method='zak')
+
+    def test_berry_phase_zak_x_trivial(self):
+        # SSH model, trivial cell
+        # |t2| > |t1|
+        g = Geometry([[0, 0, 0], [1.2, 0, 0]], Atom(1, 1.001), sc=[2, 10, 10])
+        g.set_nsc([3, 1, 1])
+        H = Hamiltonian(g)
+        H.construct([(0.1, 1.0, 1.5), (0, 0.5, 1.)])
+        # Contour
+        def func(parent, N, i):
+            return [i/N, 0, 0]
+        bz = BrillouinZone.parametrize(H, func, 101)
+        assert np.allclose(np.abs(berry_phase(bz, sub=0, method='zak')), 0.)
+        # Just to do the other branch
+        berry_phase(bz, method='zak')
+
+    def test_berry_phase_zak_y(self):
+        # SSH model, topological cell
+        g = Geometry([[0, -.6, 0], [0, 0.6, 0]], Atom(1, 1.001), sc=[10, 2, 10])
+        g.set_nsc([1, 3, 1])
+        H = Hamiltonian(g)
+        H.construct([(0.1, 1.0, 1.5), (0, 1., 0.5)])
+        # Contour
+        def func(parent, N, i):
+            return [0, i/N, 0]
+        bz = BrillouinZone.parametrize(H, func, 101)
+        assert np.allclose(np.abs(berry_phase(bz, sub=0, method='zak')), np.pi)
+        # Just to do the other branch
+        berry_phase(bz, method='zak')
+
+    def test_berry_phase_zak_offset(self):
+        # SSH model, topological cell
+        g = Geometry([[0., 0, 0], [1.2, 0, 0]], Atom(1, 1.001), sc=[2, 10, 10])
+        g.set_nsc([3, 1, 1])
+        H = Hamiltonian(g)
+        H.construct([(0.1, 1.0, 1.5), (0, 1., 0.5)])
+        # Contour
+        def func(parent, N, i):
+            return [i/N, 0, 0]
+        bz = BrillouinZone.parametrize(H, func, 101)
+        zak = berry_phase(bz, sub=0, method='zak')
+        assert np.allclose(np.abs(zak), np.pi)
+
     def test_berry_phase_method_fail(self):
         # wrong method keyword
         g = Geometry([[-.6, 0, 0], [0.6, 0, 0]], Atom(1, 1.001), sc=[2, 10, 10])
         g.set_nsc([3, 1, 1])
         H = Hamiltonian(g)
-        H.construct([(0.1, 1.0, 1.5), (0, 1., 0.5)])
-        # Contour
-        k = np.linspace(0.0, 1.0, 101)
-        K = np.zeros([k.size, 3])
-        K[:, 0] = k
-        bz = BrillouinZone(H, K)
+        def func(parent, N, i):
+            return [0, i/N, 0]
+        bz = BrillouinZone.parametrize(H, func, 101)
         with pytest.raises(ValueError):
             berry_phase(bz, method='unknown')
 
@@ -707,9 +900,10 @@ class TestHamiltonian:
 
         k = [0.1] * 3
         ie1 = H.eigenstate(k, gauge='R').berry_curvature()
-        ie2 = H.eigenstate(k, gauge='r').berry_curvature()
+        ie2 = H.eigenstate(k, gauge='r').berry_curvature(degenerate_dir=(1, 1, 0))
         assert np.allclose(ie1, ie2)
 
+    @pytest.mark.filterwarnings('ignore', category=np.ComplexWarning)
     def test_conductivity(self, setup):
         R, param = [0.1, 1.5], [1., 0.1]
         g = setup.g.tile(2, 0).tile(2, 1).tile(2, 2)
@@ -719,17 +913,38 @@ class TestHamiltonian:
         mp = MonkhorstPack(H, [11, 11, 1])
         cond = conductivity(mp)
 
-    def test_gauge_inv_eff(self, setup):
+    @pytest.mark.xfail(reason="Gauges make different decouplings")
+    def test_gauge_eff(self, setup):
+        # it is not fully clear to me why they are different
         R, param = [0.1, 1.5], [1., 0.1]
         g = setup.g.tile(2, 0).tile(2, 1).tile(2, 2)
         H = Hamiltonian(g)
         H.construct((R, param))
 
         k = [0.1] * 3
-        ie1 = H.eigenstate(k, gauge='R').inv_eff_mass_tensor()
-        ie2 = H.eigenstate(k, gauge='r').inv_eff_mass_tensor()
-        str(ie1)
-        str(ie2)
+        ie1 = H.eigenstate(k, gauge='R').effective_mass()
+        ie2 = H.eigenstate(k, gauge='r').effective_mass()
+        assert np.allclose(abs(ie1), abs(ie2))
+
+    def test_eigenstate_polarized_orthogonal_sk(self, setup):
+        R, param = [0.1, 1.5], [1., [0.1, 0.1]]
+        H = Hamiltonian(setup.g, spin='P')
+        H.construct((R, param))
+
+        k = [0.1] * 3
+        ie1 = H.eigenstate(k, spin=0, format='array').Sk()
+        ie2 = H.eigenstate(k, spin=1, format='array').Sk()
+        assert np.allclose(ie1.dot(1.), 1.)
+        assert np.allclose(ie2.dot(1.), 1.)
+
+    def test_eigenstate_polarized_non_ortogonal_sk(self, setup):
+        R, param = [0.1, 1.5], [[1., 1., 1.], [0.1, 0.1, 0.05]]
+        H = Hamiltonian(setup.g, spin='P', orthogonal=False)
+        H.construct((R, param))
+
+        k = [0.1] * 3
+        ie1 = H.eigenstate(k, spin=0, format='array').Sk()
+        ie2 = H.eigenstate(k, spin=1, format='array').Sk()
         assert np.allclose(ie1, ie2)
 
     def test_change_gauge(self, setup):
@@ -756,14 +971,14 @@ class TestHamiltonian:
         for k in ([0] *3, [0.2] * 3):
             es = H.eigenstate(k)
 
-            d = es.expectation(D)
+            d = es.inner(matrix=D)
             assert np.allclose(d, D)
-            d = es.expectation(D, diag=False)
+            d = es.inner(matrix=D, diag=False)
             assert np.allclose(d, I)
 
-            d = es.expectation(I)
+            d = es.inner(matrix=I)
             assert np.allclose(d, D)
-            d = es.expectation(I, diag=False)
+            d = es.inner(matrix=I, diag=False)
             assert np.allclose(d, I)
 
     def test_velocity_orthogonal(self, setup):
@@ -776,6 +991,7 @@ class TestHamiltonian:
             vsub = es.sub([0]).velocity()
             assert np.allclose(v[0, :], vsub)
 
+    @pytest.mark.filterwarnings('ignore', category=np.ComplexWarning)
     def test_velocity_nonorthogonal(self, setup):
         HS = setup.HS.copy()
         HS.construct([(0.1, 1.5), ((1., 1.), (0.1, 0.1))])
@@ -792,44 +1008,21 @@ class TestHamiltonian:
         E = np.linspace(-4, 4, 1000)
         for k in ([0] *3, [0.2] * 3):
             es = H.eigenstate(k)
-            v = es.velocity_matrix()
-            vsub = es.sub([0, 1]).velocity_matrix()
+            v = es.velocity(matrix=True)
+            vsub = es.sub([0, 1]).velocity(matrix=True)
             assert np.allclose(v[:2, :2, :], vsub)
 
+    @pytest.mark.filterwarnings('ignore', category=np.ComplexWarning)
     def test_velocity_matrix_nonorthogonal(self, setup):
         HS = setup.HS.copy()
         HS.construct([(0.1, 1.5), ((1., 1.), (0.1, 0.1))])
         E = np.linspace(-4, 4, 1000)
         for k in ([0] *3, [0.2] * 3):
             es = HS.eigenstate(k)
-            v = es.velocity_matrix()
-            vsub = es.sub([0, 1]).velocity_matrix()
+            v = es.velocity(matrix=True)
+            vsub = es.sub([0, 1]).velocity(matrix=True)
             assert np.allclose(v[:2, :2, :], vsub)
 
-    def test_inv_eff_mass_tensor_orthogonal(self, setup):
-        H = setup.H.copy()
-        H.construct([(0.1, 1.5), ((1., 1.))])
-        E = np.linspace(-4, 4, 1000)
-        for k in ([0] *3, [0.2] * 3):
-            es = H.eigenstate(k)
-            v = es.inv_eff_mass_tensor()
-            vsub = es.sub([0]).inv_eff_mass_tensor()
-            assert np.allclose(v[0, :], vsub)
-            vsub = es.sub([0]).inv_eff_mass_tensor(True)
-            assert np.allclose(v[0, :], _to_voight(vsub))
-
-    def test_inv_eff_mass_tensor_nonorthogonal(self, setup):
-        HS = setup.HS.copy()
-        HS.construct([(0.1, 1.5), ((1., 1.), (0.1, 0.1))])
-        E = np.linspace(-4, 4, 1000)
-        for k in ([0] *3, [0.2] * 3):
-            es = HS.eigenstate(k)
-            v = es.inv_eff_mass_tensor()
-            vsub = es.sub([0]).inv_eff_mass_tensor()
-            assert np.allclose(v[0, :], vsub)
-            vsub = es.sub([0]).inv_eff_mass_tensor(True)
-            assert np.allclose(v[0, :], _to_voight(vsub))
-
     def test_dos1(self, setup):
         HS = setup.HS.copy()
         HS.construct([(0.1, 1.5), ((1., 1.), (0.1, 0.1))])
@@ -902,6 +1095,44 @@ class TestHamiltonian:
         assert PDOS.dtype.kind == 'f'
         assert np.allclose(PDOS.sum(0), DOS)
 
+    def test_coop_against_pdos_nonortho(self, setup):
+        HS = setup.HS.copy()
+        HS.construct([(0.1, 1.5), ((0., 1.), (1., 0.1))])
+        E = np.linspace(-4, 4, 100)
+        for k in ([0] *3, [0.2] * 3):
+            es = HS.eigenstate(k)
+            COOP = es.COOP(E, 'lorentzian')
+
+            DOS = es.DOS(E, 'lorentzian')
+            COOP2DOS = np.array([C.sum() for C in COOP])
+            assert DOS.shape == COOP2DOS.shape
+            assert np.allclose(DOS, COOP2DOS)
+
+            # This one returns sparse matrices, so we have to
+            # deal with that.
+            DOS = es.PDOS(E, 'lorentzian')
+            COOP2DOS = np.array([C.sum(1).A.ravel() for C in COOP]).T
+            assert DOS.shape == COOP2DOS.shape
+            assert np.allclose(DOS, COOP2DOS)
+
+    def test_coop_against_pdos_ortho(self, setup):
+        H = setup.H.copy()
+        H.construct([(0.1, 1.5), (0., 1.)])
+        E = np.linspace(-4, 4, 100)
+        for k in ([0] *3, [0.2] * 3):
+            es = H.eigenstate(k)
+            COOP = es.COOP(E, 'lorentzian')
+
+            DOS = es.DOS(E, 'lorentzian')
+            COOP2DOS = np.array([C.sum() for C in COOP])
+            assert DOS.shape == COOP2DOS.shape
+            assert np.allclose(DOS, COOP2DOS)
+
+            DOS = es.PDOS(E, 'lorentzian')
+            COOP2DOS = np.array([C.sum(1).ravel() for C in COOP]).T
+            assert DOS.shape == COOP2DOS.shape
+            assert np.allclose(DOS, COOP2DOS)
+
     def test_spin1(self, setup):
         g = Geometry([[i, 0, 0] for i in range(10)], Atom(6, R=1.01), sc=SuperCell(100, nsc=[3, 3, 1]))
         H = Hamiltonian(g, dtype=np.int32, spin=Spin.POLARIZED)
@@ -940,6 +1171,93 @@ class TestHamiltonian:
             H2[0, j] = (i, i*2)
         assert H.spsame(H2)
 
+    def test_transform_up(self):
+        g = Geometry([[i, 0, 0] for i in range(10)], Atom(6, R=1.01), sc=SuperCell(100, nsc=[3, 3, 1]))
+        H = Hamiltonian(g, dtype=np.float64, spin=Spin.UNPOLARIZED)
+        for i in range(10):
+            H[0, i] = i + 0.1
+        Hcsr = [H.tocsr(i) for i in range(H.shape[2])]
+
+        Ht = H.transform(spin=Spin.POLARIZED)
+        assert np.abs(Hcsr[0] - Ht.tocsr(0)).sum() == 0
+        assert np.abs(Hcsr[0] - Ht.tocsr(1)).sum() == 0
+
+        Ht = H.transform(spin=Spin.NONCOLINEAR)
+        assert np.abs(Hcsr[0] - Ht.tocsr(0)).sum() == 0
+        assert np.abs(Hcsr[0] - Ht.tocsr(1)).sum() == 0
+
+        Ht = H.transform(spin=Spin.SPINORBIT)
+        assert np.abs(Hcsr[0] - Ht.tocsr(0)).sum() == 0
+        assert np.abs(Hcsr[0] - Ht.tocsr(1)).sum() == 0
+
+    def test_transform_up_nonortho(self):
+        g = Geometry([[i, 0, 0] for i in range(10)], Atom(6, R=1.01), sc=SuperCell(100, nsc=[3, 3, 1]))
+        H = Hamiltonian(g, dtype=np.float64, spin=Spin.UNPOLARIZED, orthogonal=False)
+        for i in range(10):
+            H[0, i] = (i + 0.1, 1.)
+        Hcsr = [H.tocsr(i) for i in range(H.shape[2])]
+
+        Ht = H.transform(spin=Spin.POLARIZED)
+        assert np.abs(Hcsr[0] - Ht.tocsr(0)).sum() == 0
+        assert np.abs(Hcsr[0] - Ht.tocsr(1)).sum() == 0
+        assert np.abs(Hcsr[-1] - Ht.tocsr(-1)).sum() == 0
+
+        Ht = H.transform(spin=Spin.NONCOLINEAR)
+        assert np.abs(Hcsr[0] - Ht.tocsr(0)).sum() == 0
+        assert np.abs(Hcsr[0] - Ht.tocsr(1)).sum() == 0
+        assert np.abs(Hcsr[-1] - Ht.tocsr(-1)).sum() == 0
+
+        Ht = H.transform(spin=Spin.SPINORBIT)
+        assert np.abs(Hcsr[0] - Ht.tocsr(0)).sum() == 0
+        assert np.abs(Hcsr[0] - Ht.tocsr(1)).sum() == 0
+        assert np.abs(Hcsr[-1] - Ht.tocsr(-1)).sum() == 0
+
+    def test_transform_down(self):
+        g = Geometry([[i, 0, 0] for i in range(10)], Atom(6, R=1.01), sc=SuperCell(100, nsc=[3, 3, 1]))
+        H = Hamiltonian(g, dtype=np.float64, spin=Spin.SPINORBIT)
+        for i in range(10):
+            for j in range(8):
+                H[0, i, j] = i + 0.1 + j
+        Hcsr = [H.tocsr(i) for i in range(H.shape[2])]
+
+        Ht = H.transform(spin=Spin.UNPOLARIZED)
+        assert np.abs(0.5 * Hcsr[0] + 0.5 * Hcsr[1] - Ht.tocsr(0)).sum() == 0
+
+        Ht = H.transform(spin=Spin.POLARIZED)
+        assert np.abs(Hcsr[0] - Ht.tocsr(0)).sum() == 0
+        assert np.abs(Hcsr[1] - Ht.tocsr(1)).sum() == 0
+
+        Ht = H.transform(spin=Spin.NONCOLINEAR)
+        assert np.abs(Hcsr[0] - Ht.tocsr(0)).sum() == 0
+        assert np.abs(Hcsr[1] - Ht.tocsr(1)).sum() == 0
+        assert np.abs(Hcsr[2] - Ht.tocsr(2)).sum() == 0
+        assert np.abs(Hcsr[3] - Ht.tocsr(3)).sum() == 0
+
+    def test_transform_down_nonortho(self):
+        g = Geometry([[i, 0, 0] for i in range(10)], Atom(6, R=1.01), sc=SuperCell(100, nsc=[3, 3, 1]))
+        H = Hamiltonian(g, dtype=np.float64, spin=Spin.SPINORBIT, orthogonal=False)
+        for i in range(10):
+            for j in range(8):
+                H[0, i, j] = i + 0.1 + j
+            H[0, i, -1] = 1.
+        Hcsr = [H.tocsr(i) for i in range(H.shape[2])]
+
+        Ht = H.transform(spin=Spin.UNPOLARIZED)
+        assert np.abs(0.5 * Hcsr[0]+ 0.5 * Hcsr[1] - Ht.tocsr(0)).sum() == 0
+        assert np.abs(Hcsr[-1] - Ht.tocsr(-1)).sum() == 0
+
+        Ht = H.transform(spin=Spin.POLARIZED)
+        assert np.abs(Hcsr[0] - Ht.tocsr(0)).sum() == 0
+        assert np.abs(Hcsr[1] - Ht.tocsr(1)).sum() == 0
+        assert np.abs(Hcsr[-1] - Ht.tocsr(-1)).sum() == 0
+
+        Ht = H.transform(spin=Spin.NONCOLINEAR)
+        assert np.abs(Hcsr[0] - Ht.tocsr(0)).sum() == 0
+        assert np.abs(Hcsr[1] - Ht.tocsr(1)).sum() == 0
+        assert np.abs(Hcsr[2] - Ht.tocsr(2)).sum() == 0
+        assert np.abs(Hcsr[3] - Ht.tocsr(3)).sum() == 0
+        assert np.abs(Hcsr[-1] - Ht.tocsr(-1)).sum() == 0
+
     @pytest.mark.parametrize("k", [[0, 0, 0], [0.1, 0, 0]])
     def test_spin_squared(self, setup, k):
         g = Geometry([[i, 0, 0] for i in range(10)], Atom(6, R=1.01), sc=SuperCell(1, nsc=[3, 1, 1]))
@@ -1035,7 +1353,7 @@ class TestHamiltonian:
 
             # Perform spin-moment calculation
             sm = es.spin_moment()
-            sm2 = es.expectation(SZ).real
+            sm2 = es.inner(matrix=SZ).real
             sm3 = np.diag(np.dot(np.conj(es.state), SZ).dot(es.state.T)).real
             assert np.allclose(sm[:, 2], sm2)
             assert np.allclose(sm[:, 2], sm3)
@@ -1046,14 +1364,13 @@ class TestHamiltonian:
             PDOS = es.PDOS(np.linspace(-1, 1, 100))
             DOS = es.DOS(np.linspace(-1, 1, 100))
             assert np.allclose(PDOS.sum(1)[0, :], DOS)
-            es.velocity_matrix()
-            es.inv_eff_mass_tensor()
+            es.velocity(matrix=True)
 
         # Check the velocities
         # But only compare for np.float64, we need the precision
         v = es.velocity()
-        pv = es.velocity(project=True)
-        assert np.allclose(pv.sum(1), v)
+        vv = es.velocity(matrix=True)
+        assert np.allclose(np.diagonal(vv).T, v)
 
         # Ensure we can change gauge for NC stuff
         es.change_gauge('R')
@@ -1110,14 +1427,13 @@ class TestHamiltonian:
             PDOS = es.PDOS(np.linspace(-1, 1, 100))
             DOS = es.DOS(np.linspace(-1, 1, 100))
             assert np.allclose(PDOS.sum(1)[0, :], DOS)
-            es.velocity_matrix()
-            es.inv_eff_mass_tensor()
+            es.velocity(matrix=True)
 
         # Check the velocities
         # But only compare for np.float64, we need the precision
         v = es.velocity()
-        pv = es.velocity(project=True)
-        assert np.allclose(pv.sum(1), v)
+        vv = es.velocity(matrix=True)
+        assert np.allclose(np.diagonal(vv).T, v)
 
         # Ensure we can change gauge for NC stuff
         es.change_gauge('R')
@@ -1176,7 +1492,7 @@ class TestHamiltonian:
             assert np.allclose(es.eig, eig1)
 
             sm = es.spin_moment()
-            sm2 = es.expectation(SZ).real
+            sm2 = es.inner(matrix=SZ).real
             sm3 = np.diag(np.dot(np.conj(es.state), SZ).dot(es.state.T)).real
             assert np.allclose(sm[:, 2], sm2)
             assert np.allclose(sm[:, 2], sm3)
@@ -1187,14 +1503,13 @@ class TestHamiltonian:
             PDOS = es.PDOS(np.linspace(-1, 1, 100))
             DOS = es.DOS(np.linspace(-1, 1, 100))
             assert np.allclose(PDOS.sum(1)[0, :], DOS)
-            es.velocity_matrix()
-            es.inv_eff_mass_tensor()
+            es.velocity(matrix=True)
 
         # Check the velocities
         # But only compare for np.float64, we need the precision
         v = es.velocity()
-        pv = es.velocity(project=True)
-        assert np.allclose(pv.sum(1), v)
+        vv = es.velocity(matrix=True)
+        assert np.allclose(np.diagonal(vv).T, v)
 
         # Ensure we can change gauge for SO stuff
         es.change_gauge('R')
@@ -1564,7 +1879,7 @@ def test_wavefunction2():
     ES = H.eigenstate(dtype=np.float64)
     # This is effectively plotting outside where no atoms exists
     # (there could however still be psi weight).
-    grid = Grid(0.1, sc=SuperCell([2, 2, 2], origo=[2] * 3))
+    grid = Grid(0.1, sc=SuperCell([2, 2, 2], origin=[2] * 3))
     grid.fill(0.)
     ES.sub(0).wavefunction(grid)
 
@@ -1579,7 +1894,7 @@ def test_wavefunction3():
     H.construct([R, param])
     ES = H.eigenstate()
     # Plot in the full thing
-    grid = Grid(0.1, dtype=np.complex128, sc=SuperCell([2, 2, 2], origo=[-1] * 3))
+    grid = Grid(0.1, dtype=np.complex128, sc=SuperCell([2, 2, 2], origin=[-1] * 3))
     grid.fill(0.)
     ES.sub(0).wavefunction(grid)
 
@@ -1594,6 +1909,6 @@ def test_wavefunction_eta():
     H.construct([R, param])
     ES = H.eigenstate()
     # Plot in the full thing
-    grid = Grid(0.1, dtype=np.complex128, sc=SuperCell([2, 2, 2], origo=[-1] * 3))
+    grid = Grid(0.1, dtype=np.complex128, sc=SuperCell([2, 2, 2], origin=[-1] * 3))
     grid.fill(0.)
     ES.sub(0).wavefunction(grid, eta=True)
