@@ -1,13 +1,17 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
 
-pytestmark = pytest.mark.spin
-
 import math as m
 import numpy as np
 
 from sisl import Spin
 
 
+pytestmark = [pytest.mark.physics, pytest.mark.spin]
+
+
 def test_spin1():
     for val in ['unpolarized', '', Spin.UNPOLARIZED,
                 'polarized', 'p', Spin.POLARIZED,
