@@ -1,9 +1,12 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
 
 import math as m
 import numpy as np
 
-from sisl.shape.prism4 import *
+from sisl.shape import *
 
 pytestmark = pytest.mark.shape
 
@@ -12,6 +15,7 @@ def test_create_cuboid():
     cube = Cuboid([1.0]*3)
     cube = Cuboid([1.0]*3, [1.]*3)
     cube = Cuboid([1.0, 2.0, 3.0], [1.]*3)
+    cube = Cuboid([1.0, 2.0, 3.0], origin=[1.]*3)
     v0 = [1., 0.2, 1.0]
     v1 = [1., -0.2, 1.0]
     v2 = [1., -0.2, -1.0]
@@ -26,6 +30,8 @@ def test_create_fail():
     v3 = [1., -0.2, -1.0]
     with pytest.raises(ValueError):
         el = Cuboid([v0, v1, v2, v3])
+    with pytest.raises(ValueError):
+        el = Cuboid(2, center=v1, origin=v2)
 
 
 def test_tosphere():
@@ -35,6 +41,8 @@ def test_tosphere():
     assert cube.toSphere().radius == pytest.approx(1.5 * 3 ** 0.5)
     cube = Cuboid([1., 2., 3.])
     assert cube.toSphere().radius == pytest.approx(1.5 * 3 ** 0.5)
+    assert cube.to.Sphere().radius == pytest.approx(1.5 * 3 ** 0.5)
+    assert isinstance(cube.to.Sphere(), Sphere)
 
 
 def test_toellipsoid():
@@ -79,11 +87,11 @@ def test_vol1():
     assert cube.volume() == 1.
 
 
-def test_origo():
+def test_origin():
     cube = Cuboid([1.0]*3)
-    assert np.allclose(cube.origo, -0.5)
-    cube.set_origo(1)
-    assert np.allclose(cube.origo, 1)
+    assert np.allclose(cube.origin, -0.5)
+    cube.origin = 1
+    assert np.allclose(cube.origin, 1)
 
 
 def test_within1():
