@@ -1,7 +1,8 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 from numbers import Integral
 from functools import reduce
-from operator import eq as op_eq
-from itertools import zip_longest
 
 import numpy as np
 # To speed up the _extend algorithm we limit lookups
@@ -27,11 +28,10 @@ from scipy.sparse import (
 
 from ._internal import set_module
 from . import _array as _a
-from ._array import asarrayi, arrayi, fulli
-from ._indices import indices, indices_only, sorted_unique
+from ._array import asarrayi, arrayi, fulli, array_arange
+from ._indices import indices, indices_only
 from .messages import warn, SislError
-from ._help import array_fill_repeat, get_dtype, isiterable
-from .utils.ranges import array_arange
+from ._help import array_fill_repeat, isiterable
 from .utils.mathematics import intersect_and_diff_sets
 from ._sparse import sparse_dense
 
@@ -292,7 +292,26 @@ class SparseCSR(NDArrayOperatorsMixin):
 
     def diagonal(self):
         r""" Return the diagonal elements from the matrix """
-        return np.array([self[i, i] for i in range(self.shape[0])], dtype=self.dtype)
+        # get the diagonal components
+        diag = np.zeros([self.shape[0], self.shape[2]], dtype=self.dtype)
+
+        ptr = self.ptr
+        ncol = self.ncol
+        col = self.col
+        D = self._D
+
+        # Now retrieve rows and cols
+        idx = (ncol > 0).nonzero()[0]
+        row = repeat(idx.astype(int32, copy=False), ncol[idx])
+        idx = array_arange(ptr[:-1], n=ncol, dtype=int32)
+        col = col[idx]
+        # figure out the indices where we have a diagonal index
+        diag_idx = np.equal(row, col)
+        idx = idx[diag_idx]
+        diag[row[diag_idx]] = D[idx]
+        if self.shape[2] == 1:
+            return diag.ravel()
+        return diag
 
     def diags(self, diagonals, offsets=0, dim=None, dtype=None):
         """ Create a `SparseCSR` with diagonal elements with the same shape as the routine
@@ -613,7 +632,7 @@ class SparseCSR(NDArrayOperatorsMixin):
         # We are *only* deleting columns, so if it is finalized,
         # it will still be
 
-    def translate_columns(self, old, new, clean=True):
+    def translate_columns(self, old, new, rows=None, clean=True):
         """ Takes all `old` columns and translates them to `new`.
 
         Parameters
@@ -622,6 +641,8 @@ class SparseCSR(NDArrayOperatorsMixin):
            old column indices
         new : int or array_like
            new column indices
+        rows : int or array_like
+           only translate columns for the given rows
         clean : bool, optional
            whether the new translated columns, outside the shape, should be deleted or not (default delete)
         """
@@ -629,7 +650,7 @@ class SparseCSR(NDArrayOperatorsMixin):
         new = _a.asarrayi(new)
 
         if len(old) != len(new):
-            raise ValueError(self.__class__.__name__+".translate_columns requires input and output columns with "
+            raise ValueError(f"{self.__class__.__name__}.translate_columns requires input and output columns with "
                              "equal length")
 
         if allclose(old, new):
@@ -637,17 +658,19 @@ class SparseCSR(NDArrayOperatorsMixin):
             return
 
         if np_any(old >= self.shape[1]):
-            raise ValueError(self.__class__.__name__+".translate_columns has non-existing old column values")
+            raise ValueError(f"{self.__class__.__name__}.translate_columns has non-existing old column values")
 
         # Now do the translation
         pvt = _a.arangei(self.shape[1])
         pvt[old] = new
 
         # Get indices of valid column entries
-        idx = array_arange(self.ptr[:-1], n=self.ncol)
-        # Convert the old column indices to new ones
-        col = self.col
-        col[idx] = pvt[col[idx]]
+        if rows is None:
+            idx = array_arange(self.ptr[:-1], n=self.ncol)
+        else:
+            idx = array_arange(self.ptr[rows], n=self.ncol[rows])
+            # Convert the old column indices to new ones
+        self.col[idx] = pvt[self.col[idx]]
 
         # After translation, set to not finalized
         self._finalized = False
@@ -655,32 +678,37 @@ class SparseCSR(NDArrayOperatorsMixin):
             if np_any(new >= self.shape[1]):
                 self._clean_columns()
 
-    def scale_columns(self, col, scale):
+    def scale_columns(self, cols, scale, rows=None):
         r""" Scale all values with certain column values with a number
 
         This will multiply all values with certain column values with `scale`
 
         .. math::
-            M[:, cols] *= scale
+            M[rows, cols] *= scale
 
         This is an in-place operation.
 
         Parameters
         ----------
-        col : int or array_like
-           column indices
+        cols : int or array_like
+           column indices to scale
         scale : float or array_like
            scale value for each value (if array-like it has to have the same
            dimension as the sparsity dimension)
+        rows : int or array_like, optional
+           only scale the column values that exists in these rows, default to all
         """
-        col = _a.asarrayi(col)
+        cols = _a.asarrayi(cols)
 
-        if np_any(col >= self.shape[1]):
-            raise ValueError(self.__class__.__name__+".scale_columns has non-existing old column values")
+        if np_any(cols >= self.shape[1]):
+            raise ValueError(f"{self.__class__.__name__}.scale_columns has non-existing old column values")
 
         # Find indices
-        idx = array_arange(self.ptr[:-1], n=self.ncol)
-        scale_idx = np.isin(self.col[idx], col).nonzero()[0]
+        if rows is None:
+            idx = array_arange(self.ptr[:-1], n=self.ncol)
+        else:
+            idx = array_arange(self.ptr[rows], n=self.ncol[rows])
+        scale_idx = np.isin(self.col[idx], cols).nonzero()[0]
 
         # Scale values where columns coincide with scaling factor
         self._D[idx[scale_idx]] *= scale
@@ -827,7 +855,8 @@ class SparseCSR(NDArrayOperatorsMixin):
 
         Raises
         ------
-        IndexError for indices out of bounds
+        IndexError
+            for indices out of bounds
         """
         if asarray(i).size == 0:
             return arrayi([])
@@ -944,7 +973,8 @@ class SparseCSR(NDArrayOperatorsMixin):
 
         Raises
         ------
-        IndexError for indices out of bounds
+        IndexError
+            for indices out of bounds
         """
         if i < 0 or i >= self.shape[0]:
             raise IndexError('row index is out-of-bounds')
@@ -1310,7 +1340,7 @@ class SparseCSR(NDArrayOperatorsMixin):
 
         Parameters
         ----------
-        dims : array-like, optional
+        dims : int or array-like, optional
            which dimensions to store in the copy, defaults to all.
         dtype : `numpy.dtype`
            this defaults to the dtype of the object,
@@ -1319,9 +1349,10 @@ class SparseCSR(NDArrayOperatorsMixin):
         # Create sparse matrix (with only one entry per
         # row, we overwrite it immediately afterward)
         if dims is None:
-            dim = self.dim
-        else:
-            dim = len(dims)
+            dims = range(self.dim)
+        elif isinstance(dims, Integral):
+            dims = [dims]
+        dim = len(dims)
 
         if dtype is None:
             dtype = self.dtype
@@ -1340,12 +1371,9 @@ class SparseCSR(NDArrayOperatorsMixin):
         new.col = self.col.copy()
         new._nnz = self.nnz
 
-        if dims is None:
-            new._D = self._D.astype(dtype, copy=True)
-        else:
-            new._D = empty([len(self.col), dim], dtype)
-            for i, dim in enumerate(dims):
-                new._D[:, i] = self._D[:, dim]
+        new._D = empty([len(self.col), dim], dtype)
+        for i, dim in enumerate(dims):
+            new._D[:, i] = self._D[:, dim]
 
         # Mark it as the same state as the other one
         new._finalized = self._finalized
@@ -1377,6 +1405,50 @@ class SparseCSR(NDArrayOperatorsMixin):
         return csr_matrix((self._D[idx, dim].copy(), self.col[idx], ptr.astype(int32, copy=False)),
                           shape=shape, **kwargs)
 
+    def transform(self, matrix, dtype=None):
+        r""" Apply a linear transformation :math:`R^n \rightarrow R^m` to the :math:`n`-dimensional elements of the sparse matrix
+
+        Notes
+        -----
+        The transformation matrix does *not* act on the rows and columns, only on the
+        final dimension of the matrix.
+
+        Parameters
+        ----------
+        matrix : array_like
+            transformation matrix of shape :math:`m \times n`, :math:`n` should correspond to
+            the number of elements in ``self.shape[2]``
+        dtype : numpy.dtype, optional
+            defaults to the common dtype of the object and the transformation matrix
+        """
+        matrix = np.asarray(matrix)
+
+        if dtype is None:
+            dtype = np.find_common_type([self.dtype, matrix.dtype], [])
+
+        if matrix.shape[1] != self.shape[2]:
+            raise ValueError(f"{self.__class__.__name__}.transform incompatible "
+                             f"transformation matrix and spin dimensions: "
+                             f"matrix.shape={matrix.shape} and self.spin={self.shape[2]} ; out.spin={matrix.shape[0]}")
+
+        # set dimension of new sparse matrix
+        new_dim = matrix.shape[0]
+        shape = list(self.shape[:])
+        shape[2] = new_dim
+
+        new = self.__class__(shape, dtype=dtype, nnz=1)
+
+        copyto(new.ptr, self.ptr, casting='no')
+        copyto(new.ncol, self.ncol, casting='no')
+        new.col = self.col.copy()
+        new._nnz = self.nnz
+
+        new._D = self._D.dot(matrix.T).astype(dtype, copy=False)
+
+        new._finalized = self._finalized
+
+        return new
+
     @classmethod
     def fromsp(cls, *sps, dtype=None):
         """ Combine multiple single-dimension sparse matrices into one SparseCSR matrix
@@ -1526,36 +1598,82 @@ class SparseCSR(NDArrayOperatorsMixin):
 
         return csr
 
-    def sum(self, axis=None):
-        """ Calculate the sum, if `axis` is ``None`` the sum of all elements are returned, else a new sparse matrix is returned
+    def transpose(self, sort=True):
+        """ Create the transposed sparse matrix
 
         Parameters
         ----------
-        axis : int, optional
-           which axis to perform the sum of. If ``None`` the element sum is returned, if either ``0`` or ``1`` is passed a
-           vector is returned, and for ``2`` it returns a new sparse matrix with the last dimension reduced to 1 (summed).
+        sort : bool, optional
+           the returned columns for the transposed structure will be sorted
+           if this is true, default
 
-        Raises
-        ------
-        NotImplementedError : when ``axis = 1``
+        Notes
+        -----
+        The components for each sparse element are not changed in this method.
+
+        Returns
+        -------
+        object
+            an equivalent sparse matrix with transposed matrix elements
         """
-        if axis is None:
-            return self._D.sum()
-        if axis == -1 or axis == 2:
-            # We simply create a new sparse matrix with only one entry
-            ret = self.copy([0])
-            ret._D[:, 0] = self._D.sum(1)
-        elif axis == -2 or axis == 1:
-            ret = zeros(self.shape[1], dtype=self.dtype)
-            raise NotImplementedError('Currently performing a sum on the columns is not implemented')
-        elif axis == 0:
-            ret = empty([self.shape[0], self.shape[2]], dtype=self.dtype)
-            ptr = self.ptr
-            ncol = self.ncol
-            for r in range(self.shape[0]):
-                ret[r, :] = self._D[ptr[r]:ptr[r]+ncol[r], :].sum(0)
+        # Create a temporary copy to put data into
+        T = self.copy()
+        # properly set the shape!
+        T._shape = (self.shape[1], self.shape[0], self.shape[2])
+        # clean memory to not crowd memory too much
+        T.ptr = None
+        T.col = None
+        T.ncol = None
+        T._D = None
 
-        return ret
+        # First extract the actual data
+        ncol = self.ncol.view()
+        if self.finalized:
+            #ptr = self.ptr.view()
+            col = self.col.copy()
+            D = self._D.copy()
+        else:
+            idx = array_arange(self.ptr[:-1], n=ncol, dtype=int32)
+            #ptr = _ncol_to_indptr(ncol)
+            col = self.col[idx]
+            D = self._D[idx, :].copy()
+            del idx
+
+        # figure out rows where ncol is > 0
+        # we skip the first column
+        row_nonzero = (ncol > 0).nonzero()[0]
+        row = repeat(row_nonzero.astype(np.int32, copy=False), ncol[row_nonzero])
+
+        # Now we have the DOK format
+        #  row, col, _D
+
+        # Now we can re-create the sparse matrix
+        # All we need is to count the number of non-zeros per column.
+        rows, nrow = unique(col, return_counts=True)
+        T.ncol = _a.zerosi(T.shape[0])
+        T.ncol[rows] = nrow
+        del rows
+
+        if sort:
+            # also sort individual rows for each column
+            idx = lexsort((row, col))
+        else:
+            # sort columns to get transposed values.
+            # This will randomize the rows
+            idx = argsort(col)
+
+        # Our new data will then be
+        T.col = row[idx]
+        del row
+        T._D = D[idx]
+        del D
+        T.ptr = _ncol_to_indptr(T.ncol)
+
+        # If `sort` we have everything sorted, otherwise it
+        # is not ensured
+        T._finalized = sort
+
+        return T
 
     def __str__(self):
         """ Representation of the sparse matrix model """
@@ -1569,6 +1687,7 @@ class SparseCSR(NDArrayOperatorsMixin):
     __array_priority__ = 14
 
     def __array_ufunc__(self, ufunc, method, *inputs, **kwargs):
+        #print(f"{self.__class__.__name__}.__array_ufunc__ :", ufunc, method)
         out = kwargs.pop("out", None)
 
         if getattr(ufunc, "signature", None) is not None:
@@ -1580,23 +1699,22 @@ class SparseCSR(NDArrayOperatorsMixin):
             kwargs["dtype"] = out.dtype
 
         if method == "__call__":
-            result = _ufunc_pre(ufunc, *inputs, **kwargs)
+            result = _ufunc_call(ufunc, *inputs, **kwargs)
         elif method == "reduce":
-            # to be handled
-            return NotImplemented
+            result = _ufunc_reduce(ufunc, *inputs, **kwargs)
         elif method == "outer":
+            #print("running outer")
             # Currently I don't know what to do here
             # We don't have multidimensional sparse matrices,
             # but perhaps that could be needed later?
             return NotImplemented
         else:
+            #print("running method = ", method)
             return NotImplemented
 
         if out is None:
-            return result
-
-        # we have to explicitly save it *somewhere*
-        if isinstance(out, ndarray):
+            out = result
+        elif isinstance(out, ndarray):
             out[...] = result[...]
         elif isinstance(out, SparseCSR):
             if out.shape != result.shape:
@@ -1608,8 +1726,9 @@ class SparseCSR(NDArrayOperatorsMixin):
             # this will copy
             out.col = result.col
             out._D = result._D.astype(kwargs.get("dtype", out.dtype))
-            return out
-        return NotImplemented
+        else:
+            out = NotImplemented
+        return out
 
     def __getstate__(self):
         """ Return dictionary with the current state (finalizing the object may reduce memory footprint) """
@@ -1642,25 +1761,6 @@ def _get_reduced_shape(shape):
     return tuple(s for s in shape[::-1] if s > 1)[::-1]
 
 
-def _get_bcast_shape(*shapes):
-    """ Calculate the b-casted shape of the results """
-    res_shape = ()
-    for shape in shapes:
-        shape = shape[::-1]
-        if not all((l1 == l2) or (l1 == 1)
-                   for l1, l2 in zip(shape, res_shape)):
-            raise ValueError(f"operands could not be broadcast together "
-                             f"with shapes {shape[::-1]}, {res_shape[::-1]} <- ({shapes})")
-
-        # Create b-cast shape
-        # https://stackoverflow.com/a/47244284/774273
-        res_shape = tuple(max(l1, l2)
-                          for l1, l2 in
-                          zip_longest(shape, res_shape, fillvalue=1))
-
-    return res_shape[::-1]
-
-
 def _ufunc(ufunc, a, b, **kwargs):
     if isinstance(a, (SparseCSR, spmatrix, tuple)):
         if isinstance(b, (SparseCSR, spmatrix, tuple)):
@@ -1760,18 +1860,21 @@ def _ufunc_sp_sp(ufunc, a, b, **kwargs):
         bsl = arange(bsl.start, bsl.stop)
 
         # Common indices
-        idx, aover, bover, iaonly, ibonly = intersect_and_diff_sets(aidx, bidx)
-        out._D[idx, :] = ufunc(adata[asl[aover], :],
-                               bdata[bsl[bover], :], **kwargs)
+        iover, aover, bover, iaonly, ibonly = intersect_and_diff_sets(aidx, bidx)
 
-        # Remaining indices
+        # overlapping indices
+        out._D[iover, :] = ufunc(adata[asl[aover], :],
+                                 bdata[bsl[bover], :], **kwargs)
+
+        # only a
         out._D[aidx[iaonly]] = ufunc(adata[asl[iaonly], :], 0, **kwargs)
+        # only b
         out._D[bidx[ibonly]] = ufunc(0, bdata[bsl[ibonly], :], **kwargs)
 
     return out
 
 
-def _ufunc_pre(ufunc, *in_args, **kwargs):
+def _ufunc_call(ufunc, *in_args, **kwargs):
 
     # first process in_args to args
     # by numpy-fying and checking for sparsecsr
@@ -1779,12 +1882,11 @@ def _ufunc_pre(ufunc, *in_args, **kwargs):
     for arg in in_args:
         if isinstance(arg, SparseCSR):
             args.append(arg)
-        elif isscalar(arg) or isinstance(arg, ndarray):
+        elif isscalar(arg) or isinstance(arg, (tuple, list, ndarray)):
             args.append(asarray(arg))
         elif isinstance(arg, spmatrix):
             args.append(arg)
         else:
-            args = None
             return
 
     if "dtype" not in kwargs:
@@ -1801,7 +1903,7 @@ def _ufunc_pre(ufunc, *in_args, **kwargs):
             # but SparseCSR always have 3, so we pad with ones.
             return arg.shape + (1,)
         return arg.shape
-    shape = _get_bcast_shape(*tuple(spshape(arg) for arg in args))
+    #shape = _get_bcast_shape(*tuple(spshape(arg) for arg in args))
 
     if len(args) == 1:
         a = args[0]
@@ -1821,6 +1923,47 @@ def _ufunc_pre(ufunc, *in_args, **kwargs):
     return reduce(_, args)
 
 
+def _ufunc_reduce(ufunc, array, axis=0, *args, **kwargs):
+
+    if "dtype" not in kwargs:
+        kwargs["dtype"] = np.result_type(array, *args)
+
+    # currently the initial argument does not work properly if the
+    # size isn't correct
+    if np.asarray(kwargs.get("initial", 0.)).ndim > 1:
+        raise ValueError(f"{array.__class__.__name__}.{ufunc.__name__}.reduce currently does not implement initial values in different dimensions")
+
+    # correct axis
+    if axis is None:
+        # reduction on all axes
+        return ufunc.reduce(array._D, axis=None, *args, **kwargs)
+    elif axis < 0:
+        # correct for negative axis specification
+        axis = axis + len(array.shape)
+
+    if axis == 0:
+        # no need to sorting
+        array = array.transpose(sort=False)
+    elif axis == 1:
+        pass
+    elif axis == (0, 1) or axis == (1, 0):
+        return ufunc.reduce(array._D, axis=0, *args, **kwargs)
+    elif axis == 2:
+        out = array.copy(dims=range(array.shape[2] - 1), dtype=kwargs.get("dtype"))
+        out._D[:, 0] = ufunc.reduce(array._D, axis=1, *args, **kwargs)
+        return out
+    else:
+        raise ValueError(f"Unknown axis argument in ufunc.reduce call on {array.__class__.__name__}")
+
+    ret = empty([array.shape[0], array.shape[2]], dtype=kwargs.get("dtype", array.dtype))
+    # Now do ufunc calculations, note that initial gets passed directly
+    ptr = array.ptr
+    ncol = array.ncol
+    for r in range(array.shape[0]):
+        ret[r, :] = ufunc.reduce(array._D[ptr[r]:ptr[r]+ncol[r], :], axis=0, *args, **kwargs)
+    return ret
+
+
 @set_module("sisl")
 def ispmatrix(matrix, map_row=None, map_col=None):
     """ Iterator for iterating rows and columns for non-zero elements in a `scipy.sparse.*_matrix` (or `SparseCSR`)
