@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
 
 import math as m
@@ -73,7 +76,7 @@ def test_set1():
     assert atom[1] == Atom('Au')
 
 
-@pytest.mark.filterwarnings("ignore: Replacing atom")
+@pytest.mark.filterwarnings("ignore", message="*Replacing atom")
 def test_set2():
     # Add new atoms to the set
     atom = Atoms(['C', 'C'])
@@ -108,7 +111,7 @@ def test_set3():
     assert len(atom.atom) == 2
 
 
-@pytest.mark.filterwarnings("ignore: Replacing atom")
+@pytest.mark.filterwarnings("ignore", message="*Replacing atom")
 def test_replace1():
     # Add new atoms to the set
     atom = Atoms(['C'] * 10 + ['B'] * 2)
@@ -132,8 +135,8 @@ def test_replace1():
         assert atom[i] == Atom('B')
 
 
-@pytest.mark.filterwarnings("ignore: Substituting atom")
-@pytest.mark.filterwarnings("ignore: Replacing atom")
+@pytest.mark.filterwarnings("ignore", message="*Substituting atom")
+@pytest.mark.filterwarnings("ignore", message="*Replacing atom")
 def test_replace2():
     # Add new atoms to the set
     atom = Atoms(['C'] * 10 + ['B'] * 2)
