@@ -1,7 +1,11 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
-
+import os.path as osp
 import itertools
 import math as m
+
 import numpy as np
 
 import sisl.geom as sisl_geom
@@ -9,6 +13,10 @@ from sisl import SislWarning, SislError
 from sisl import Cube, Sphere
 from sisl import Geometry, Atom, SuperCell
 
+_dir = osp.join('sisl')
+
+pytestmark = [pytest.mark.geom, pytest.mark.geometry]
+
 
 @pytest.fixture
 def setup():
@@ -28,8 +36,6 @@ def setup():
     return t()
 
 
-@pytest.mark.geom
-@pytest.mark.geometry
 class TestGeometry:
 
     def test_objects(self, setup):
@@ -303,22 +309,22 @@ class TestGeometry:
                                                         [1.42, 0, 0]])
         assert np.allclose(setup.g.oRij(0, 2), [1.42, 0, 0])
 
-    def test_cut(self, setup):
+    def test_untile_warns(self, setup):
         with pytest.warns(SislWarning) as warns:
-            assert len(setup.g.cut(1, 1)) == 2
-            assert len(setup.g.cut(2, 1)) == 1
-            assert len(setup.g.cut(2, 1, 1)) == 1
+            assert len(setup.g.untile(1, 1)) == 2
+            assert len(setup.g.untile(2, 1)) == 1
+            assert len(setup.g.untile(2, 1, 1)) == 1
         assert len(warns) == 2
 
-    def test_cut2(self, setup):
+    def test_untile_check_same(self, setup):
         with pytest.warns(SislWarning) as warns:
-            c1 = setup.g.cut(2, 1)
-            c2 = setup.g.cut(2, 1, 1)
+            c1 = setup.g.untile(2, 1)
+            c2 = setup.g.untile(2, 1, 1)
         assert len(warns) == 2
         assert np.allclose(c1.xyz[0, :], setup.g.xyz[0, :])
         assert np.allclose(c2.xyz[0, :], setup.g.xyz[1, :])
 
-    def test_cut3(self, setup):
+    def test_untile_algo(self, setup):
         nr = range(2, 5)
         g = setup.g.copy()
         for x in nr:
@@ -327,13 +333,32 @@ class TestGeometry:
                 gy = gx.tile(y, 1)
                 for z in nr:
                     gz = gy.tile(z, 2)
-                    G = gz.cut(z, 2)
+                    G = gz.untile(z, 2)
                     assert np.allclose(G.xyz, gy.xyz)
                     assert np.allclose(G.cell, gy.cell)
-                G = gy.cut(y, 1)
+                G = gy.untile(y, 1)
                 assert np.allclose(G.xyz, gx.xyz)
                 assert np.allclose(G.cell, gx.cell)
-            G = gx.cut(x, 0)
+            G = gx.untile(x, 0)
+            assert np.allclose(G.xyz, g.xyz)
+            assert np.allclose(G.cell, g.cell)
+
+    def test_unrepeat_algo(self, setup):
+        nr = range(2, 5)
+        g = setup.g.copy()
+        for x in nr:
+            gx = g.repeat(x, 0)
+            for y in nr:
+                gy = gx.repeat(y, 1)
+                for z in nr:
+                    gz = gy.repeat(z, 2)
+                    G = gz.unrepeat(z, 2)
+                    assert np.allclose(G.xyz, gy.xyz)
+                    assert np.allclose(G.cell, gy.cell)
+                G = gy.unrepeat(y, 1)
+                assert np.allclose(G.xyz, gx.xyz)
+                assert np.allclose(G.cell, gx.cell)
+            G = gx.unrepeat(x, 0)
             assert np.allclose(G.xyz, g.xyz)
             assert np.allclose(G.cell, g.cell)
 
@@ -550,12 +575,12 @@ class TestGeometry:
         assert np.allclose(setup.g.cell[1, :], s.cell[0, :])
 
     def test_center(self, setup):
-        one = setup.g.center(atoms=[0])
-        assert np.allclose(setup.g[0], one)
-        al = setup.g.center()
-        assert np.allclose(np.mean(setup.g.xyz, axis=0), al)
-        al = setup.g.center(what='mass')
-        al = setup.g.center(what='mm(xyz)')
+        g = setup.g.copy()
+        assert np.allclose(g[1], g.center(atoms=[1]))
+        assert np.allclose(np.mean(g.xyz, axis=0), g.center())
+        # in this case the pbc COM is equivalent to the simple one
+        assert np.allclose(g.center(what='mass'), g.center(what='mass:pbc'))
+        assert np.allclose(g.center(what='mm:xyz'), g.center(what='mm(xyz)'))
 
     def test_center_raise(self, setup):
         with pytest.raises(ValueError):
@@ -597,6 +622,7 @@ class TestGeometry:
         g = setup.g.copy()
         assert g * 2 == g.tile(2, 0).tile(2, 1).tile(2, 2)
         assert g * [2, 1] == g.tile(2, 1)
+        assert [2, 1] * g == g.repeat(2, 1)
         assert g * (2, 2, 2) == g.tile(2, 0).tile(2, 1).tile(2, 2)
         assert g * [1, 2, 2] == g.tile(1, 0).tile(2, 1).tile(2, 2)
         assert g * [1, 3, 2] == g.tile(1, 0).tile(3, 1).tile(2, 2)
@@ -636,8 +662,11 @@ class TestGeometry:
     def test_angle(self, setup):
         # There are 2 orbitals per C atom
         g = Geometry([[0] * 3, [1, 0, 0]])
+        cell = g.cell.copy()
         g.angle([0])
         g.angle([0], ref=1)
+        g.angle([0], dir=1)
+        assert np.allclose(g.cell[1], cell[1])
 
     def test_2uc(self, setup):
         # functions for any-thing to UC
@@ -682,6 +711,24 @@ class TestGeometry:
         assert len(two) == len(setup.g)
         assert np.allclose(two.xyz[:, :] / 2., setup.g.xyz)
 
+    def test_scale_vector_abc(self, setup):
+        two = setup.g.scale([2, 1, 1], what="abc")
+        assert len(two) == len(setup.g)
+        # Check that cell has been scaled accordingly
+        assert np.allclose(two.cell[0] / 2., setup.g.cell[0])
+        assert np.allclose(two.cell[1:], setup.g.cell[1:])
+        # Now check that fractional coordinates are still the same
+        assert np.allclose(two.fxyz, setup.g.fxyz)
+
+    def test_scale_vector_xyz(self, setup):
+        two = setup.g.scale([2, 1, 1], what="xyz")
+        assert len(two) == len(setup.g)
+        # Check that cell has been scaled accordingly
+        assert np.allclose(two.cell[:, 0] / 2., setup.g.cell[:, 0])
+        assert np.allclose(two.cell[:, 1:], setup.g.cell[:, 1:])
+        # Now check that fractional coordinates are still the same
+        assert np.allclose(two.fxyz, setup.g.fxyz)
+
     def test_close1(self, setup):
         three = range(3)
         for ia in setup.mol:
@@ -766,8 +813,8 @@ class TestGeometry:
         g = setup.mol.translate([0.05] * 3)
         sc = SuperCell(1.5)
         for o in range(10):
-            origo = [o - 0.5, -0.5, -0.5]
-            sc.origo = origo
+            origin = [o - 0.5, -0.5, -0.5]
+            sc.origin = origin
             idx = g.within_inf(sc)[0]
             assert len(idx) == 1
             assert idx[0] == o
@@ -1142,9 +1189,9 @@ class TestGeometry:
                 '--tile', '2', 'x',
                 '--tile', '2', 'y',
                 '--tile', '2', 'z',
-                '--cut', '2', 'z',
-                '--cut', '2', 'y',
-                '--cut', '2', 'x',
+                '--untile', '2', 'z',
+                '--untile', '2', 'y',
+                '--untile', '2', 'x',
         ]
         if kwargs.get('limit_arguments', True):
             opts.extend(['--rotate', '-90', 'x',
@@ -1176,7 +1223,7 @@ class TestGeometry:
         g = setup.g.attach(0, setup.mol, 0, dist='calc', axis=2)
         g = setup.g.attach(0, setup.mol, 0, dist=[0, 0, 1.42])
 
-    def test_mirror1(self, setup):
+    def test_mirror_function(self, setup):
         g = setup.g
         for plane in ['xy', 'xz', 'yz', 'ab', 'bc', 'ac']:
             g.mirror(plane)
@@ -1186,6 +1233,18 @@ class TestGeometry:
 
         assert g.mirror('xy', [0]) == g.mirror([0, 0, 1], [0])
 
+    def test_mirror_point(self):
+        g = Geometry([[0, 0, 0], [0, 0, 1]])
+        out = g.mirror('z')
+        assert np.allclose(out.xyz[:, 2], [0, -1])
+        assert np.allclose(out.xyz[:, :2], 0)
+        out = g.mirror('z', point=(0, 0, 0.5))
+        assert np.allclose(out.xyz[:, 2], [1, 0])
+        assert np.allclose(out.xyz[:, :2], 0)
+        out = g.mirror('z', point=(0, 0, 1))
+        assert np.allclose(out.xyz[:, 2], [2, 1])
+        assert np.allclose(out.xyz[:, :2], 0)
+
     def test_pickle(self, setup):
         import pickle as p
         s = p.dumps(setup.g)
@@ -1228,10 +1287,10 @@ class TestGeometry:
         with pytest.raises(ValueError):
             g.as_primary(3)
 
-    def test_geometry_cut_raise_nondivisable(self):
+    def test_geometry_untile_raise_nondivisable(self):
         g = sisl_geom.graphene()
         with pytest.raises(ValueError):
-            g.cut(3, 0)
+            g.untile(3, 0)
 
     def test_geometry_iR_negative_R(self):
         g = sisl_geom.graphene()
@@ -1294,10 +1353,18 @@ class TestGeometry:
         from_ase = gr.new(to_ase)
         assert gr.equal(from_ase, R=False)
 
+    def test_geometry_pymatgen_to(self):
+        pytest.importorskip("pymatgen", reason="pymatgen not available")
+        gr = sisl_geom.graphene()
+        to_pymatgen = gr.to.pymatgen()
+        from_pymatgen = gr.new(to_pymatgen)
+        assert gr.equal(from_pymatgen, R=False)
+        # TODO add test for Molecule as well (cell will then be different)
+
     def test_geometry_pandas_to(self):
         pytest.importorskip("pandas", reason="pandas not available")
         gr = sisl_geom.graphene()
-        df = gr.to.df()
+        df = gr.to.dataframe()
         assert np.allclose(df["x"], gr.xyz[:, 0])
         assert np.allclose(df["y"], gr.xyz[:, 1])
         assert np.allclose(df["z"], gr.xyz[:, 2])
@@ -1442,6 +1509,10 @@ def test_geometry_sort_group():
     assert np.allclose(BN.atoms.Z, BN2.atoms.Z)
     assert np.allclose(BN.atoms.Z, BN3.atoms.Z)
 
+    mass = bi.sort(group='mass')
+    Z = bi.sort(group='Z')
+    assert np.allclose(mass.atoms.Z, Z.atoms.Z)
+
 
 def test_geometry_sort_fail_keyword():
     with pytest.raises(ValueError):
@@ -1450,7 +1521,7 @@ def test_geometry_sort_fail_keyword():
 
 @pytest.mark.category
 @pytest.mark.geom_category
-def test_geometry_sanitize_atom():
+def test_geometry_sanitize_atom_category():
     bi = sisl_geom.bilayer(bottom_atoms=Atom[6], top_atoms=(Atom[5], Atom[7])).tile(2, 0).repeat(2, 1)
     C_idx = (bi.atoms.Z == 6).nonzero()[0]
     check_C = bi.axyz(C_idx)
@@ -1475,3 +1546,92 @@ def test_geometry_sanitize_atom():
                        bi._sanitize_atoms(list_01))
     assert np.allclose(bi._sanitize_atoms(ndarray_01),
                        bi._sanitize_atoms(list_01))
+
+
+def test_geometry_sanitize_atom_shape():
+    bi = sisl_geom.bilayer(bottom_atoms=Atom[6], top_atoms=(Atom[5], Atom[7])).tile(2, 0).repeat(2, 1)
+    cube = Cube(10)
+    assert len(bi.axyz(cube)) != 0
+
+
+def test_geometry_sanitize_atom_0_length():
+    gr = sisl_geom.graphene()
+    assert len(gr.axyz([])) == 0
+
+
+@pytest.mark.parametrize("atoms", [[True, False],
+                                   (True, False),
+                                   [0], (0,)
+])
+def test_geometry_sanitize_atom_other_bool(atoms):
+    gr = sisl_geom.graphene()
+    assert len(gr.axyz(atoms)) == 1
+
+
+def test_geometry_sanitize_atom_0_length_float_fail():
+    gr = sisl_geom.graphene()
+    with pytest.raises(IndexError):
+        gr.axyz(np.array([], dtype=np.float64))
+
+
+def test_geometry_sanitize_orbs():
+    bot = Atom(6, [1, 2, 3])
+    top = [Atom(5, [1, 2]), Atom(7, [1, 2])]
+
+    bi = sisl_geom.bilayer(bottom_atoms=bot, top_atoms=top).tile(2, 0).repeat(2, 1)
+
+    C_idx = (bi.atoms.Z == 6).nonzero()[0]
+    assert np.allclose(bi._sanitize_orbs({bot: [0]}), bi.firsto[C_idx])
+    assert np.allclose(bi._sanitize_orbs({bot: 1}), bi.firsto[C_idx] + 1)
+    assert np.allclose(bi._sanitize_orbs({bot: [1, 2]}), np.add.outer(bi.firsto[C_idx], [1, 2]).ravel())
+
+
+def test_geometry_sub_orbitals():
+    atom = Atom(6, [1, 2, 3])
+    gr = sisl_geom.graphene(atoms=atom)
+    assert gr.no == 6
+
+    # try and do sub
+    gr2 = gr.sub_orbital(atom, [atom.orbitals[0], atom.orbitals[2]])
+    assert gr2.atoms[0][0] == atom.orbitals[0]
+    assert gr2.atoms[0][1] == atom.orbitals[2]
+
+    gr2 = gr.sub_orbital(atom, atom.orbitals[1])
+    assert gr2.atoms[0][0] == atom.orbitals[1]
+
+
+def test_geometry_new_xyz(sisl_tmp):
+    # test that Geometry.new works
+    out = sisl_tmp('out.xyz', _dir)
+    C = Atom[6]
+    gr = sisl_geom.graphene(atoms=C)
+    # writing doesn't save orbital information, so we force
+    # an empty atom
+    gr.write(out)
+
+    gr2 = Geometry.new(out)
+    assert np.allclose(gr.xyz, gr2.xyz)
+    assert gr == gr2
+
+    gr2 = gr.new(out)
+    assert np.allclose(gr.xyz, gr2.xyz)
+    assert gr == gr2
+
+
+def test_translate2uc():
+    gr = sisl_geom.graphene() * (2, 3, 1)
+    gr = gr.move([5, 5, 5])
+    gr2 = gr.translate2uc()
+    assert not np.allclose(gr.xyz, gr2.xyz)
+
+
+def test_translate2uc_axes():
+    gr = sisl_geom.graphene() * (2, 3, 1)
+    gr = gr.move([5, 5, 5])
+    gr_once = gr.translate2uc()
+    gr_individual = gr.translate2uc(axes=0)
+    assert not np.allclose(gr_once.xyz, gr_individual.xyz)
+    gr_individual = gr_individual.translate2uc(axes=1)
+    assert np.allclose(gr_once.xyz, gr_individual.xyz)
+    gr_individual = gr_individual.translate2uc(axes=2)
+    assert np.allclose(gr_once.xyz, gr_individual.xyz)
