@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
 
 import operator as ops
@@ -52,7 +55,7 @@ def test_oplist_imath(op, key):
     l2 = d[key]
     try:
         l1 = oplist(data * 2 for data in l2)
-    except:
+    except Exception:
         l1 = oplist([ar.aranged(1, 10), ar.aranged(2, 3)])
     op(l1, l2)
 
