@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
 
 import math as m
@@ -10,22 +13,7 @@ pytestmark = pytest.mark.version
 
 def test_version():
     sisl.__bibtex__
-    sisl.__git_revision__
     sisl.__version__
-    sisl.__major__
-    sisl.__minor__
-    sisl.__micro__
-    # Currently we only do bibtex citation
-    assert sisl.__bibtex__ == sisl.cite()
-    sisl.info.bibtex
-    sisl.info.git_revision
-    sisl.info.version
-    sisl.info.major
-    sisl.info.minor
-    sisl.info.micro
-    sisl.info.release
-    sisl.info.git_revision
-    sisl.info.git_revision_short
 
 
 def test_import1():
