@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
 
 import math as m
@@ -13,7 +16,7 @@ from sisl.sparse_geometry import *
 def setup():
     class t():
         def __init__(self):
-            self.g = fcc(1., Atom(1, R=1.5)) * 2
+            self.g = fcc(1., Atom(1, R=1.495)) * 2
             self.s1 = SparseAtom(self.g)
             self.s2 = SparseAtom(self.g, 2)
     return t()
@@ -106,26 +109,101 @@ class TestSparseAtom:
         with pytest.raises(ValueError):
             s1.construct([[0.1, 1.5], [1]])
 
-    def test_cut1(self, setup):
+    def test_untile1(self, setup):
         s1 = SparseAtom(setup.g)
         s1.construct([[0.1, 1.5], [1, 2]])
         s2 = SparseAtom(setup.g * 2)
         s2.construct([[0.1, 1.5], [1, 2]])
-        s2 = s2.cut(2, 2).cut(2, 1).cut(2, 0)
+        s2 = s2.untile(2, 2).untile(2, 1).untile(2, 0)
         assert s1.spsame(s2)
 
         s1 = SparseAtom(setup.g)
         s1.construct([[0.1, 1.5], [1, 2]])
         s2 = SparseAtom(setup.g * [2, 1, 1])
         s2.construct([[0.1, 1.5], [1, 2]])
-        s2 = s2.cut(2, 0)
+        s2 = s2.untile(2, 0)
         assert s1.spsame(s2)
 
         s1 = SparseAtom(setup.g)
         s1.construct([[0.1, 1.5], [1, 2]])
         s2 = SparseAtom(setup.g * [1, 2, 1])
         s2.construct([[0.1, 1.5], [1, 2]])
-        s2 = s2.cut(2, 1)
+        s2 = s2.untile(2, 1)
+        assert s1.spsame(s2)
+
+    def test_untile_wrong_usage(self):
+        # one should not untile
+        geometry = Geometry([0] * 3, Atom(1, R=1.001), SuperCell(1, nsc=[1]* 3))
+        geometry = geometry.tile(4, 0)
+        s = SparseAtom(geometry)
+        s.construct([[0.1, 1.01], [1, 2]])
+        s[0, 3] = 2
+        s[3, 0] = 2
+
+        # check that untiling twice is not the same as untiling 4 times and coupling it
+        s2 = s.untile(2, 0)
+        s4 = s.untile(4, 0).tile(2, 0)
+        ds = s2 - s4
+        ds.finalize()
+        assert np.absolute(ds)._csr._D.sum() > 0
+
+    def test_untile_segment_single(self):
+        # one should not untile
+        geometry = Geometry([0] * 3, Atom(1, R=1.001), SuperCell(1, nsc=[1]* 3))
+        geometry = geometry.tile(4, 0)
+        s = SparseAtom(geometry)
+        s.construct([[0.1, 1.01], [1, 2]])
+        s[0, 3] = 2
+        s[3, 0] = 2
+
+        # check that untiling twice is not the same as untiling 4 times and coupling it
+        s4 = s.untile(4, 0).tile(2, 0)
+        for seg in range(1, 4):
+            sx = s.untile(4, 0, segment=seg).tile(2, 0)
+            ds = s4 - sx
+            ds.finalize()
+            assert np.absolute(ds)._csr._D.sum() == pytest.approx(0.)
+
+    @pytest.mark.parametrize("axis", [0, 1, 2])
+    def test_untile_segment_three(self, axis):
+        # one should not untile
+        nsc = [3] * 3
+        nsc[axis] = 1
+        geometry = Geometry([0] * 3, Atom(1, R=1.001), SuperCell(1, nsc=nsc))
+        geometry = geometry.tile(4, axis)
+        s = SparseAtom(geometry)
+        s.construct([[0.1, 1.01], [1, 2]])
+        s[0, 3] = 2
+        s[3, 0] = 2
+
+        # check that untiling twice is not the same as untiling 4 times and coupling it
+        s4 = s.untile(4, axis).tile(2, axis)
+        for seg in range(1, 4):
+            sx = s.untile(4, axis, segment=seg).tile(2, axis)
+            ds = s4 - sx
+            ds.finalize()
+            assert np.absolute(ds)._csr._D.sum() == pytest.approx(0.)
+
+    def test_unrepeat_setup(self, setup):
+        s1 = SparseAtom(setup.g)
+        s1.construct([[0.1, 1.5], [1, 2]])
+        s2 = SparseAtom(setup.g * ((2, 2, 2), 'r'))
+        s2.construct([[0.1, 1.5], [1, 2]])
+        s2 = s2.unrepeat(2, 2).unrepeat(2, 1).unrepeat(2, 0)
+        assert s1.spsame(s2)
+
+        s1 = SparseAtom(setup.g)
+        s1.construct([[0.1, 1.5], [1, 2]])
+        s2 = SparseAtom(setup.g * ([2, 1, 1], 'r'))
+        s2.construct([[0.1, 1.5], [1, 2]])
+        s2 = s2.unrepeat(2, 0)
+        assert s1.spsame(s2)
+
+        s1 = SparseAtom(setup.g)
+        s1.construct([[0.1, 1.5], [1, 2]])
+        s2 = SparseAtom(setup.g * ([1, 2, 1], 'r'))
+        s2.construct([[0.1, 1.5], [1, 2]])
+        s2 = s2.unrepeat(2, 1)
         assert s1.spsame(s2)
 
     def test_iter(self, setup):
@@ -217,11 +295,11 @@ class TestSparseAtom:
         s1.finalize()
         s2.finalize()
         assert np.allclose(s1._csr._D, s2._csr._D)
-        s2 = s2.cut(2, 1).cut(2, 0)
+        s2 = s2.untile(2, 1).untile(2, 0)
         assert setup.s1.spsame(s2)
         s2.finalize()
         assert np.allclose(setup.s1._csr._D, s2._csr._D)
-        s1 = s1.cut(2, 1).cut(2, 0)
+        s1 = s1.untile(2, 1).untile(2, 0)
         assert setup.s1.spsame(s1)
         s1.finalize()
         assert np.allclose(s1._csr._D, setup.s1._csr._D)
@@ -237,11 +315,11 @@ class TestSparseAtom:
         s1.finalize()
         s2.finalize()
         assert np.allclose(s1._csr._D, s2._csr._D)
-        s2 = s2.cut(2, 1).cut(2, 0)
+        s2 = s2.untile(2, 1).untile(2, 0)
         assert setup.s1.spsame(s2)
         s2.finalize()
         assert np.allclose(setup.s1._csr._D, s2._csr._D)
-        s1 = s1.cut(2, 1).cut(2, 0)
+        s1 = s1.untile(2, 1).untile(2, 0)
         assert setup.s1.spsame(s1)
         s1.finalize()
         assert np.allclose(s1._csr._D, setup.s1._csr._D)
@@ -552,3 +630,23 @@ def test_sparse_orbital_add_no_axis():
     s2 = SparseOrbital(g.add(g, offset=[0, 0, 3]))
     s2.construct([[0.1, 1.5], [1, 2]])
     assert s1.spsame(s2)
+
+
+def test_sparse_orbital_sub_orbital():
+    atom = Atom(1, (1, 2, 3))
+    g = fcc(1., atom) * 2
+    s = SparseOrbital(g)
+
+    # take out some orbitals
+    s1 = s.sub_orbital(atom, 1)
+    assert s1.geometry.no == s1.geometry.na
+
+    s2 = s.sub_orbital(atom, atom.orbitals[1])
+    assert s1 == s2
+
+    s2 = s.sub_orbital(atom, [atom.orbitals[1]])
+    assert s1 == s2
+
+    s2 = s.sub_orbital(atom, [atom.orbitals[1], atom.orbitals[0]])
+    assert s2.geometry.atoms[0].orbitals[0] == atom.orbitals[0]
+    assert s2.geometry.atoms[0].orbitals[1] == atom.orbitals[1]
