@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """
 Unit conversion
 ===============
@@ -42,9 +45,5 @@ will use the unit definitions in `Siesta`_.
 """
 from .base import unit_group, unit_convert, unit_default, units
 
-__all__ = [s for s in dir() if not s.startswith('_')]
-
 # Enable the siesta unit-conversion
 from . import siesta
-
-__all__ += ['siesta']
