@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 """
 Utility routines
 ================
@@ -23,11 +26,11 @@ Miscellaneous routines
    angle - radian to degree
    iter_shape
    math_eval
+   batched_indices
 
 """
+from ._arrays import *
 from .cmd import *
 from .misc import *
 from .ranges import *
 from . import mathematics as math
-
-__all__ = [s for s in dir() if not s.startswith('_')]
