@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import sys
 import inspect
 import functools
@@ -11,7 +14,7 @@ import importlib
 __all__ = ["merge_instances", "str_spec", "direction", "angle"]
 __all__ += ["iter_shape", "math_eval", "allow_kwargs"]
 __all__ += ["import_attr", "lazy_import"]
-__all__ += ["PropertyDict"]
+__all__ += ["PropertyDict", "NotNonePropertyDict"]
 
 
 # supported operators
@@ -146,7 +149,7 @@ def str_spec(name):
 
 
 # Transform a string to a Cartesian direction
-def direction(d):
+def direction(d, abc=None, xyz=None):
     """ Index coordinate corresponding to the Cartesian coordinate system.
 
     Parameters
@@ -154,11 +157,18 @@ def direction(d):
     d : {0, "x", "a", 1, "y", "b", 2, "z", "c"}
        returns the integer that corresponds to the coordinate index (strings are
        lower-cased).
+    abc : (3, 3), optional
+       for ``"abc"`` inputs the returned value will be the vector ``abc[direction(d)]``
 
     Returns
     -------
-    int
-       The index of the Cartesian coordinate system.
+    index : int 
+       index of the Cartesian coordinate system, only if both `abc` and `xyz` are none
+       or if the requested direction is not present, only returned if the corresponding direction
+       is none
+    vector : (3,)
+       the vector corresponding to the value gotten from `abc` or `xyz`, only returned
+       if the corresponding direction is not none
 
     Examples
     --------
@@ -174,6 +184,14 @@ def direction(d):
     2
     >>> direction("b")
     1
+    >>> direction("b", abc=np.diag([1, 2, 3])
+    [0, 2, 0]
+    >>> direction("x", abc=np.diag([1, 2, 3])
+    0
+    >>> direction(1, abc=np.diag([1, 2, 3])
+    [0, 2, 0]
+    >>> direction(1, abc=np.diag([1, 2, 3], xyz=np.diag([4, 5, 6])
+    [0, 2, 0]
     """
     if isinstance(d, Integral):
         # pass through to find it
@@ -181,10 +199,14 @@ def direction(d):
 
     # We take it as a string
     d = d.lower().strip()
-    # We must use an array to not allow 'xy' input
-    if d in ("x", "y", "z", "a", "b", "c", "0", "1", "2"):
-        return "xa0yb1zc2".index(d) // 3
 
+    if not abc is None and d in "abc012":
+        return abc["a0b1c2".index(d) // 2]
+    elif not xyz is None and d in "xyz012":
+        return xyz["x0y1z2".index(d) // 2]
+    else:
+        if d in ("x", "y", "z", "a", "b", "c", "0", "1", "2"):
+            return "xa0yb1zc2".index(d) // 3
     raise ValueError("direction: Input direction is not an integer, nor a string in 'xyz/abc/012'")
 
 
@@ -289,8 +311,7 @@ def allow_kwargs(*args):
         arg_names = []
         kwargs_name = None
         for name, p in sig.parameters.items():
-            if p.kind == p.POSITIONAL_ONLY or p.kind == p.POSITIONAL_OR_KEYWORD \
-               or p.kind == p.KEYWORD_ONLY:
+            if p.kind in (p.POSITIONAL_ONLY, p.POSITIONAL_OR_KEYWORD, p.KEYWORD_ONLY):
                 arg_names.append(name)
             elif p.kind == p.VAR_KEYWORD:
                 kwargs_name = name
@@ -397,3 +418,11 @@ class PropertyDict(dict):
 
     def __dir__(self):
         return list(self.keys())
+
+
+class NotNonePropertyDict(PropertyDict):
+
+    def __setitem__(self, key, value):
+        if value is None:
+            return
+        super().__setitem__(key, value)
