@@ -1,11 +1,11 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import re
 from itertools import groupby
 
-from numpy import zeros, ones, cumsum, take, int32, int64
-from numpy import asarray
 
 __all__ = ["strmap", "strseq", "lstranges", "erange", "list2str", "fileindex"]
-__all__ += ["array_arange"]
 
 
 # Function to change a string to a range of integers
@@ -31,6 +31,8 @@ def strmap(func, s, start=None, end=None, sep="b"):
     [1]
     >>> strmap(int, "1-2")
     [(1, 2)]
+    >>> strmap(int, "-2", end=4)
+    [2]
     >>> strmap(int, "1-")
     [(1, None)]
     >>> strmap(int, "1-", end=4)
@@ -38,10 +40,10 @@ def strmap(func, s, start=None, end=None, sep="b"):
     >>> strmap(int, "1-10[2-3]")
     [((1, 10), [(2, 3)])]
     """
-    if sep == "b":
+    if sep in ("b", "[", "]"):
         segment = re.compile(r"\[(.+)\]\[(.+)\]|(.+)\[(.+)\]|(.+)")
         sep1, sep2 = "[", "]"
-    elif sep == "c":
+    elif sep in ("c", "{", "}"):
         segment = re.compile(r"\{(.+)\}\{(.+)\}|(.+)\{(.+)\}|(.+)")
         sep1, sep2 = "{", "}"
     else:
@@ -113,6 +115,14 @@ def strseq(cast, s, start=None, end=None):
     3
     >>> strseq(int, "3-6")
     (3, 6)
+    >>> strseq(int, "-2")
+    -2
+    >>> strseq(int, "-2", end=7)
+    5
+    >>> strseq(int, "-2:2", end=7)
+    (-2, 2)
+    >>> strseq(int, "-2-2", end=7)
+    (-2, 2)
     >>> strseq(int, "3-")
     (3, None)
     >>> strseq(int, "3:2:7")
@@ -126,6 +136,10 @@ def strseq(cast, s, start=None, end=None):
     """
     if ":" in s:
         s = [ss.strip() for ss in s.split(":")]
+    elif s.startswith("-") and len(s) > 1:
+        if s.count("-") > 1:
+            s = [ss.strip() for ss in s[1:].split("-")]
+            s[0] = f"-{s[0]}"
     elif "-" in s:
         s = [ss.strip() for ss in s.split("-")]
     if isinstance(s, list):
@@ -134,7 +148,10 @@ def strseq(cast, s, start=None, end=None):
         if len(s[-1]) == 0:
             s[-1] = end
         return tuple(cast(ss) if ss is not None else None for ss in s)
-    return cast(s)
+    ret = cast(s)
+    if ret < 0 and end is not None:
+        return ret + end
+    return ret
 
 
 def erange(start, step, end=None):
@@ -256,62 +273,3 @@ def fileindex(f, cast=int):
     if len(rng) == 1:
         return fname, rng[0]
     return fname, rng
-
-
-def array_arange(start, end=None, n=None, dtype=int64):
-    """ Creates a single array from a sequence of `numpy.arange`
-
-    Parameters
-    ----------
-    start : array_like
-       a list of start elements for `numpy.arange`
-    end : array_like
-       a list of end elements (exclusive) for `numpy.arange`.
-       This argument is not used if `n` is passed.
-    n : array_like
-       a list of counts of elements for `numpy.arange`.
-       This is equivalent to ``end=start + n``.
-    dtype : numpy.dtype
-       the returned lists data-type
-
-    Examples
-    --------
-    >>> array_arange([1, 5], [3, 6])
-    array([1, 2, 5], dtype=int64)
-    >>> array_arange([1, 6], [4, 9])
-    array([1, 2, 3, 6, 7, 8], dtype=int64)
-    >>> array_arange([1, 6], n=[2, 2])
-    array([1, 2, 6, 7], dtype=int64)
-    """
-    # Tests show that the below code is faster than
-    # implicit for-loops, or list-comprehensions
-    # concatenate(map(..)
-    # The below is much faster and does not require _any_ loops
-    if n is None:
-        # We need n to speed things up
-        n = asarray(end) - asarray(start)
-    else:
-        n = asarray(n)
-    # The below algorithm only works for non-zero n
-    idx = n.nonzero()[0]
-
-    # Grab corner case
-    if len(idx) == 0:
-        return zeros(0, dtype=dtype)
-
-    # Reduce size
-    start = take(start, idx)
-    n = take(n, idx)
-
-    # Create array of 1's.
-    # The 1's are important when issuing the cumultative sum
-    a = ones(n.sum(), dtype=dtype)
-
-    # set pointers such that we can
-    # correct for final cumsum
-    ptr = cumsum(n[:-1])
-    a[0] = start[0]
-    # Define start and correct for previous values
-    a[ptr] = start[1:] - start[:-1] - n[:-1] + 1
-
-    return cumsum(a, dtype=dtype)
