@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import pytest
 
 import argparse
@@ -49,8 +52,8 @@ def test_decorators1():
     p = argparse.ArgumentParser(description='SECOND DEFAULT')
     p = myArgParser(None, p)
     assert "SPBS" not in p.format_help()
-    assert "MY DEFAULT STUFF" not in p.format_help()
-    assert "SECOND DEFAULT" in p.format_help()
+    assert "MY DEFAULT STUFF" in p.format_help()
+    assert "SECOND DEFAULT" not in p.format_help()
 
 
 def test_decorators2():
