@@ -1,3 +1,6 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at https://mozilla.org/MPL/2.0/.
 r""" Atom input/output writer
 
 Developer: Nick Papior
@@ -22,10 +25,12 @@ made available through the `stoolbox`.
 import sys
 from functools import reduce
 from pathlib import Path
+from collections import Iterable
 
 import numpy as np
+from scipy.interpolate import interp1d
 import sisl as si
-from sisl.utils import PropertyDict
+from sisl.utils import PropertyDict, NotNonePropertyDict
 
 
 __all__ = ["AtomInput", "atom_plot_cli"]
@@ -68,6 +73,7 @@ _shell_order = [
     '4f', '5d', '6s', '6p', # [Rn]
     '7s', '5f', '6d', '7p' # [Og]
 ]
+_spdfg = 'spdfg'
 
 
 class AtomInput:
@@ -150,7 +156,7 @@ class AtomInput:
         # default to true if set
         self.opts.setdefault("cc", "rcore" in self.opts)
         # rcore only used if cc is True
-        self.opts.setdefault("rcore", 2.)
+        self.opts.setdefault("rcore", 0.)
         self.opts.setdefault("xc", "pb")
 
         # Read in the core valence shells for this atom
@@ -161,12 +167,10 @@ class AtomInput:
         # e.g if the minimum valence shell is 2p, it would mean that
         #  _shell_order.index("2p") == 2
         # which has 1s and 2s occupied.
-        spdf = 'spdf'
         try:
-            core = reduce(min, (_shell_order.index(f"{orb.n}{spdf[orb.l]}")
-                                for orb in atom),
-                          len(_shell_order))
-        except:
+            core = reduce(min, (_shell_order.index(f"{orb.n}{_spdfg[orb.l]}")
+                                for orb in atom), len(_shell_order))
+        except Exception:
             core = -1
 
         self.opts.setdefault("core", core)
@@ -175,12 +179,11 @@ class AtomInput:
 
         # Store the defined names
         if define is None:
-            self.define = []
+            define = []
         elif isinstance(define, str):
-            self.define = [define]
-        else:
-            # must be list-like
-            self.define = define
+            define = [define]
+        # store
+        self.define = define
 
     @classmethod
     def from_input(cls, inp):
@@ -292,13 +295,61 @@ class AtomInput:
         atom = si.Atom(symbol, orbs)
         return cls(atom, defines, **opts)
 
-    def _write_header(self, f):
+    @classmethod
+    def from_yaml(cls, file, nodes=()):
+        """ Parse the yaml file """
+        from sisl_toolbox.siesta.minimizer._yaml_reader import read_yaml, parse_variable
+        dic = read_yaml(file, nodes)
+
+        element = dic["element"]
+        tag = dic.get("tag")
+        mass = dic.get("mass", None)
+
+        # get default options for pseudo
+        opts = NotNonePropertyDict()
+        pseudo = dic["pseudo"]
+        opts["logr"] = parse_variable(pseudo.get("log-radii"), unit="Ang").value
+        opts["rcore"] = parse_variable(pseudo.get("core-correction"), unit="Ang").value
+        opts["xc"] = pseudo.get("xc")
+        opts["equation"] = pseudo.get("equation")
+        opts["flavor"] = pseudo.get("flavor")
+        define = pseudo.get("define", ('NEW_CC', 'FREE_FORMAT_RC_INPUT', 'NO_PS_CUTOFFS'))
+
+        # Now on to parsing the valence shells
+        orbs = []
+        for key in dic:
+            if key not in _shell_order:
+                continue
+
+            # Now we know the occupation is a shell
+            pseudo = dic[key].get("pseudo", {})
+            cutoff = parse_variable(pseudo.get("cutoff"), 2.1, "Ang").value
+            charge = parse_variable(pseudo.get("charge"), 0.).value
+            orbs.append(si.AtomicOrbital(key, m=0, R=cutoff, q0=charge))
+
+        atom = si.Atom(element, orbs, mass=mass, tag=tag)
+        return cls(atom, define, **opts)
+
+    def write_header(self, f):
         f.write("# This file is generated by sisl pseudo\n")
         # Define all names
         for define in self.define:
             f.write(f"%define {define.upper()}\n")
 
-    def _write_middle(self, f):
+    def write_generation(self, f):
+        if self.opts.cc:
+            # use core corrections
+            pg = "pe"
+        else:
+            # do not use core corrections
+            pg = "pg"
+        logr = self.opts.logr * _Ang2Bohr
+        f.write(f"   {pg:2s} {self.atom.symbol} pseudo potential\n")
+        if logr < 0.:
+            f.write(f"        {self.opts.flavor:3s}\n")
+        else:
+            f.write(f"        {self.opts.flavor:3s}{logr:9.3f}\n")
+
         xc = self.opts.xc
         equation = self.opts.equation
         rcore = self.opts.rcore * _Ang2Bohr
@@ -313,50 +364,146 @@ class AtomInput:
 
         f.write(f"{core:5d}{valence:5d}\n")
 
-        orbs = sorted(atom.orbitals, key=lambda x: x.l)
         Rs = [0.] * 4 # always 4: s, p, d, f
-        for orb in orbs:
+        for orb in sorted(atom.orbitals, key=lambda x: x.l):
             # Write the configuration of this orbital
-            n = orb.n
-            l = orb.l
-            # for now this is a single integer
-            q0 = orb.q0
-            f.write(f"{n:5d}{l:5d}{q0:10.3f}{0.0:10.3f}\n")
+            n, l = orb.n, orb.l
+            f.write(f"{n:5d}{l:5d}{orb.q0:10.3f}{0.0:10.3f}\n")
             Rs[l] = orb.R * _Ang2Bohr
         f.write(f"{Rs[0]:10.7f} {Rs[1]:10.7f} {Rs[2]:10.7f} {Rs[3]:10.7f} {0.0:10.7f} {rcore:10.7f}\n")
 
+    def write_all_electron(self, f, charges=(0.,)):
+        q0 = self.atom.q0.sum()
+        xc = self.opts.xc
+        equation = self.opts.equation
+        core = self.opts.core
+        for charge in charges:
+            if isinstance(charge, dict):
+                out = self.excite(**charge)
+            else:
+                out = self.excite(charge)
+            q = out.atom.q0.sum()
+            f.write(f"   ae {out.atom.symbol} # all-electron q={q0 - q}\n")
+            f.write(f"   {out.atom.symbol:2s}   {xc:2s}{equation:1s}")
+            if "libxc" in out.opts:
+                f.write(f" {out.opts.libxc:8d}")
+            f.write(f"\n  {0.0:5.1f}\n")
+            # now extract the charges for each orbital
+            atom = out.atom
+            valence = len(atom)
+
+            f.write(f"{core:5d}{valence:5d}\n")
+            for orb in sorted(atom.orbitals, key=lambda x: x.l):
+                n, l = orb.n, orb.l
+                f.write(f"{n:5d}{l:5d}{orb.q0:10.3f}{0.0:10.3f}\n")
+
+    def write_test(self, f, charges=(0,)):
+        q0 = self.atom.q0.sum()
+        xc = self.opts.xc
+        equation = self.opts.equation
+        core = self.opts.core
+        for charge in charges:
+            if isinstance(charge, dict):
+                out = self.excite(**charge)
+            else:
+                out = self.excite(charge)
+            q = out.atom.q0.sum()
+            f.write(f"   pt {out.atom.symbol} # pseudo test q={q0 - q}\n")
+            f.write(f"   {out.atom.symbol:2s}   {xc:2s}{equation:1s}")
+            if "libxc" in out.opts:
+                f.write(f" {out.opts.libxc:8d}")
+            f.write(f"\n  {0.0:5.1f}\n")
+            # now extract the charges for each orbital
+            atom = out.atom
+            valence = len(atom)
+
+            f.write(f"{core:5d}{valence:5d}\n")
+            for orb in sorted(atom.orbitals, key=lambda x: x.l):
+                n, l = orb.n, orb.l
+                f.write(f"{n:5d}{l:5d}{orb.q0:10.3f}{0.0:10.3f}\n")
+
     def _get_out(self, path, filename):
         if path is None:
             return Path(filename)
         return Path(path) / Path(filename)
 
-    def ae(self, filename="INP", path=None):
+    def __call__(self, filename="INP", path=None):
+        """ Open a file and return self """
         out = self._get_out(path, filename)
-        with open(out, 'w') as f:
-            self._write_header(f)
-            # Now prepare data
-            f.write(f"   ae {self.atom.symbol} ground state calculation\n")
-            self._write_middle(f)
+        self._enter = open(out, 'w'), self.atom
+        return self
+
+    def __enter__(self):
+        if not hasattr(self, "_enter"):
+            self()
+        return self._enter[0]
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        if hasattr(self, "_enter"):
+            self.atom = self._enter[1]
+            del self._enter
+
+    def ae(self, filename="INP", path=None):
+        with self(filename, path) as f:
+            self.write_header(f)
+            self.write_all_electron(f)
 
     def pg(self, filename="INP", path=None):
-        # check whether we need core corrections
-        out = self._get_out(path, filename)
-        if self.opts.cc:
-            # use core corrections
-            pg = "pe"
+        with self(filename, path) as f:
+            self.write_header(f)
+            self.write_generation(f)
+
+    def excite(self, *charge, **lq):
+        """ Excite contained atom to another charge
+
+        Notes
+        -----
+        This is charge, _not_ electrons.
+        """
+        if len(charge) > 1:
+            raise ValueError(f"{self.__class__.__name__}.excite takes only "
+                             "a single argument or [spdf]=charge arguments")
+        elif len(charge) == 1:
+            charge = charge[0]
+            if "charge" in lq:
+                raise ValueError(f"{self.__class__.__name__}.excite does not accept "
+                                 "both charge as argument and keyword argument")
         else:
-            # do not use core corrections
-            pg = "pg"
-        logr = self.opts.logr * _Ang2Bohr
-        with open(out, 'w') as f:
-            self._write_header(f)
-            # Now prepare data
-            f.write(f"   {pg:2s} {self.atom.symbol} pseudo potential generation\n")
-            if logr < 0.:
-                f.write(f"        {self.opts.flavor:3s}\n")
-            else:
-                f.write(f"        {self.opts.flavor:3s}{logr:9.3f}\n")
-            self._write_middle(f)
+            charge = 0
+        charge = lq.pop("charge", charge)
+        # get indices of the orders
+        shell_idx = [_shell_order.index(f"{orb.n}{_spdfg[orb.l]}")
+                     for orb in self.atom]
+        def _charge(idx):
+            return {'s': 2, 'p': 6, 'd': 10, 'f': 14}[_shell_order[idx][1]]
+        orig_charge = [_charge(idx) for idx in shell_idx]
+        atom = self.atom.copy()
+        # find order of orbitals (from highest index to lowest)
+        # depending on the sign of charge
+        shell_sort = shell_idx[:]
+        shell_sort.sort(reverse=charge > 0)
+
+        for l, q in lq.items():
+            l = _spdfg.index(l)
+            for orb in atom:
+                if orb.l == l:
+                    orb._q0 -= q
+                    assert orb.q0 >= 0.
+
+        # now finalize the charge
+        while abs(charge) > 1e-9:
+            for idx_sort in shell_sort:
+                idx = shell_idx.index(idx_sort)
+                orb = atom[idx]
+                if charge > 0 and orb.q0 > 0:
+                    dq = min(orb.q0, charge)
+                elif charge < 0 and orb.q0 < orig_charge[idx]:
+                    dq = max(orb.q0 - orig_charge[idx], charge)
+                else:
+                    dq = 0.
+                orb._q0 -= dq
+                charge -= dq
+        return self.__class__(atom, self.define, **self.opts)
 
     def plot(self, path=None,
              plot=('wavefunction', 'charge', 'log', 'potential'),
@@ -403,7 +550,6 @@ class AtomInput:
             y = (data * yfactors.reshape(1, -1)).sum(1)
             return x, y
 
-        spdfg = 'spdfg'
         l2i = {
             's': 0, 0: 0,
             'p': 1, 1: 1,
@@ -429,13 +575,13 @@ class AtomInput:
 
                 r, w = get_xy(f"AEWFNR{il}")
                 if not r is None:
-                    p = ax.plot(r, w, label=f"AE {spdfg[il]}")
+                    p = ax.plot(r, w, label=f"AE {_spdfg[il]}")
                     color = p[0].get_color()
                     ax.axvline(orb.R * _Ang2Bohr, color=color, alpha=0.5)
 
                 r, w = get_xy(f"PSWFNR{il}")
                 if not r is None:
-                    ax.plot(r, w, '--', label=f"PS {spdfg[il]}")
+                    ax.plot(r, w, '--', label=f"PS {_spdfg[il]}")
 
             ax.set_xlim(0, atom_r * 5)
             ax.autoscale(enable=True, axis='y', tight=True)
@@ -468,7 +614,7 @@ class AtomInput:
             # and the pseudopotential valence charge
             if np.allclose(ae_r, ps_r):
                 # Determine dR
-                dr = ae_r[1] - ae_r[0]
+                #dr = ae_r[1] - ae_r[0]
 
                 # Integrate number of core-electrons and valence electrons
                 core_c = np.trapz(ae_cc, ae_r)
@@ -484,21 +630,20 @@ class AtomInput:
                 # Tests show that it might be located where the core charge density is from 1 to 2 times
                 # larger than the valence charge density
                 with np.errstate(divide='ignore', invalid='ignore'):
-                    core_over_valence = ae_cc / ps_vc
+                    fraction = ae_cc / ps_vc
+                    np.nan_to_num(fraction, copy=False)
 
                 ax2 = ax.twinx() # instantiate a second axes that shares the same x-axis
-                ax2.plot(ae_r, core_over_valence, 'k', alpha=0.5)
+                ax2.plot(ae_r, fraction, 'k', alpha=0.5, label='c/v')
 
-                # Now mark 1, 1.5 and 2 times
-                factor_marks = [2., 1.5, 1]
-                r_marks = []
-                for mark in factor_marks:
-                    # last value closest to function
-                    idx = (core_over_valence > mark).nonzero()[0][-1]
-                    r_marks.append(ae_r[idx])
-                ax2.scatter(r_marks, factor_marks, alpha=0.5)
+                marks = np.array([0.5, 1., 1.5])
+                min_x = (ae_r > 0.2).nonzero()[0].min()
+                max_x = (fraction[min_x:] > 0.).nonzero()[0].max() + min_x
+
+                r_marks = interp1d(fraction[min_x:max_x], ae_r[min_x:max_x])(marks)
+                ax2.scatter(r_marks, marks, alpha=0.5)
                 ax2.set_ylim(0, 3)
-                print(f"Core-correction r_pc {factor_marks}: {r_marks} Bohr")
+                print(f"Core-correction r_pc {marks}: {r_marks} Bohr")
 
             ax.set_xlim(0, atom_r)
             ax.set_ylim(0)
@@ -518,7 +663,7 @@ class AtomInput:
                     emark.shape = (1, -1)
                 emark = emark[:, 0]
                 if not e is None:
-                    p = ax.plot(e, log, label=f"AE {spdfg[il]}")
+                    p = ax.plot(e, log, label=f"AE {_spdfg[il]}")
 
                     idx_mark = (np.fabs(e.reshape(-1, 1) - emark.reshape(1, -1))
                                 .argmin(axis=0))
@@ -531,7 +676,7 @@ class AtomInput:
                     emark.shape = (1, -1)
                 emark = emark[:, 0]
                 if not e is None:
-                    p = ax.plot(e, log, label=f"PS {spdfg[il]}")
+                    p = ax.plot(e, log, ':', label=f"PS {_spdfg[il]}")
 
                     idx_mark = (np.fabs(e.reshape(-1, 1) - emark.reshape(1, -1))
                                 .argmin(axis=0))
@@ -550,7 +695,7 @@ class AtomInput:
 
                 r, V = get_xy(f"PSPOTR{il}")
                 if not r is None:
-                    p = ax.plot(r, V, label=f"PS {spdfg[il]}")
+                    p = ax.plot(r, V, label=f"PS {_spdfg[il]}")
                     color = p[0].get_color()
                     ax.axvline(orb.R * _Ang2Bohr, color=color, alpha=0.5)
 
@@ -593,13 +738,14 @@ def atom_plot_cli(subp=None):
 
     is_sub = not subp is None
 
-    title = "Plotting facility for atom output"
+    title = "Plotting facility for atom output (run in the atom output directory)"
     if is_sub:
         global _script
         _script = f"{_script} atom-plot"
         p = subp.add_parser("atom-plot", description=title, help=title)
     else:
-        p = argp.ArgumentParser(title)
+        import argparse
+        p = argparse.ArgumentParser(title)
 
     p.add_argument("--plot", '-P', action='append', type=str,
                    choices=('wavefunction', 'charge', 'log', 'potential'),
